@@ -4,11 +4,11 @@
 # The first step is to define the variational problem at hand. We define
 # the variational problem in UFL terms in a separate form file
 # :download:`poisson.py`.  We begin by defining the finite element::
-from basix.ufl_wrapper import create_vector_element
-from ufl import (Coefficient, Constant, FiniteElement, FunctionSpace, Mesh,
-                 TestFunction, TrialFunction, ds, dx, grad, inner, triangle)
+from basix.ufl import element
+from ufl import (Coefficient, Constant, FunctionSpace, Mesh, TestFunction,
+                 TrialFunction, ds, dx, grad, inner)
 
-element = FiniteElement("Lagrange", triangle, 1)
+e = element("Lagrange", "triangle", 1)
 
 # The first argument to :py:class:`FiniteElement` is the finite element
 # family, the second argument specifies the domain, while the third
@@ -21,10 +21,10 @@ element = FiniteElement("Lagrange", triangle, 1)
 # (:math:`u` and :math:`v`) and the coefficient functions (:math:`f` and
 # :math:`g`)::
 
-coord_element = create_vector_element("Lagrange", "triangle", 1)
+coord_element = element("Lagrange", "triangle", 1, shape=(2,))
 mesh = Mesh(coord_element)
 
-V = FunctionSpace(mesh, element)
+V = FunctionSpace(mesh, e)
 
 u = TrialFunction(V)
 v = TestFunction(V)
