@@ -135,7 +135,7 @@ public:
   /// Number of ghost indices on this process
   std::int32_t num_ghosts() const noexcept;
 
-  /// Number of indices owned by on this process
+  /// Number of indices owned by this process
   std::int32_t size_local() const noexcept;
 
   /// Number indices across communicator
@@ -228,6 +228,22 @@ public:
   /// false.
   bool overlapped() const noexcept;
 
+  /// @brief Returns the imbalance of the current IndexMap.
+  ///
+  /// The imbalance is a measure of load balancing across all processes, defined
+  /// as the maximum number of indices on any process divided by the average
+  /// number of indices per process. This function calculates the imbalance
+  /// separately for owned indices and ghost indices and returns them as a
+  /// std::array<double, 2>. If the total number of owned or ghost indices is
+  /// zero, the respective entry in the array is set to -1.
+  ///
+  /// @note This is a collective operation and must be called by all processes
+  /// in the communicator associated with the IndexMap.
+  ///
+  /// @return An array containing the imbalance in owned indices
+  /// (first element) and the imbalance in ghost indices (second element).
+  std::array<double, 2> imbalance() const;
+
 private:
   // Range of indices (global) owned by this process
   std::array<std::int64_t, 2> _local_range;
