@@ -119,11 +119,14 @@ Table Table::reduce(MPI_Comm comm, Table::Reduction reduction) const
   // Get string data size on each process
   std::vector<int> pcounts(mpi_size), offsets(mpi_size + 1, 0);
   const int local_size_str = keys.size();
-  MPI_Gather(&local_size_str, 1, MPI_INT, pcounts.data(), 1, MPI_INT, 0, comm);
+  int err = MPI_Gather(&local_size_str, 1, MPI_INT, pcounts.data(), 1, MPI_INT,
+                       0, comm);
+  dolfinx::MPI::check_error(comm, err);
   std::partial_sum(pcounts.begin(), pcounts.end(), offsets.begin() + 1);
   std::vector<char> out_str(offsets.back());
-  MPI_Gatherv(keys.data(), keys.size(), MPI_CHAR, out_str.data(),
-              pcounts.data(), offsets.data(), MPI_CHAR, 0, comm);
+  err = MPI_Gatherv(keys.data(), keys.size(), MPI_CHAR, out_str.data(),
+                    pcounts.data(), offsets.data(), MPI_CHAR, 0, comm);
+  dolfinx::MPI::check_error(comm, err);
 
   // Rebuild string
   std::vector<std::string> keys_all(mpi_size);
@@ -135,11 +138,15 @@ Table Table::reduce(MPI_Comm comm, Table::Reduction reduction) const
 
   // Get value data size on each process
   const int local_size = values.size();
-  MPI_Gather(&local_size, 1, MPI_INT, pcounts.data(), 1, MPI_INT, 0, comm);
+  err = MPI_Gather(&local_size, 1, MPI_INT, pcounts.data(), 1, MPI_INT, 0,
+                   comm);
+  dolfinx::MPI::check_error(comm, err);
   std::partial_sum(pcounts.begin(), pcounts.end(), offsets.begin() + 1);
+
   std::vector<double> values_all(offsets.back());
-  MPI_Gatherv(values.data(), values.size(), MPI_DOUBLE, values_all.data(),
-              pcounts.data(), offsets.data(), MPI_DOUBLE, 0, comm);
+  err = MPI_Gatherv(values.data(), values.size(), MPI_DOUBLE, values_all.data(),
+                    pcounts.data(), offsets.data(), MPI_DOUBLE, 0, comm);
+  dolfinx::MPI::check_error(comm, err);
 
   // Return empty table on rank > 0
   if (MPI::rank(comm) > 0)
