@@ -15,7 +15,7 @@ namespace dolfinx
 std::string version();
 
 /// Return UFC signature string
-std::string ufc_signature();
+std::string ufcx_signature();
 
 /// Return git changeset hash (returns "unknown" if changeset is
 /// not known)
@@ -25,10 +25,6 @@ std::string git_commit_hash();
 /// i.e., with assertions on
 bool has_debug();
 
-/// Return true if DOLFINx is configured with PETSc compiled
-/// with scalars represented as complex numbers
-bool has_petsc_complex();
-
 /// Return true if DOLFINx is compiled with SLEPc
 bool has_slepc();
 
@@ -41,4 +37,7 @@ bool has_parmetis();
 /// Return true if DOLFINx is compiled with KaHIP
 bool has_kahip();
 
+/// Return true if DOLFINX is compiled with ADIOS2
+bool has_adios2();
+
 } // namespace dolfinx
