@@ -22,9 +22,7 @@ void dolfinx::init_logging(int argc, char* argv[])
   loguru::Options options = {"-dolfinx_loglevel", "main", signals};
 
   // Make a copy of argv, as loguru may modify it
-  std::vector<char*> argv_copy;
-  for (int i = 0; i < argc; ++i)
-    argv_copy.push_back(argv[i]);
+  std::vector<char*> argv_copy(argv, argv + argc);
   argv_copy.push_back(nullptr);
 
   loguru::init(argc, argv_copy.data(), options);
