@@ -8,10 +8,12 @@
 #pragma once
 
 #include "Constant.h"
+#include "DofMap.h"
 #include "Function.h"
 #include "FunctionSpace.h"
 #include <array>
 #include <concepts>
+#include <dolfinx/common/types.h>
 #include <functional>
 #include <memory>
 #include <span>
@@ -28,8 +30,8 @@ namespace dolfinx::fem
 /// elements are associated with the cell even if they may appear to be
 /// associated with a facet/edge/vertex.
 ///
-/// @param[in] V The function (sub)space on which degrees-of-freedom
-/// (DOFs) will be located.
+/// @param[in] topology Mesh topology.
+/// @param[in] dofmap Dofmap that associated DOFs with cells.
 /// @param[in] dim Topological dimension of mesh entities on which
 /// degrees-of-freedom will be located
 /// @param[in] entities Indices of mesh entities. All DOFs associated
@@ -45,7 +47,7 @@ namespace dolfinx::fem
 /// space V. The array uses the block size of the dofmap associated
 /// with V.
 std::vector<std::int32_t>
-locate_dofs_topological(const FunctionSpace& V, int dim,
+locate_dofs_topological(mesh::Topology& topology, const DofMap& dofmap, int dim,
                         std::span<const std::int32_t> entities,
                         bool remote = true);
 
@@ -54,9 +56,8 @@ locate_dofs_topological(const FunctionSpace& V, int dim,
 /// elements are associated with the cell even if they may appear to be
 /// associated with a facet/edge/vertex.
 ///
-/// @param[in] V The function (sub)spaces on which degrees-of-freedom
-/// (DOFs) will be located. The spaces must share the same mesh and
-/// element type.
+/// @param[in] topology Mesh topology.
+/// @param[in] dofmaps The dofmaps.
 /// @param[in] dim Topological dimension of mesh entities on which
 /// degrees-of-freedom will be located
 /// @param[in] entities Indices of mesh entities. All DOFs associated
@@ -73,13 +74,14 @@ locate_dofs_topological(const FunctionSpace& V, int dim,
 /// V[0] and array[1](i) is the corresponding DOF entry in the space
 /// V[1]. The returned dofs are 'unrolled', i.e. block size = 1.
 std::array<std::vector<std::int32_t>, 2> locate_dofs_topological(
-    const std::array<std::reference_wrapper<const FunctionSpace>, 2>& V,
-    int dim, std::span<const std::int32_t> entities, bool remote = true);
+    mesh::Topology& topology,
+    std::array<std::reference_wrapper<const DofMap>, 2> dofmaps, int dim,
+    std::span<const std::int32_t> entities, bool remote = true);
 
-/// Finds degrees of freedom whose geometric coordinate is true for the
-/// provided marking function.
+/// @brief Find degrees of freedom whose geometric coordinate is true
+/// for the provided marking function.
 ///
-/// @attention This function is slower than the topological version
+/// @attention This function is slower than the topological version.
 ///
 /// @param[in] V The function (sub)space on which degrees of freedom
 /// will be located.
@@ -87,14 +89,43 @@ std::array<std::vector<std::int32_t>, 2> locate_dofs_topological(
 /// @return Array of DOF index blocks (local to the MPI rank) in the
 /// space V. The array uses the block size of the dofmap associated
 /// with V.
-std::vector<std::int32_t> locate_dofs_geometrical(
-    const FunctionSpace& V,
-    const std::function<std::vector<std::int8_t>(
-        std::experimental::mdspan<
-            const double,
-            std::experimental::extents<std::size_t, 3,
-                                       std::experimental::dynamic_extent>>)>&
-        marker_fn);
+template <std::floating_point T, typename U>
+std::vector<std::int32_t> locate_dofs_geometrical(const FunctionSpace<T>& V,
+                                                  U marker_fn)
+{
+  // FIXME: Calling V.tabulate_dof_coordinates() is very expensive,
+  // especially when we usually want the boundary dofs only. Add
+  // interface that computes dofs coordinates only for specified cell.
+
+  assert(V.element());
+  if (V.element()->is_mixed())
+  {
+    throw std::runtime_error(
+        "Cannot locate dofs geometrically for mixed space. Use subspaces.");
+  }
+
+  // Compute dof coordinates
+  const std::vector<T> dof_coordinates = V.tabulate_dof_coordinates(true);
+
+  using cmdspan3x_t = MDSPAN_IMPL_STANDARD_NAMESPACE::mdspan<
+      const T,
+      MDSPAN_IMPL_STANDARD_NAMESPACE::extents<
+          std::size_t, 3, MDSPAN_IMPL_STANDARD_NAMESPACE::dynamic_extent>>;
+
+  // Compute marker for each dof coordinate
+  cmdspan3x_t x(dof_coordinates.data(), 3, dof_coordinates.size() / 3);
+  const std::vector<std::int8_t> marked_dofs = marker_fn(x);
+
+  std::vector<std::int32_t> dofs;
+  dofs.reserve(std::count(marked_dofs.begin(), marked_dofs.end(), true));
+  for (std::size_t i = 0; i < marked_dofs.size(); ++i)
+  {
+    if (marked_dofs[i])
+      dofs.push_back(i);
+  }
+
+  return dofs;
+}
 
 /// Finds degrees of freedom whose geometric coordinate is true for the
 /// provided marking function.
@@ -109,14 +140,100 @@ std::vector<std::int32_t> locate_dofs_geometrical(
 /// V[0] and V[1]. The array[0](i) entry is the DOF index in the space
 /// V[0] and array[1](i) is the corresponding DOF entry in the space
 /// V[1]. The returned dofs are 'unrolled', i.e. block size = 1.
+template <std::floating_point T, typename U>
 std::array<std::vector<std::int32_t>, 2> locate_dofs_geometrical(
-    const std::array<std::reference_wrapper<const FunctionSpace>, 2>& V,
-    const std::function<std::vector<std::int8_t>(
-        std::experimental::mdspan<
-            const double,
-            std::experimental::extents<std::size_t, 3,
-                                       std::experimental::dynamic_extent>>)>&
-        marker_fn);
+    const std::array<std::reference_wrapper<const FunctionSpace<T>>, 2>& V,
+    U marker_fn)
+{
+  // FIXME: Calling V.tabulate_dof_coordinates() is very expensive,
+  // especially when we usually want the boundary dofs only. Add
+  // interface that computes dofs coordinates only for specified cell.
+
+  // Get function spaces
+  const FunctionSpace<T>& V0 = V.at(0).get();
+  const FunctionSpace<T>& V1 = V.at(1).get();
+
+  // Get mesh
+  auto mesh = V0.mesh();
+  assert(mesh);
+  assert(V1.mesh());
+  if (mesh != V1.mesh())
+    throw std::runtime_error("Meshes are not the same.");
+  const int tdim = mesh->topology()->dim();
+
+  assert(V0.element());
+  assert(V1.element());
+  if (*V0.element() != *V1.element())
+    throw std::runtime_error("Function spaces must have the same element.");
+
+  // Compute dof coordinates
+  const std::vector<T> dof_coordinates = V1.tabulate_dof_coordinates(true);
+
+  using cmdspan3x_t = MDSPAN_IMPL_STANDARD_NAMESPACE::mdspan<
+      const T,
+      MDSPAN_IMPL_STANDARD_NAMESPACE::extents<
+          std::size_t, 3, MDSPAN_IMPL_STANDARD_NAMESPACE::dynamic_extent>>;
+
+  // Evaluate marker for each dof coordinate
+  cmdspan3x_t x(dof_coordinates.data(), 3, dof_coordinates.size() / 3);
+  const std::vector<std::int8_t> marked_dofs = marker_fn(x);
+
+  // Get dofmaps
+  std::shared_ptr<const DofMap> dofmap0 = V0.dofmap();
+  assert(dofmap0);
+  const int bs0 = dofmap0->bs();
+  std::shared_ptr<const DofMap> dofmap1 = V1.dofmap();
+  assert(dofmap1);
+  const int bs1 = dofmap1->bs();
+
+  const int element_bs = dofmap0->element_dof_layout().block_size();
+  assert(element_bs == dofmap1->element_dof_layout().block_size());
+
+  // Iterate over cells
+  auto topology = mesh->topology();
+  assert(topology);
+  std::vector<std::array<std::int32_t, 2>> bc_dofs;
+  for (int c = 0; c < topology->connectivity(tdim, 0)->num_nodes(); ++c)
+  {
+    // Get cell dofmaps
+    auto cell_dofs0 = dofmap0->cell_dofs(c);
+    auto cell_dofs1 = dofmap1->cell_dofs(c);
+
+    // Loop over cell dofs and add to bc_dofs if marked.
+    for (std::size_t i = 0; i < cell_dofs1.size(); ++i)
+    {
+      if (marked_dofs[cell_dofs1[i]])
+      {
+        // Unroll over blocks
+        for (int k = 0; k < element_bs; ++k)
+        {
+          const int local_pos = element_bs * i + k;
+          const std::div_t pos0 = std::div(local_pos, bs0);
+          const std::div_t pos1 = std::div(local_pos, bs1);
+          const std::int32_t dof_index0
+              = bs0 * cell_dofs0[pos0.quot] + pos0.rem;
+          const std::int32_t dof_index1
+              = bs1 * cell_dofs1[pos1.quot] + pos1.rem;
+          bc_dofs.push_back({dof_index0, dof_index1});
+        }
+      }
+    }
+  }
+
+  // Remove duplicates
+  std::sort(bc_dofs.begin(), bc_dofs.end());
+  bc_dofs.erase(std::unique(bc_dofs.begin(), bc_dofs.end()), bc_dofs.end());
+
+  // Copy to separate array
+  std::array dofs = {std::vector<std::int32_t>(bc_dofs.size()),
+                     std::vector<std::int32_t>(bc_dofs.size())};
+  std::transform(bc_dofs.cbegin(), bc_dofs.cend(), dofs[0].begin(),
+                 [](auto dof) { return dof[0]; });
+  std::transform(bc_dofs.cbegin(), bc_dofs.cend(), dofs[1].begin(),
+                 [](auto dof) { return dof[1]; });
+
+  return dofs;
+}
 
 /// Object for setting (strong) Dirichlet boundary conditions
 ///
@@ -128,25 +245,26 @@ std::array<std::vector<std::int32_t>, 2> locate_dofs_geometrical(
 /// A DirichletBC is specified by the function \f$g\f$, the function
 /// space (trial space) and degrees of freedom to which the boundary
 /// condition applies.
-template <typename T>
+template <dolfinx::scalar T,
+          std::floating_point U = dolfinx::scalar_value_type_t<T>>
 class DirichletBC
 {
 private:
   /// Compute number of owned dofs indices. Will contain 'gaps' for
   /// sub-spaces.
-  std::size_t num_owned(const FunctionSpace& V,
+  std::size_t num_owned(const DofMap& dofmap,
                         std::span<const std::int32_t> dofs)
   {
-    int bs = V.dofmap()->index_map_bs();
-    std::int32_t map_size = V.dofmap()->index_map->size_local();
+    int bs = dofmap.index_map_bs();
+    std::int32_t map_size = dofmap.index_map->size_local();
     std::int32_t owned_size = bs * map_size;
     auto it = std::lower_bound(dofs.begin(), dofs.end(), owned_size);
     return std::distance(dofs.begin(), it);
   }
 
   /// Unroll dofs for block size.
-  std::vector<std::int32_t> unroll_dofs(std::span<const std::int32_t> dofs,
-                                        int bs)
+  static std::vector<std::int32_t>
+  unroll_dofs(std::span<const std::int32_t> dofs, int bs)
   {
     std::vector<std::int32_t> dofs_unrolled(bs * dofs.size());
     for (std::size_t i = 0; i < dofs.size(); ++i)
@@ -173,11 +291,13 @@ public:
   /// @note The size of of `g` must be equal to the block size if `V`.
   /// Use the Function version if this is not the case, e.g. for some
   /// mixed spaces.
-  template <typename S, std::convertible_to<std::vector<std::int32_t>> U,
+  template <typename S, typename X,
             typename
             = std::enable_if_t<std::is_convertible_v<S, T>
                                or std::is_convertible_v<S, std::span<const T>>>>
-  DirichletBC(const S& g, U&& dofs, std::shared_ptr<const FunctionSpace> V)
+    requires std::is_convertible_v<std::remove_cvref_t<X>,
+                                   std::vector<std::int32_t>>
+  DirichletBC(const S& g, X&& dofs, std::shared_ptr<const FunctionSpace<U>> V)
       : DirichletBC(std::make_shared<Constant<T>>(g), dofs, V)
   {
   }
@@ -197,11 +317,13 @@ public:
   /// @note The size of of `g` must be equal to the block size if `V`.
   /// Use the Function version if this is not the case, e.g. for some
   /// mixed spaces.
-  template <std::convertible_to<std::vector<std::int32_t>> U>
-  DirichletBC(std::shared_ptr<const Constant<T>> g, U&& dofs,
-              std::shared_ptr<const FunctionSpace> V)
-      : _function_space(V), _g(g), _dofs0(std::forward<U>(dofs)),
-        _owned_indices0(num_owned(*V, _dofs0))
+  template <typename X>
+    requires std::is_convertible_v<std::remove_cvref_t<X>,
+                                   std::vector<std::int32_t>>
+  DirichletBC(std::shared_ptr<const Constant<T>> g, X&& dofs,
+              std::shared_ptr<const FunctionSpace<U>> V)
+      : _function_space(V), _g(g), _dofs0(std::forward<X>(dofs)),
+        _owned_indices0(num_owned(*V->dofmap(), _dofs0))
   {
     assert(g);
     assert(V);
@@ -245,11 +367,13 @@ public:
   /// @note The indices in `dofs` are for *blocks*, e.g. a block index
   /// corresponds to 3 degrees-of-freedom if the dofmap associated with
   /// `g` has block size 3.
-  template <std::convertible_to<std::vector<std::int32_t>> U>
-  DirichletBC(std::shared_ptr<const Function<T>> g, U&& dofs)
+  template <typename X>
+    requires std::is_convertible_v<std::remove_cvref_t<X>,
+                                   std::vector<std::int32_t>>
+  DirichletBC(std::shared_ptr<const Function<T, U>> g, X&& dofs)
       : _function_space(g->function_space()), _g(g),
-        _dofs0(std::forward<U>(dofs)),
-        _owned_indices0(num_owned(*_function_space, _dofs0))
+        _dofs0(std::forward<X>(dofs)),
+        _owned_indices0(num_owned(*_function_space->dofmap(), _dofs0))
   {
     assert(_function_space);
 
@@ -282,13 +406,13 @@ public:
   /// @param[in] V The function (sub)space on which the boundary
   /// condition is applied
   /// @note The indices in `dofs` are unrolled and not for blocks.
-  template <typename U>
-  DirichletBC(std::shared_ptr<const Function<T>> g, U&& V_g_dofs,
-              std::shared_ptr<const FunctionSpace> V)
+  template <typename X>
+  DirichletBC(std::shared_ptr<const Function<T, U>> g, X&& V_g_dofs,
+              std::shared_ptr<const FunctionSpace<U>> V)
       : _function_space(V), _g(g),
-        _dofs0(std::forward<typename U::value_type>(V_g_dofs[0])),
-        _dofs1_g(std::forward<typename U::value_type>(V_g_dofs[1])),
-        _owned_indices0(num_owned(*_function_space, _dofs0))
+        _dofs0(std::forward<typename X::value_type>(V_g_dofs[0])),
+        _dofs1_g(std::forward<typename X::value_type>(V_g_dofs[1])),
+        _owned_indices0(num_owned(*_function_space->dofmap(), _dofs0))
   {
   }
 
@@ -312,14 +436,14 @@ public:
 
   /// The function space to which boundary conditions are applied
   /// @return The function space
-  std::shared_ptr<const FunctionSpace> function_space() const
+  std::shared_ptr<const FunctionSpace<U>> function_space() const
   {
     return _function_space;
   }
 
   /// Return boundary value function g
   /// @return The boundary values Function
-  std::variant<std::shared_ptr<const Function<T>>,
+  std::variant<std::shared_ptr<const Function<T, U>>,
                std::shared_ptr<const Constant<T>>>
   value() const
   {
@@ -348,11 +472,11 @@ public:
   /// of the array @p x should be equal to the number of dofs owned by
   /// this rank.
   /// @param[in] scale The scaling value to apply
-  void set(std::span<T> x, double scale = 1.0) const
+  void set(std::span<T> x, T scale = 1) const
   {
-    if (std::holds_alternative<std::shared_ptr<const Function<T>>>(_g))
+    if (std::holds_alternative<std::shared_ptr<const Function<T, U>>>(_g))
     {
-      auto g = std::get<std::shared_ptr<const Function<T>>>(_g);
+      auto g = std::get<std::shared_ptr<const Function<T, U>>>(_g);
       assert(g);
       std::span<const T> values = g->x()->array();
       auto dofs1_g = _dofs1_g.empty() ? std::span(_dofs0) : std::span(_dofs1_g);
@@ -385,11 +509,11 @@ public:
   /// @param[in] x The array in which to set `scale * (x0 - x_bc)`
   /// @param[in] x0 The array used in compute the value to set
   /// @param[in] scale The scaling value to apply
-  void set(std::span<T> x, std::span<const T> x0, double scale = 1.0) const
+  void set(std::span<T> x, std::span<const T> x0, T scale = 1) const
   {
-    if (std::holds_alternative<std::shared_ptr<const Function<T>>>(_g))
+    if (std::holds_alternative<std::shared_ptr<const Function<T, U>>>(_g))
     {
-      auto g = std::get<std::shared_ptr<const Function<T>>>(_g);
+      auto g = std::get<std::shared_ptr<const Function<T, U>>>(_g);
       assert(g);
       std::span<const T> values = g->x()->array();
       assert(x.size() <= x0.size());
@@ -428,9 +552,9 @@ public:
   /// (the space of the function that provides the dof values)
   void dof_values(std::span<T> values) const
   {
-    if (std::holds_alternative<std::shared_ptr<const Function<T>>>(_g))
+    if (std::holds_alternative<std::shared_ptr<const Function<T, U>>>(_g))
     {
-      auto g = std::get<std::shared_ptr<const Function<T>>>(_g);
+      auto g = std::get<std::shared_ptr<const Function<T, U>>>(_g);
       assert(g);
       std::span<const T> g_values = g->x()->array();
       auto dofs1_g = _dofs1_g.empty() ? std::span(_dofs0) : std::span(_dofs1_g);
@@ -465,10 +589,10 @@ public:
 
 private:
   // The function space (possibly a sub function space)
-  std::shared_ptr<const FunctionSpace> _function_space;
+  std::shared_ptr<const FunctionSpace<U>> _function_space;
 
   // The function
-  std::variant<std::shared_ptr<const Function<T>>,
+  std::variant<std::shared_ptr<const Function<T, U>>,
                std::shared_ptr<const Constant<T>>>
       _g;
 
