@@ -22,8 +22,8 @@ namespace dolfinx::fem::impl
 {
 
 /// Assemble functional over cells
-template <typename T>
-T assemble_cells(const mesh::Geometry& geometry,
+template <dolfinx::scalar T>
+T assemble_cells(mdspan2_t x_dofmap, std::span<const scalar_value_type_t<T>> x,
                  std::span<const std::int32_t> cells, FEkernel<T> auto fn,
                  std::span<const T> constants, std::span<const T> coeffs,
                  int cstride)
@@ -32,13 +32,8 @@ T assemble_cells(const mesh::Geometry& geometry,
   if (cells.empty())
     return value;
 
-  // Prepare cell geometry
-  const graph::AdjacencyList<std::int32_t>& x_dofmap = geometry.dofmap();
-  const std::size_t num_dofs_g = geometry.cmap().dim();
-  std::span<const double> x = geometry.x();
-
   // Create data structures used in assembly
-  std::vector<scalar_value_type_t<T>> coordinate_dofs(3 * num_dofs_g);
+  std::vector<scalar_value_type_t<T>> coordinate_dofs(3 * x_dofmap.extent(1));
 
   // Iterate over all cells
   for (std::size_t index = 0; index < cells.size(); ++index)
@@ -46,7 +41,9 @@ T assemble_cells(const mesh::Geometry& geometry,
     std::int32_t c = cells[index];
 
     // Get cell coordinates/geometry
-    auto x_dofs = x_dofmap.links(c);
+    auto x_dofs
+        = MDSPAN_IMPL_STANDARD_NAMESPACE::MDSPAN_IMPL_PROPOSED_NAMESPACE::
+            submdspan(x_dofmap, c, MDSPAN_IMPL_STANDARD_NAMESPACE::full_extent);
     for (std::size_t i = 0; i < x_dofs.size(); ++i)
     {
       std::copy_n(std::next(x.begin(), 3 * x_dofs[i]), 3,
@@ -62,8 +59,9 @@ T assemble_cells(const mesh::Geometry& geometry,
 }
 
 /// Execute kernel over exterior facets and accumulate result
-template <typename T>
-T assemble_exterior_facets(const mesh::Geometry& geometry,
+template <dolfinx::scalar T>
+T assemble_exterior_facets(mdspan2_t x_dofmap,
+                           std::span<const scalar_value_type_t<T>> x,
                            std::span<const std::int32_t> facets,
                            FEkernel<T> auto fn, std::span<const T> constants,
                            std::span<const T> coeffs, int cstride)
@@ -72,13 +70,8 @@ T assemble_exterior_facets(const mesh::Geometry& geometry,
   if (facets.empty())
     return value;
 
-  // Prepare cell geometry
-  const graph::AdjacencyList<std::int32_t>& x_dofmap = geometry.dofmap();
-  const std::size_t num_dofs_g = geometry.cmap().dim();
-  std::span<const double> x = geometry.x();
-
   // Create data structures used in assembly
-  std::vector<scalar_value_type_t<T>> coordinate_dofs(3 * num_dofs_g);
+  std::vector<scalar_value_type_t<T>> coordinate_dofs(3 * x_dofmap.extent(1));
 
   // Iterate over all facets
   assert(facets.size() % 2 == 0);
@@ -88,7 +81,9 @@ T assemble_exterior_facets(const mesh::Geometry& geometry,
     std::int32_t local_facet = facets[index + 1];
 
     // Get cell coordinates/geometry
-    auto x_dofs = x_dofmap.links(cell);
+    auto x_dofs = MDSPAN_IMPL_STANDARD_NAMESPACE::
+        MDSPAN_IMPL_PROPOSED_NAMESPACE::submdspan(
+            x_dofmap, cell, MDSPAN_IMPL_STANDARD_NAMESPACE::full_extent);
     for (std::size_t i = 0; i < x_dofs.size(); ++i)
     {
       std::copy_n(std::next(x.begin(), 3 * x_dofs[i]), 3,
@@ -104,8 +99,10 @@ T assemble_exterior_facets(const mesh::Geometry& geometry,
 }
 
 /// Assemble functional over interior facets
-template <typename T>
-T assemble_interior_facets(const mesh::Geometry& geometry, int num_cell_facets,
+template <dolfinx::scalar T>
+T assemble_interior_facets(mdspan2_t x_dofmap,
+                           std::span<const scalar_value_type_t<T>> x,
+                           int num_cell_facets,
                            std::span<const std::int32_t> facets,
                            FEkernel<T> auto fn, std::span<const T> constants,
                            std::span<const T> coeffs, int cstride,
@@ -116,16 +113,12 @@ T assemble_interior_facets(const mesh::Geometry& geometry, int num_cell_facets,
   if (facets.empty())
     return value;
 
-  // Prepare cell geometry
-  const graph::AdjacencyList<std::int32_t>& x_dofmap = geometry.dofmap();
-  const std::size_t num_dofs_g = geometry.cmap().dim();
-  std::span<const double> x = geometry.x();
-
   // Create data structures used in assembly
   using X = scalar_value_type_t<T>;
-  std::vector<X> coordinate_dofs(2 * num_dofs_g * 3);
-  std::span<X> cdofs0(coordinate_dofs.data(), num_dofs_g * 3);
-  std::span<X> cdofs1(coordinate_dofs.data() + num_dofs_g * 3, num_dofs_g * 3);
+  std::vector<X> coordinate_dofs(2 * x_dofmap.extent(1) * 3);
+  std::span<X> cdofs0(coordinate_dofs.data(), x_dofmap.extent(1) * 3);
+  std::span<X> cdofs1(coordinate_dofs.data() + x_dofmap.extent(1) * 3,
+                      x_dofmap.extent(1) * 3);
 
   std::vector<T> coeff_array(2 * offsets.back());
   assert(offsets.back() == cstride);
@@ -139,13 +132,17 @@ T assemble_interior_facets(const mesh::Geometry& geometry, int num_cell_facets,
         = {facets[index + 1], facets[index + 3]};
 
     // Get cell geometry
-    auto x_dofs0 = x_dofmap.links(cells[0]);
+    auto x_dofs0 = MDSPAN_IMPL_STANDARD_NAMESPACE::
+        MDSPAN_IMPL_PROPOSED_NAMESPACE::submdspan(
+            x_dofmap, cells[0], MDSPAN_IMPL_STANDARD_NAMESPACE::full_extent);
     for (std::size_t i = 0; i < x_dofs0.size(); ++i)
     {
       std::copy_n(std::next(x.begin(), 3 * x_dofs0[i]), 3,
                   std::next(cdofs0.begin(), 3 * i));
     }
-    auto x_dofs1 = x_dofmap.links(cells[1]);
+    auto x_dofs1 = MDSPAN_IMPL_STANDARD_NAMESPACE::
+        MDSPAN_IMPL_PROPOSED_NAMESPACE::submdspan(
+            x_dofmap, cells[1], MDSPAN_IMPL_STANDARD_NAMESPACE::full_extent);
     for (std::size_t i = 0; i < x_dofs1.size(); ++i)
     {
       std::copy_n(std::next(x.begin(), 3 * x_dofs1[i]), 3,
@@ -161,55 +158,60 @@ T assemble_interior_facets(const mesh::Geometry& geometry, int num_cell_facets,
   return value;
 }
 
-/// Assemble functional into an scalar
-template <typename T>
+/// Assemble functional into an scalar with provided mesh geometry.
+template <dolfinx::scalar T, std::floating_point U>
 T assemble_scalar(
-    const fem::Form<T>& M, std::span<const T> constants,
+    const fem::Form<T, U>& M, mdspan2_t x_dofmap,
+    std::span<const scalar_value_type_t<T>> x, std::span<const T> constants,
     const std::map<std::pair<IntegralType, int>,
                    std::pair<std::span<const T>, int>>& coefficients)
 {
-  std::shared_ptr<const mesh::Mesh> mesh = M.mesh();
+  std::shared_ptr<const mesh::Mesh<U>> mesh = M.mesh();
   assert(mesh);
 
   T value = 0;
   for (int i : M.integral_ids(IntegralType::cell))
   {
-    const auto& fn = M.kernel(IntegralType::cell, i);
-    const auto& [coeffs, cstride] = coefficients.at({IntegralType::cell, i});
-    const std::vector<std::int32_t>& cells = M.cell_domains(i);
-    value += impl::assemble_cells(mesh->geometry(), cells, fn, constants,
-                                  coeffs, cstride);
+    auto fn = M.kernel(IntegralType::cell, i);
+    assert(fn);
+    auto& [coeffs, cstride] = coefficients.at({IntegralType::cell, i});
+    std::span<const std::int32_t> cells = M.domain(IntegralType::cell, i);
+    value += impl::assemble_cells(x_dofmap, x, cells, fn, constants, coeffs,
+                                  cstride);
   }
 
   for (int i : M.integral_ids(IntegralType::exterior_facet))
   {
-    const auto& fn = M.kernel(IntegralType::exterior_facet, i);
-    const auto& [coeffs, cstride]
+    auto fn = M.kernel(IntegralType::exterior_facet, i);
+    assert(fn);
+    auto& [coeffs, cstride]
         = coefficients.at({IntegralType::exterior_facet, i});
-    const std::vector<std::int32_t>& facets = M.exterior_facet_domains(i);
-    value += impl::assemble_exterior_facets(mesh->geometry(), facets, fn,
-                                            constants, coeffs, cstride);
+    value += impl::assemble_exterior_facets(
+        x_dofmap, x, M.domain(IntegralType::exterior_facet, i), fn, constants,
+        coeffs, cstride);
   }
 
   if (M.num_integrals(IntegralType::interior_facet) > 0)
   {
-    mesh->topology_mutable().create_entity_permutations();
-
+    mesh->topology_mutable()->create_entity_permutations();
     const std::vector<std::uint8_t>& perms
-        = mesh->topology().get_facet_permutations();
-
-    int num_cell_facets = mesh::cell_num_entities(mesh->topology().cell_type(),
-                                                  mesh->topology().dim() - 1);
+        = mesh->topology()->get_facet_permutations();
+    auto cell_types = mesh->topology()->cell_types();
+    if (cell_types.size() > 1)
+      throw std::runtime_error("Multiple cell types in the assembler");
+    int num_cell_facets = mesh::cell_num_entities(cell_types.back(),
+                                                  mesh->topology()->dim() - 1);
     const std::vector<int> c_offsets = M.coefficient_offsets();
     for (int i : M.integral_ids(IntegralType::interior_facet))
     {
-      const auto& fn = M.kernel(IntegralType::interior_facet, i);
-      const auto& [coeffs, cstride]
+      auto fn = M.kernel(IntegralType::interior_facet, i);
+      assert(fn);
+      auto& [coeffs, cstride]
           = coefficients.at({IntegralType::interior_facet, i});
-      const std::vector<std::int32_t>& facets = M.interior_facet_domains(i);
-      value += impl::assemble_interior_facets(mesh->geometry(), num_cell_facets,
-                                              facets, fn, constants, coeffs,
-                                              cstride, c_offsets, perms);
+      value += impl::assemble_interior_facets(
+          x_dofmap, x, num_cell_facets,
+          M.domain(IntegralType::interior_facet, i), fn, constants, coeffs,
+          cstride, c_offsets, perms);
     }
   }
 
