@@ -9,24 +9,28 @@
 #include "assemble_matrix_impl.h"
 #include "assemble_scalar_impl.h"
 #include "assemble_vector_impl.h"
+#include "utils.h"
+#include <concepts>
 #include <cstdint>
+#include <dolfinx/common/types.h>
 #include <memory>
 #include <span>
 #include <vector>
 
 namespace dolfinx::fem
 {
-template <typename T>
+template <dolfinx::scalar T, std::floating_point U>
 class DirichletBC;
-template <typename T>
+template <dolfinx::scalar T, std::floating_point U>
 class Form;
+template <std::floating_point T>
 class FunctionSpace;
 
 // -- Helper functions -----------------------------------------------------
 
 /// @brief Create a map of `std::span`s from a map of `std::vector`s
-template <typename T>
-std::map<std::pair<fem::IntegralType, int>, std::pair<std::span<const T>, int>>
+template <dolfinx::scalar T>
+std::map<std::pair<IntegralType, int>, std::pair<std::span<const T>, int>>
 make_coefficients_span(const std::map<std::pair<IntegralType, int>,
                                       std::pair<std::vector<T>, int>>& coeffs)
 {
@@ -52,13 +56,26 @@ make_coefficients_span(const std::map<std::pair<IntegralType, int>,
 /// @param[in] coefficients The coefficients that appear in `M`
 /// @return The contribution to the form (functional) from the local
 /// process
-template <typename T>
+template <dolfinx::scalar T, std::floating_point U>
 T assemble_scalar(
-    const Form<T>& M, std::span<const T> constants,
+    const Form<T, U>& M, std::span<const T> constants,
     const std::map<std::pair<IntegralType, int>,
                    std::pair<std::span<const T>, int>>& coefficients)
 {
-  return impl::assemble_scalar(M, constants, coefficients);
+  std::shared_ptr<const mesh::Mesh<U>> mesh = M.mesh();
+  assert(mesh);
+  if constexpr (std::is_same_v<U, scalar_value_type_t<T>>)
+  {
+    return impl::assemble_scalar(M, mesh->geometry().dofmap(),
+                                 mesh->geometry().x(), constants, coefficients);
+  }
+  else
+  {
+    auto x = mesh->geometry().x();
+    std::vector<scalar_value_type_t<T>> _x(x.begin(), x.end());
+    return impl::assemble_scalar(M, mesh->geometry().dofmap(), _x, constants,
+                                 coefficients);
+  }
 }
 
 /// Assemble functional into scalar
@@ -66,8 +83,8 @@ T assemble_scalar(
 /// @param[in] M The form (functional) to assemble
 /// @return The contribution to the form (functional) from the local
 ///   process
-template <typename T>
-T assemble_scalar(const Form<T>& M)
+template <dolfinx::scalar T, std::floating_point U>
+T assemble_scalar(const Form<T, U>& M)
 {
   const std::vector<T> constants = pack_constants(M);
   auto coefficients = allocate_coefficient_storage(M);
@@ -88,9 +105,9 @@ T assemble_scalar(const Form<T>& M)
 /// @param[in] L The linear forms to assemble into b
 /// @param[in] constants The constants that appear in `L`
 /// @param[in] coefficients The coefficients that appear in `L`
-template <typename T>
+template <dolfinx::scalar T, std::floating_point U>
 void assemble_vector(
-    std::span<T> b, const Form<T>& L, std::span<const T> constants,
+    std::span<T> b, const Form<T, U>& L, std::span<const T> constants,
     const std::map<std::pair<IntegralType, int>,
                    std::pair<std::span<const T>, int>>& coefficients)
 {
@@ -101,8 +118,8 @@ void assemble_vector(
 /// @param[in,out] b The vector to be assembled. It will not be zeroed
 /// before assembly.
 /// @param[in] L The linear forms to assemble into b
-template <typename T>
-void assemble_vector(std::span<T> b, const Form<T>& L)
+template <dolfinx::scalar T, std::floating_point U>
+void assemble_vector(std::span<T> b, const Form<T, U>& L)
 {
   auto coefficients = allocate_coefficient_storage(L);
   pack_coefficients(L, coefficients);
@@ -129,16 +146,41 @@ void assemble_vector(std::span<T> b, const Form<T>& L)
 ///
 /// Ghost contributions are not accumulated (not sent to owner). Caller
 /// is responsible for calling VecGhostUpdateBegin/End.
-template <typename T>
+template <dolfinx::scalar T, std::floating_point U>
 void apply_lifting(
-    std::span<T> b, const std::vector<std::shared_ptr<const Form<T>>>& a,
+    std::span<T> b, const std::vector<std::shared_ptr<const Form<T, U>>>& a,
     const std::vector<std::span<const T>>& constants,
     const std::vector<std::map<std::pair<IntegralType, int>,
                                std::pair<std::span<const T>, int>>>& coeffs,
-    const std::vector<std::vector<std::shared_ptr<const DirichletBC<T>>>>& bcs1,
-    const std::vector<std::span<const T>>& x0, double scale)
+    const std::vector<std::vector<std::shared_ptr<const DirichletBC<T, U>>>>&
+        bcs1,
+    const std::vector<std::span<const T>>& x0, T scale)
 {
-  impl::apply_lifting(b, a, constants, coeffs, bcs1, x0, scale);
+  std::shared_ptr<const mesh::Mesh<U>> mesh;
+  for (auto& a_i : a)
+  {
+    if (a_i and !mesh)
+      mesh = a_i->mesh();
+    if (a_i and mesh and a_i->mesh() != mesh)
+      throw std::runtime_error("Mismatch between meshes.");
+  }
+
+  if (!mesh)
+    throw std::runtime_error("Unable to extract a mesh.");
+
+  if constexpr (std::is_same_v<U, scalar_value_type_t<T>>)
+  {
+    impl::apply_lifting<T>(b, a, mesh->geometry().dofmap(),
+                           mesh->geometry().x(), constants, coeffs, bcs1, x0,
+                           scale);
+  }
+  else
+  {
+    auto x = mesh->geometry().x();
+    std::vector<scalar_value_type_t<T>> _x(x.begin(), x.end());
+    impl::apply_lifting<T>(b, a, mesh->geometry().dofmap(), _x, constants,
+                           coeffs, bcs1, x0, scale);
+  }
 }
 
 /// Modify b such that:
@@ -153,11 +195,12 @@ void apply_lifting(
 ///
 /// Ghost contributions are not accumulated (not sent to owner). Caller
 /// is responsible for calling VecGhostUpdateBegin/End.
-template <typename T>
+template <dolfinx::scalar T, std::floating_point U>
 void apply_lifting(
-    std::span<T> b, const std::vector<std::shared_ptr<const Form<T>>>& a,
-    const std::vector<std::vector<std::shared_ptr<const DirichletBC<T>>>>& bcs1,
-    const std::vector<std::span<const T>>& x0, double scale)
+    std::span<T> b, const std::vector<std::shared_ptr<const Form<T, U>>>& a,
+    const std::vector<std::vector<std::shared_ptr<const DirichletBC<T, U>>>>&
+        bcs1,
+    const std::vector<std::span<const T>>& x0, T scale)
 {
   std::vector<
       std::map<std::pair<IntegralType, int>, std::pair<std::vector<T>, int>>>
@@ -186,11 +229,51 @@ void apply_lifting(
       _coeffs;
   std::transform(coeffs.cbegin(), coeffs.cend(), std::back_inserter(_coeffs),
                  [](auto& c) { return make_coefficients_span(c); });
+
   apply_lifting(b, a, _constants, _coeffs, bcs1, x0, scale);
 }
 
 // -- Matrices ---------------------------------------------------------------
 
+/// @brief Assemble bilinear form into a matrix. Matrix must already be
+/// initialised. Does not zero or finalise the matrix.
+/// @param[in] mat_add The function for adding values into the matrix
+/// @param[in] a The bilinear form to assemble
+/// @param[in] constants Constants that appear in `a`
+/// @param[in] coefficients Coefficients that appear in `a`
+/// @param[in] dof_marker0 Boundary condition markers for the rows. If
+/// bc[i] is true then rows i in A will be zeroed. The index i is a
+/// local index.
+/// @param[in] dof_marker1 Boundary condition markers for the columns.
+/// If bc[i] is true then rows i in A will be zeroed. The index i is a
+/// local index.
+template <dolfinx::scalar T, std::floating_point U>
+void assemble_matrix(
+    la::MatSet<T> auto mat_add, const Form<T, U>& a,
+    std::span<const T> constants,
+    const std::map<std::pair<IntegralType, int>,
+                   std::pair<std::span<const T>, int>>& coefficients,
+    std::span<const std::int8_t> dof_marker0,
+    std::span<const std::int8_t> dof_marker1)
+
+{
+  std::shared_ptr<const mesh::Mesh<U>> mesh = a.mesh();
+  assert(mesh);
+  if constexpr (std::is_same_v<U, scalar_value_type_t<T>>)
+  {
+    impl::assemble_matrix(mat_add, a, mesh->geometry().dofmap(),
+                          mesh->geometry().x(), constants, coefficients,
+                          dof_marker0, dof_marker1);
+  }
+  else
+  {
+    auto x = mesh->geometry().x();
+    std::vector<scalar_value_type_t<T>> _x(x.begin(), x.end());
+    impl::assemble_matrix(mat_add, a, mesh->geometry().dofmap(), _x, constants,
+                          coefficients, dof_marker0, dof_marker1);
+  }
+}
+
 /// Assemble bilinear form into a matrix
 /// @param[in] mat_add The function for adding values into the matrix
 /// @param[in] a The bilinear from to assemble
@@ -198,12 +281,12 @@ void apply_lifting(
 /// @param[in] coefficients Coefficients that appear in `a`
 /// @param[in] bcs Boundary conditions to apply. For boundary condition
 ///  dofs the row and column are zeroed. The diagonal  entry is not set.
-template <typename T>
+template <dolfinx::scalar T, std::floating_point U>
 void assemble_matrix(
-    auto mat_add, const Form<T>& a, std::span<const T> constants,
+    auto mat_add, const Form<T, U>& a, std::span<const T> constants,
     const std::map<std::pair<IntegralType, int>,
                    std::pair<std::span<const T>, int>>& coefficients,
-    const std::vector<std::shared_ptr<const DirichletBC<T>>>& bcs)
+    const std::vector<std::shared_ptr<const DirichletBC<T, U>>>& bcs)
 {
   // Index maps for dof ranges
   auto map0 = a.function_spaces().at(0)->dofmap()->index_map;
@@ -235,8 +318,8 @@ void assemble_matrix(
   }
 
   // Assemble
-  impl::assemble_matrix(mat_add, a, constants, coefficients, dof_marker0,
-                        dof_marker1);
+  assemble_matrix(mat_add, a, constants, coefficients, dof_marker0,
+                  dof_marker1);
 }
 
 /// Assemble bilinear form into a matrix
@@ -244,10 +327,10 @@ void assemble_matrix(
 /// @param[in] a The bilinear from to assemble
 /// @param[in] bcs Boundary conditions to apply. For boundary condition
 ///  dofs the row and column are zeroed. The diagonal  entry is not set.
-template <typename T>
+template <dolfinx::scalar T, std::floating_point U>
 void assemble_matrix(
-    auto mat_add, const Form<T>& a,
-    const std::vector<std::shared_ptr<const DirichletBC<T>>>& bcs)
+    auto mat_add, const Form<T, U>& a,
+    const std::vector<std::shared_ptr<const DirichletBC<T, U>>>& bcs)
 {
   // Prepare constants and coefficients
   const std::vector<T> constants = pack_constants(a);
@@ -263,39 +346,14 @@ void assemble_matrix(
 /// initialised. Does not zero or finalise the matrix.
 /// @param[in] mat_add The function for adding values into the matrix
 /// @param[in] a The bilinear form to assemble
-/// @param[in] constants Constants that appear in `a`
-/// @param[in] coefficients Coefficients that appear in `a`
 /// @param[in] dof_marker0 Boundary condition markers for the rows. If
 /// bc[i] is true then rows i in A will be zeroed. The index i is a
 /// local index.
 /// @param[in] dof_marker1 Boundary condition markers for the columns.
 /// If bc[i] is true then rows i in A will be zeroed. The index i is a
 /// local index.
-template <typename T>
-void assemble_matrix(
-    auto mat_add, const Form<T>& a, std::span<const T> constants,
-    const std::map<std::pair<IntegralType, int>,
-                   std::pair<std::span<const T>, int>>& coefficients,
-    std::span<const std::int8_t> dof_marker0,
-    std::span<const std::int8_t> dof_marker1)
-
-{
-  impl::assemble_matrix(mat_add, a, constants, coefficients, dof_marker0,
-                        dof_marker1);
-}
-
-/// @brief Assemble bilinear form into a matrix. Matrix must already be
-/// initialised. Does not zero or finalise the matrix.
-/// @param[in] mat_add The function for adding values into the matrix
-/// @param[in] a The bilinear form to assemble
-/// @param[in] dof_marker0 Boundary condition markers for the rows. If
-/// bc[i] is true then rows i in A will be zeroed. The index i is a
-/// local index.
-/// @param[in] dof_marker1 Boundary condition markers for the columns.
-/// If bc[i] is true then rows i in A will be zeroed. The index i is a
-/// local index.
-template <typename T>
-void assemble_matrix(auto mat_add, const Form<T>& a,
+template <dolfinx::scalar T, std::floating_point U>
+void assemble_matrix(auto mat_add, const Form<T, U>& a,
                      std::span<const std::int8_t> dof_marker0,
                      std::span<const std::int8_t> dof_marker1)
 
@@ -322,7 +380,7 @@ void assemble_matrix(auto mat_add, const Form<T>& a,
 /// value to the diagonal
 /// @param[in] diagonal The value to add to the diagonal for the
 /// specified rows
-template <typename T>
+template <dolfinx::scalar T>
 void set_diagonal(auto set_fn, std::span<const std::int32_t> rows,
                   T diagonal = 1.0)
 {
@@ -349,12 +407,13 @@ void set_diagonal(auto set_fn, std::span<const std::int32_t> rows,
 /// @param[in] bcs The Dirichlet boundary conditions
 /// @param[in] diagonal The value to add to the diagonal for rows with a
 /// boundary condition applied
-template <typename T>
-void set_diagonal(auto set_fn, const fem::FunctionSpace& V,
-                  const std::vector<std::shared_ptr<const DirichletBC<T>>>& bcs,
-                  T diagonal = 1.0)
+template <dolfinx::scalar T, std::floating_point U>
+void set_diagonal(
+    auto set_fn, const FunctionSpace<U>& V,
+    const std::vector<std::shared_ptr<const DirichletBC<T, U>>>& bcs,
+    T diagonal = 1.0)
 {
-  for (const auto& bc : bcs)
+  for (auto& bc : bcs)
   {
     assert(bc);
     if (V.contains(*bc->function_space()))
@@ -375,14 +434,14 @@ void set_diagonal(auto set_fn, const fem::FunctionSpace& V,
 /// Set bc values in owned (local) part of the vector, multiplied by
 /// 'scale'. The vectors b and x0 must have the same local size. The bcs
 /// should be on (sub-)spaces of the form L that b represents.
-template <typename T>
+template <dolfinx::scalar T, std::floating_point U>
 void set_bc(std::span<T> b,
-            const std::vector<std::shared_ptr<const DirichletBC<T>>>& bcs,
-            std::span<const T> x0, double scale = 1.0)
+            const std::vector<std::shared_ptr<const DirichletBC<T, U>>>& bcs,
+            std::span<const T> x0, T scale = 1)
 {
   if (b.size() > x0.size())
     throw std::runtime_error("Size mismatch between b and x0 vectors.");
-  for (const auto& bc : bcs)
+  for (auto& bc : bcs)
   {
     assert(bc);
     bc->set(b, x0, scale);
@@ -392,12 +451,12 @@ void set_bc(std::span<T> b,
 /// Set bc values in owned (local) part of the vector, multiplied by
 /// 'scale'. The bcs should be on (sub-)spaces of the form L that b
 /// represents.
-template <typename T>
+template <dolfinx::scalar T, std::floating_point U>
 void set_bc(std::span<T> b,
-            const std::vector<std::shared_ptr<const DirichletBC<T>>>& bcs,
-            double scale = 1.0)
+            const std::vector<std::shared_ptr<const DirichletBC<T, U>>>& bcs,
+            T scale = 1)
 {
-  for (const auto& bc : bcs)
+  for (auto& bc : bcs)
   {
     assert(bc);
     bc->set(b, scale);
