@@ -1,4 +1,4 @@
-// Copyright (C) 2012-2018 Chris N. Richardson and Garth N. Wells
+// Copyright (C) 2012-2023 Chris N. Richardson and Garth N. Wells
 //
 // This file is part of DOLFINx (https://www.fenicsproject.org)
 //
@@ -7,6 +7,8 @@
 #pragma once
 
 #include <complex>
+#include <concepts>
+#include <dolfinx/common/types.h>
 #include <hdf5.h>
 #include <mpi.h>
 
@@ -19,24 +21,17 @@ namespace dolfinx
 {
 namespace fem
 {
-template <typename T>
+template <dolfinx::scalar T, std::floating_point U>
 class Function;
 }
 
-namespace io
-{
 /// Low-level methods for reading/writing XDMF files
-namespace xdmf_function
+namespace io::xdmf_function
 {
 
-/// TODO
-void add_function(MPI_Comm comm, const fem::Function<double>& u, const double t,
+/// Write a fem::Function to XDMF
+template <dolfinx::scalar T, std::floating_point U>
+void add_function(MPI_Comm comm, const fem::Function<T, U>& u, double t,
                   pugi::xml_node& xml_node, const hid_t h5_id);
-
-/// TODO
-void add_function(MPI_Comm comm, const fem::Function<std::complex<double>>& u,
-                  const double t, pugi::xml_node& xml_node, const hid_t h5_id);
-
-} // namespace xdmf_function
-} // namespace io
+} // namespace io::xdmf_function
 } // namespace dolfinx
