@@ -68,59 +68,11 @@ graph::AdjacencyList<T> all_to_all(MPI_Comm comm,
   return graph::AdjacencyList<T>(std::move(recv_values),
                                  std::move(recv_offset));
 }
-
-/// Compute values at all mesh 'nodes'
-/// @return The values at all geometric points
-/// @warning This function will be removed soon. Use interpolation
-/// instead.
-template <typename T>
-std::pair<std::vector<T>, std::array<std::size_t, 2>>
-compute_point_values(const fem::Function<T>& u)
-{
-  auto V = u.function_space();
-  assert(V);
-  std::shared_ptr<const mesh::Mesh> mesh = V->mesh();
-  assert(mesh);
-  const int tdim = mesh->topology().dim();
-
-  // Compute in tensor (one for scalar function, . . .)
-  const std::size_t value_size_loc = V->element()->value_size();
-
-  const std::size_t num_points = mesh->geometry().x().size() / 3;
-
-  // Resize Array for holding point values
-  std::vector<T> point_values(num_points * value_size_loc);
-
-  // Prepare cell geometry
-  const graph::AdjacencyList<std::int32_t>& x_dofmap
-      = mesh->geometry().dofmap();
-  const int num_dofs_g = mesh->geometry().cmap().dim();
-
-  // Interpolate point values on each cell (using last computed value if
-  // not continuous, e.g. discontinuous Galerkin methods)
-  auto map = mesh->topology().index_map(tdim);
-  assert(map);
-  const std::int32_t num_cells = map->size_local() + map->num_ghosts();
-
-  std::vector<std::int32_t> cells(num_points, -1);
-  for (std::int32_t c = 0; c < num_cells; ++c)
-  {
-    // Get coordinates for all points in cell
-    std::span<const std::int32_t> dofs = x_dofmap.links(c);
-    for (int i = 0; i < num_dofs_g; ++i)
-      cells[dofs[i]] = c;
-  }
-
-  u.eval(mesh->geometry().x(), {num_points, 3}, cells, point_values,
-         {num_points, value_size_loc});
-
-  return {std::move(point_values), {num_points, value_size_loc}};
-}
-
 //-----------------------------------------------------------------------------
 // Get data width - normally the same as u.value_size(), but expand for
 // 2D vector/tensor because XDMF presents everything as 3D
-std::int64_t get_padded_width(const fem::FiniteElement& e)
+template <std::floating_point U>
+std::int64_t get_padded_width(const fem::FiniteElement<U>& e)
 {
   const int width = e.value_size();
   const int rank = e.value_shape().size();
@@ -132,115 +84,6 @@ std::int64_t get_padded_width(const fem::FiniteElement& e)
     return width;
 }
 //-----------------------------------------------------------------------------
-template <typename Scalar>
-std::vector<Scalar> _get_point_data_values(const fem::Function<Scalar>& u)
-{
-  std::shared_ptr<const mesh::Mesh> mesh = u.function_space()->mesh();
-  assert(mesh);
-  const auto [data_values, dshape] = compute_point_values(u);
-
-  const int width = get_padded_width(*u.function_space()->element());
-  assert(mesh->geometry().index_map());
-  const std::size_t num_local_points
-      = mesh->geometry().index_map()->size_local();
-  assert(dshape[0] >= num_local_points);
-
-  // FIXME: Unpick the below code for the new layout of data from
-  //        GenericFunction::compute_vertex_values
-  std::vector<Scalar> values(width * num_local_points, 0.0);
-  const int value_rank = u.function_space()->element()->value_shape().size();
-  if (value_rank > 0)
-  {
-    // Transpose vector/tensor data arrays
-    const int value_size = u.function_space()->element()->value_size();
-    for (std::size_t i = 0; i < num_local_points; i++)
-    {
-      for (int j = 0; j < value_size; j++)
-      {
-        int tensor2d_off
-            = (j > 1 && value_rank == 2 && value_size == 4) ? 1 : 0;
-        values[i * width + j + tensor2d_off] = data_values[i * dshape[1] + j];
-      }
-    }
-  }
-  else
-  {
-    values.assign(data_values.begin(),
-                  std::next(data_values.begin(), num_local_points * dshape[1]));
-  }
-
-  return values;
-}
-//-----------------------------------------------------------------------------
-template <typename Scalar>
-std::vector<Scalar> _get_cell_data_values(const fem::Function<Scalar>& u)
-{
-  assert(u.function_space()->dofmap());
-  const auto mesh = u.function_space()->mesh();
-  const int value_size = u.function_space()->element()->value_size();
-  const int value_rank = u.function_space()->element()->value_shape().size();
-
-  // Allocate memory for function values at cell centres
-  const int tdim = mesh->topology().dim();
-  const std::int32_t num_local_cells
-      = mesh->topology().index_map(tdim)->size_local();
-  const std::int32_t local_size = num_local_cells * value_size;
-
-  // Build lists of dofs and create map
-  std::vector<std::int32_t> dof_set;
-  dof_set.reserve(local_size);
-  const auto dofmap = u.function_space()->dofmap();
-  const int ndofs = dofmap->element_dof_layout().num_dofs();
-  const int bs = dofmap->bs();
-  assert(ndofs * bs == value_size);
-
-  for (int cell = 0; cell < num_local_cells; ++cell)
-  {
-    // Tabulate dofs
-    auto dofs = dofmap->cell_dofs(cell);
-    for (int i = 0; i < ndofs; ++i)
-    {
-      for (int j = 0; j < bs; ++j)
-        dof_set.push_back(bs * dofs[i] + j);
-    }
-  }
-
-  // Get values
-  std::vector<Scalar> values(dof_set.size());
-  std::span<const Scalar> _u = u.x()->array();
-  for (std::size_t i = 0; i < dof_set.size(); ++i)
-    values[i] = _u[dof_set[i]];
-
-  // Pad out data for 2D vectors/tensors
-  if (value_rank == 1 and value_size == 2)
-  {
-    values.resize(3 * num_local_cells);
-    for (int j = (num_local_cells - 1); j >= 0; --j)
-    {
-      std::array<Scalar, 3> nd = {values[j * 2], values[j * 2 + 1], 0.0};
-      std::copy(nd.begin(), nd.end(), std::next(values.begin(), 3 * j));
-    }
-  }
-  else if (value_rank == 2 and value_size == 4)
-  {
-    values.resize(9 * num_local_cells);
-    for (int j = (num_local_cells - 1); j >= 0; --j)
-    {
-      std::array<Scalar, 9> nd = {values[j * 4],
-                                  values[j * 4 + 1],
-                                  0.0,
-                                  values[j * 4 + 2],
-                                  values[j * 4 + 3],
-                                  0.0,
-                                  0.0,
-                                  0.0,
-                                  0.0};
-      std::copy(nd.begin(), nd.end(), std::next(values.begin(), 9 * j));
-    }
-  }
-  return values;
-}
-//-----------------------------------------------------------------------------
 
 } // namespace
 
@@ -276,6 +119,7 @@ xdmf_utils::get_cell_type(const pugi::xml_node& topology_node)
     throw std::runtime_error("Cannot recognise cell type. Unknown value: "
                              + cell_type);
   }
+
   return it->second;
 }
 //----------------------------------------------------------------------------
@@ -376,8 +220,8 @@ std::int64_t xdmf_utils::get_num_cells(const pugi::xml_node& topology_node)
   if (tdims.size() != 2 and num_cells_topology == -1)
     throw std::runtime_error("Cannot determine number of cells in XDMF mesh");
 
-  // Check for consistency if number of cells appears in both the topology
-  // and DataItem nodes
+  // Check for consistency if number of cells appears in both the
+  // topology and DataItem nodes
   if (num_cells_topology != -1 and tdims.size() == 2)
   {
     if (num_cells_topology != tdims[0])
@@ -387,30 +231,6 @@ std::int64_t xdmf_utils::get_num_cells(const pugi::xml_node& topology_node)
   return std::max(num_cells_topology, tdims[0]);
 }
 //----------------------------------------------------------------------------
-std::vector<double>
-xdmf_utils::get_point_data_values(const fem::Function<double>& u)
-{
-  return _get_point_data_values(u);
-}
-//-----------------------------------------------------------------------------
-std::vector<std::complex<double>>
-xdmf_utils::get_point_data_values(const fem::Function<std::complex<double>>& u)
-{
-  return _get_point_data_values(u);
-}
-//-----------------------------------------------------------------------------
-std::vector<double>
-xdmf_utils::get_cell_data_values(const fem::Function<double>& u)
-{
-  return _get_cell_data_values(u);
-}
-//-----------------------------------------------------------------------------
-std::vector<std::complex<double>>
-xdmf_utils::get_cell_data_values(const fem::Function<std::complex<double>>& u)
-{
-  return _get_cell_data_values(u);
-}
-//-----------------------------------------------------------------------------
 std::string xdmf_utils::vtk_cell_type_str(mesh::CellType cell_type,
                                           int num_nodes)
 {
@@ -442,22 +262,29 @@ std::string xdmf_utils::vtk_cell_type_str(mesh::CellType cell_type,
 }
 //-----------------------------------------------------------------------------
 std::pair<std::vector<std::int32_t>, std::vector<std::int32_t>>
-xdmf_utils::distribute_entity_data(const mesh::Mesh& mesh, int entity_dim,
-                                   std::span<const std::int64_t> entities,
-                                   std::span<const std::int32_t> data)
+xdmf_utils::distribute_entity_data(
+    const mesh::Topology& topology, const std::vector<std::int64_t>& nodes_g,
+    std::int64_t num_nodes_g, const fem::ElementDofLayout& cmap_dof_layout,
+    MDSPAN_IMPL_STANDARD_NAMESPACE::mdspan<
+        const std::int32_t,
+        MDSPAN_IMPL_STANDARD_NAMESPACE::dextents<std::size_t, 2>>
+        xdofmap,
+    int entity_dim, std::span<const std::int64_t> entities,
+    std::span<const std::int32_t> data)
 {
   LOG(INFO) << "XDMF distribute entity data";
 
+  auto cell_types = topology.cell_types();
+  if (cell_types.size() > 1)
+    throw std::runtime_error("cell type IO");
+
   // Use ElementDofLayout of the cell to get vertex dof indices (local
   // to a cell), i.e. build a map from local vertex index to associated
   // local dof index
   std::vector<int> cell_vertex_dofs;
   {
     // Get layout of dofs on 0th cell entity of dimension entity_dim
-    const fem::ElementDofLayout cmap_dof_layout
-        = mesh.geometry().cmap().create_dof_layout();
-    for (int i = 0; i < mesh::cell_num_entities(mesh.topology().cell_type(), 0);
-         ++i)
+    for (int i = 0; i < mesh::cell_num_entities(cell_types.back(), 0); ++i)
     {
       const std::vector<int>& local_index = cmap_dof_layout.entity_dofs(0, i);
       assert(local_index.size() == 1);
@@ -470,19 +297,15 @@ xdmf_utils::distribute_entity_data(const mesh::Mesh& mesh, int entity_dim,
   //    'postmaster' rank, and receive global "input" nodes for which
   //    this rank is the postmaster
 
-  auto postmaster_global_nodes_sendrecv = [](const mesh::Mesh& mesh)
+  auto postmaster_global_nodes_sendrecv
+      = [](const mesh::Topology& topology,
+           const std::vector<std::int64_t>& nodes_g, std::int64_t num_nodes_g)
   {
-    const MPI_Comm comm = mesh.comm();
+    const MPI_Comm comm = topology.comm();
     const int comm_size = dolfinx::MPI::size(comm);
 
-    // Get "input" global node indices (as in the input file before any
-    // internal re-ordering)
-    const std::vector<std::int64_t>& nodes_g
-        = mesh.geometry().input_global_indices();
-
     // Send input global indices to 'post master' rank, based on input
     // global index value
-    const std::int64_t num_nodes_g = mesh.geometry().index_map()->size_global();
     // NOTE: could make this int32_t be sending: index <- index -
     // dest_rank_offset
     std::vector<std::vector<std::int64_t>> nodes_g_send(comm_size);
@@ -496,7 +319,7 @@ xdmf_utils::distribute_entity_data(const mesh::Mesh& mesh, int entity_dim,
     }
 
     // Send/receive
-    LOG(INFO) << "XDMF send entity nodes size:(" << num_nodes_g << ")";
+    LOG(INFO) << "XDMF send entity nodes size: (" << num_nodes_g << ")";
     graph::AdjacencyList<std::int64_t> nodes_g_recv
         = all_to_all(comm, graph::AdjacencyList<std::int64_t>(nodes_g_send));
 
@@ -504,7 +327,7 @@ xdmf_utils::distribute_entity_data(const mesh::Mesh& mesh, int entity_dim,
   };
 
   const graph::AdjacencyList<std::int64_t> nodes_g_recv
-      = postmaster_global_nodes_sendrecv(mesh);
+      = postmaster_global_nodes_sendrecv(topology, nodes_g, num_nodes_g);
 
   // -------------------
   // 2. Send the entity key (nodes list) and tag to the postmaster based
@@ -515,22 +338,25 @@ xdmf_utils::distribute_entity_data(const mesh::Mesh& mesh, int entity_dim,
   //    communication in Step 1 could be make non-blocking.
 
   auto postmaster_global_ent_sendrecv
-      = [&cell_vertex_dofs](const mesh::Mesh& mesh, int entity_dim,
-                            std::span<const std::int64_t> entities,
-                            std::span<const std::int32_t> data)
+      = [&cell_vertex_dofs](
+            const mesh::Topology& topology, std::int64_t num_nodes_g,
+            const fem::ElementDofLayout& cmap_dof_layout, int entity_dim,
+            std::span<const std::int64_t> entities,
+            std::span<const std::int32_t> data)
   {
-    const MPI_Comm comm = mesh.comm();
+    const MPI_Comm comm = topology.comm();
     const int comm_size = dolfinx::MPI::size(comm);
-    const std::int64_t num_nodes_g = mesh.geometry().index_map()->size_global();
+
+    auto cell_types = topology.cell_types();
+    if (cell_types.size() > 1)
+      throw std::runtime_error("cell type IO");
 
     const std::size_t num_vert_per_entity = mesh::cell_num_entities(
-        mesh::cell_entity_type(mesh.topology().cell_type(), entity_dim, 0), 0);
-    auto c_to_v = mesh.topology().connectivity(mesh.topology().dim(), 0);
+        mesh::cell_entity_type(cell_types.back(), entity_dim, 0), 0);
+    auto c_to_v = topology.connectivity(topology.dim(), 0);
     if (!c_to_v)
       throw std::runtime_error("Missing cell-vertex connectivity.");
 
-    const fem::ElementDofLayout cmap_dof_layout
-        = mesh.geometry().cmap().create_dof_layout();
     const std::vector<int> entity_layout
         = cmap_dof_layout.entity_closure_dofs(entity_dim, 0);
 
@@ -587,8 +413,8 @@ xdmf_utils::distribute_entity_data(const mesh::Mesh& mesh, int entity_dim,
     return std::pair(entities_recv, data_recv);
   };
 
-  const auto [entities_recv, data_recv]
-      = postmaster_global_ent_sendrecv(mesh, entity_dim, entities, data);
+  const auto [entities_recv, data_recv] = postmaster_global_ent_sendrecv(
+      topology, num_nodes_g, cmap_dof_layout, entity_dim, entities, data);
 
   // -------------------
   // 3. As 'postmaster', send back the entity key (vertex list) and tag
@@ -601,16 +427,19 @@ xdmf_utils::distribute_entity_data(const mesh::Mesh& mesh, int entity_dim,
   // end.
 
   auto postmaster_send_to_candidates
-      = [](const mesh::Mesh& mesh, int entity_dim,
+      = [](const mesh::Topology& topology, int entity_dim,
            const graph::AdjacencyList<std::int64_t>& nodes_g_recv,
            const graph::AdjacencyList<std::int64_t>& entities_recv,
            const graph::AdjacencyList<std::int32_t>& data_recv)
   {
-    const MPI_Comm comm = mesh.comm();
+    const MPI_Comm comm = topology.comm();
     const int comm_size = dolfinx::MPI::size(comm);
 
+    auto cell_types = topology.cell_types();
+    if (cell_types.size() > 1)
+      throw std::runtime_error("cell type IO");
     const std::size_t num_vert_per_entity = mesh::cell_num_entities(
-        mesh::cell_entity_type(mesh.topology().cell_type(), entity_dim, 0), 0);
+        mesh::cell_entity_type(cell_types.back(), entity_dim, 0), 0);
 
     // Build map from global node index to ranks that have the node
     std::multimap<std::int64_t, int> node_to_rank;
@@ -659,7 +488,7 @@ xdmf_utils::distribute_entity_data(const mesh::Mesh& mesh, int entity_dim,
   };
 
   const auto [recv_ents, recv_vals] = postmaster_send_to_candidates(
-      mesh, entity_dim, nodes_g_recv, entities_recv, data_recv);
+      topology, entity_dim, nodes_g_recv, entities_recv, data_recv);
 
   // -------------------
   // 4. From the received (key, value) data, determine which keys
@@ -676,31 +505,38 @@ xdmf_utils::distribute_entity_data(const mesh::Mesh& mesh, int entity_dim,
   //       entities.
 
   auto determine_my_entities
-      = [&cell_vertex_dofs](const mesh::Mesh& mesh, int entity_dim,
-                            const graph::AdjacencyList<std::int64_t>& recv_ents,
-                            const graph::AdjacencyList<std::int32_t>& recv_vals)
+      = [&cell_vertex_dofs](
+            const mesh::Topology& topology,
+            const std::vector<std::int64_t>& nodes_g,
+            MDSPAN_IMPL_STANDARD_NAMESPACE::mdspan<
+                const std::int32_t,
+                MDSPAN_IMPL_STANDARD_NAMESPACE::dextents<std::size_t, 2>>
+                x_dofmap,
+            int entity_dim, const graph::AdjacencyList<std::int64_t>& recv_ents,
+            const graph::AdjacencyList<std::int32_t>& recv_vals)
   {
     // Build map from input global indices to local vertex numbers
     LOG(INFO) << "XDMF build map";
 
+    auto cell_types = topology.cell_types();
+    if (cell_types.size() > 1)
+      throw std::runtime_error("cell type IO");
+
     const std::size_t num_vert_per_entity = mesh::cell_num_entities(
-        mesh::cell_entity_type(mesh.topology().cell_type(), entity_dim, 0), 0);
-    auto c_to_v = mesh.topology().connectivity(mesh.topology().dim(), 0);
+        mesh::cell_entity_type(cell_types.back(), entity_dim, 0), 0);
+    auto c_to_v = topology.connectivity(topology.dim(), 0);
     if (!c_to_v)
       throw std::runtime_error("Missing cell-vertex connectivity.");
 
-    const std::vector<std::int64_t>& nodes_g
-        = mesh.geometry().input_global_indices();
-
-    const graph::AdjacencyList<std::int32_t>& x_dofmap
-        = mesh.geometry().dofmap();
     std::map<std::int64_t, std::int32_t> igi_to_vertex;
     for (int c = 0; c < c_to_v->num_nodes(); ++c)
     {
       auto vertices = c_to_v->links(c);
-      auto x_dofs = x_dofmap.links(c);
+      auto xdofs = MDSPAN_IMPL_STANDARD_NAMESPACE::
+          MDSPAN_IMPL_PROPOSED_NAMESPACE::submdspan(
+              x_dofmap, c, MDSPAN_IMPL_STANDARD_NAMESPACE::full_extent);
       for (std::size_t v = 0; v < vertices.size(); ++v)
-        igi_to_vertex[nodes_g[x_dofs[cell_vertex_dofs[v]]]] = vertices[v];
+        igi_to_vertex[nodes_g[xdofs[cell_vertex_dofs[v]]]] = vertices[v];
     }
 
     std::vector<std::int32_t> entities_new;
@@ -738,8 +574,8 @@ xdmf_utils::distribute_entity_data(const mesh::Mesh& mesh, int entity_dim,
     return std::pair(std::move(entities_new), std::move(data_new));
   };
 
-  auto [entities_new, data_new]
-      = determine_my_entities(mesh, entity_dim, recv_ents, recv_vals);
+  auto [entities_new, data_new] = determine_my_entities(
+      topology, nodes_g, xdofmap, entity_dim, recv_ents, recv_vals);
 
   return {std::move(entities_new), std::move(data_new)};
 }
