@@ -6,16 +6,12 @@
 
 #pragma once
 
-#include "Geometry.h"
-#include "Mesh.h"
 #include "Topology.h"
 #include <algorithm>
-#include <concepts>
 #include <dolfinx/common/IndexMap.h>
 #include <dolfinx/common/log.h>
 #include <dolfinx/common/utils.h>
 #include <dolfinx/graph/AdjacencyList.h>
-#include <dolfinx/graph/partition.h>
 #include <dolfinx/io/cells.h>
 #include <memory>
 #include <span>
@@ -25,7 +21,7 @@
 namespace dolfinx::mesh
 {
 
-/// @brief MeshTags associate values with mesh entities.
+/// @brief MeshTags associate values with mesh topology entities.
 ///
 /// The entity index (local to process) identifies the entity. MeshTags
 /// is a *sparse* data storage class; it allows tags to be associated
@@ -39,17 +35,21 @@ public:
   /// @brief Create a MeshTag from entities of given dimension on a
   /// mesh.
   ///
-  /// @param[in] mesh The mesh on which the tags are associated.
+  /// @param[in] topology Mesh topology on which the tags are associated.
   /// @param[in] dim Topological dimension of mesh entities to tag.
   /// @param[in] indices List of entity indices (indices local to the
   /// process).
   /// @param[in] values List of values for each index in indices. The
   /// size must be equal to the size of `indices`.
   /// @pre `indices` must be sorted and unique.
-  template <std::convertible_to<std::vector<std::int32_t>> U,
-            std::convertible_to<std::vector<T>> V>
-  MeshTags(std::shared_ptr<const Mesh> mesh, int dim, U&& indices, V&& values)
-      : _mesh(mesh), _dim(dim), _indices(std::forward<U>(indices)),
+  template <typename U, typename V>
+    requires std::is_convertible_v<std::remove_cvref_t<U>,
+                                   std::vector<std::int32_t>>
+                 and std::is_convertible_v<std::remove_cvref_t<V>,
+                                           std::vector<T>>
+  MeshTags(std::shared_ptr<const Topology> topology, int dim, U&& indices,
+           V&& values)
+      : _topology(topology), _dim(dim), _indices(std::forward<U>(indices)),
         _values(std::forward<V>(values))
   {
     if (_indices.size() != _values.size())
@@ -96,27 +96,27 @@ public:
     return indices;
   }
 
-  /// Indices of tagged mesh entities (local-to-process). The indices
-  /// are sorted.
+  /// Indices of tagged topology entities (local-to-process). The
+  /// indices are sorted.
   std::span<const std::int32_t> indices() const { return _indices; }
 
-  /// Values attached to mesh entities
+  /// Values attached to topology entities
   std::span<const T> values() const { return _values; }
 
   /// Return topological dimension of tagged entities
   int dim() const { return _dim; }
 
-  /// Return mesh
-  std::shared_ptr<const Mesh> mesh() const { return _mesh; }
+  /// Return topology
+  std::shared_ptr<const Topology> topology() const { return _topology; }
 
   /// Name
   std::string name = "mesh_tags";
 
 private:
-  // Associated mesh
-  std::shared_ptr<const Mesh> _mesh;
+  // Associated topology
+  std::shared_ptr<const Topology> _topology;
 
-  // Topological dimension of tagged mesh entities
+  // Topological dimension of tagged topology entities
   int _dim;
 
   // Local-to-process indices of tagged entities
@@ -127,28 +127,26 @@ private:
 };
 
 /// @brief Create MeshTags from arrays
-/// @param[in] mesh The Mesh that the tags are associated with
+/// @param[in] topology Mesh topology that the tags are associated with
 /// @param[in] dim Topological dimension of tagged entities
 /// @param[in] entities Local vertex indices for tagged entities.
-///
 /// @param[in] values Tag values for each entity in `entities`. The
 /// length of `values` must be equal to number of rows in `entities`.
 /// @note Entities that do not exist on this rank are ignored.
 /// @warning `entities` must not contain duplicate entities.
 template <typename T>
-MeshTags<T> create_meshtags(std::shared_ptr<const Mesh> mesh, int dim,
+MeshTags<T> create_meshtags(std::shared_ptr<const Topology> topology, int dim,
                             const graph::AdjacencyList<std::int32_t>& entities,
                             std::span<const T> values)
 {
   LOG(INFO)
-      << "Building MeshTgas object from tagged entities (defined by vertices).";
+      << "Building MeshTags object from tagged entities (defined by vertices).";
 
-  assert(mesh);
-
-  // Compute the indices of the mesh entities (index is set to -1 if it
-  // can't be found)
+  // Compute the indices of the topology entities (index is set to -1 if
+  // it can't be found)
+  assert(topology);
   const std::vector<std::int32_t> indices
-      = entities_to_index(mesh->topology(), dim, entities);
+      = entities_to_index(*topology, dim, entities);
   if (indices.size() != values.size())
   {
     throw std::runtime_error(
@@ -165,7 +163,7 @@ MeshTags<T> create_meshtags(std::shared_ptr<const Mesh> mesh, int dim,
   values_sorted.erase(values_sorted.begin(),
                       std::next(values_sorted.begin(), pos0));
 
-  return MeshTags<T>(mesh, dim, std::move(indices_sorted),
+  return MeshTags<T>(topology, dim, std::move(indices_sorted),
                      std::move(values_sorted));
 }
 } // namespace dolfinx::mesh
