@@ -11,6 +11,7 @@
 #include <dolfinx/common/MPI.h>
 #include <memory>
 #include <span>
+#include <tuple>
 #include <vector>
 
 namespace dolfinx::common
@@ -43,7 +44,7 @@ class Topology
 {
 public:
   /// Create empty mesh topology
-  Topology(MPI_Comm comm, CellType type);
+  Topology(MPI_Comm comm, std::vector<CellType> type);
 
   /// Copy constructor
   Topology(const Topology& topology) = default;
@@ -63,9 +64,29 @@ public:
   /// @brief Return the topological dimension of the mesh.
   int dim() const noexcept;
 
+  /// @brief Set the offsets for each group of entities of a particular
+  /// dimension. See `entity_group_offsets`.
+  /// @param dim Dimension of the entities
+  /// @param offsets The offsets
+  void set_entity_group_offsets(int dim,
+                                const std::vector<std::int32_t>& offsets);
+
+  /// @brief Get the offsets for each group of entities of a particular
+  /// dimension.
+  ///
+  /// The topology may consist of more than one cell type or facet type.
+  /// In that case, the cells of the same types are grouped together in
+  /// blocks, firstly for regular cells, then repeated for ghost cells.
+  /// For example, a mesh with two triangles, three quads and no ghosts
+  /// would have offsets: 0, 2, 5, 5, 5. A mesh with twenty tetrahedra
+  /// and two ghost tetrahedra has offsets: 0, 20, 22.
+  /// @param dim Dimension of the entities
+  /// @return The offsets
+  const std::vector<std::int32_t>& entity_group_offsets(int dim) const;
+
   /// @todo Merge with set_connectivity
   ///
-  /// Set the IndexMap for dimension dim
+  /// @brief Set the IndexMap for dimension dim
   /// @warning This is experimental and likely to change
   void set_index_map(int dim, std::shared_ptr<const common::IndexMap> map);
 
@@ -93,7 +114,7 @@ public:
   void set_connectivity(std::shared_ptr<graph::AdjacencyList<std::int32_t>> c,
                         int d0, int d1);
 
-  /// Returns the permutation information
+  /// @brief Returns the permutation information
   const std::vector<std::uint32_t>& get_cell_permutation_info() const;
 
   /// @brief Get the permutation number to apply to a facet.
@@ -110,9 +131,9 @@ public:
   /// computed
   const std::vector<std::uint8_t>& get_facet_permutations() const;
 
-  /// Cell type
-  /// @return Cell type that the topology is for
-  CellType cell_type() const noexcept;
+  /// @brief Cell type
+  /// @return Cell types that the topology is for
+  std::vector<CellType> cell_types() const noexcept;
 
   /// @brief Create entities of given topological dimension.
   /// @param[in] dim Topological dimension
@@ -126,10 +147,11 @@ public:
   /// @param[in] d1 Topological dimension
   void create_connectivity(int d0, int d1);
 
-  /// Compute entity permutations and reflections
+  /// @brief Compute entity permutations and reflections.
   void create_entity_permutations();
 
-  /// List of inter-process facets, if facet topology has been computed
+  /// @brief List of inter-process facets, if facet topology has been
+  /// computed.
   const std::vector<std::int32_t>& interprocess_facets() const;
 
   /// Original cell index
@@ -143,8 +165,11 @@ private:
   // MPI communicator
   dolfinx::MPI::Comm _comm;
 
-  // Cell type
-  CellType _cell_type;
+  // Cell types
+  std::vector<CellType> _cell_types;
+
+  // Entity group offsets
+  std::array<std::vector<std::int32_t>, 4> _entity_group_offsets;
 
   // Parallel layout of entities for each dimension
   std::array<std::shared_ptr<const common::IndexMap>, 4> _index_map;
@@ -179,7 +204,9 @@ private:
 /// with each cell
 /// @param[in] ghost_owners The owning rank of each ghost cell (ghost
 /// cells are always at the end of the list of `cells`)
-/// @param[in] cell_type The cell shape
+/// @param[in] cell_type A vector with cell shapes
+/// @param[in] cell_group_offsets vector with each group offset, including
+/// ghosts.
 /// @param[in] boundary_vertices List of vertices on the exterior of the
 /// local mesh which may be shared with other processes.
 /// @return A distributed mesh topology
@@ -187,9 +214,24 @@ Topology create_topology(MPI_Comm comm,
                          const graph::AdjacencyList<std::int64_t>& cells,
                          std::span<const std::int64_t> original_cell_index,
                          std::span<const int> ghost_owners,
-                         const CellType& cell_type,
+                         const std::vector<CellType>& cell_type,
+                         const std::vector<std::int32_t>& cell_group_offsets,
                          std::span<const std::int64_t> boundary_vertices);
 
+/// @brief Create a topology for a subset of entities of a given
+/// topological dimension.
+/// @param topology Original topology.
+/// @param dim Topological dimension of the entities in the new topology.
+/// @param entities The indices of the entities in `topology` to include
+/// in the new topology.
+/// @return New topology of dimension `dim` with all entities in
+/// `entities`, map from entities of dimension `dim` in new sub-topology
+/// to entities in `topology`, and map from vertices in new sub-topology
+/// to vertices in `topology`.
+std::tuple<Topology, std::vector<int32_t>, std::vector<int32_t>>
+create_subtopology(const Topology& topology, int dim,
+                   std::span<const std::int32_t> entities);
+
 /// @brief Get entity indices for entities defined by their vertices.
 ///
 /// @warning This function may be removed in the future.
