@@ -6,7 +6,9 @@
 //
 // Unit tests for Distributed la::Vector
 
-#include <catch2/catch.hpp>
+#include <catch2/catch_template_test_macros.hpp>
+#include <catch2/catch_test_macros.hpp>
+#include <complex>
 #include <dolfinx/common/IndexMap.h>
 #include <dolfinx/common/MPI.h>
 #include <dolfinx/la/Vector.h>
@@ -33,18 +35,18 @@ void test_vector()
                                             (mpi_rank + 1) % mpi_size);
 
   // Create an IndexMap
-  const auto index_map = std::make_shared<common::IndexMap>(
+  auto index_map = std::make_shared<common::IndexMap>(
       MPI_COMM_WORLD, size_local, ghosts, global_ghost_owner);
 
   la::Vector<T> v(index_map, 1);
   std::fill(v.mutable_array().begin(), v.mutable_array().end(), 1.0);
 
-  const double norm2 = la::squared_norm(v);
+  double norm2 = la::squared_norm(v);
   CHECK(norm2 == mpi_size * size_local);
 
   std::fill(v.mutable_array().begin(), v.mutable_array().end(), mpi_rank);
 
-  const double sumn2
+  double sumn2
       = size_local * (mpi_size - 1) * mpi_size * (2 * mpi_size - 1) / 6;
   CHECK(la::squared_norm(v) == sumn2);
   CHECK(la::norm(v, la::Norm::l2) == std::sqrt(sumn2));
