@@ -19,43 +19,42 @@
 # Scattering" p138-139. In real mode, the Method of Manufactured
 # Solutions is used to produce the exact solution and source term.
 
+from mpi4py import MPI
+from petsc4py import PETSc
+
 # +
 import numpy as np
 
 import ufl
-from dolfinx.fem import Function, FunctionSpace, assemble_scalar, form
+from dolfinx.fem import Function, assemble_scalar, form, functionspace
 from dolfinx.fem.petsc import LinearProblem
 from dolfinx.io import XDMFFile
 from dolfinx.mesh import create_unit_square
 from ufl import dx, grad, inner
 
-from mpi4py import MPI
-from petsc4py import PETSc
-
-# wavenumber
+# Wavenumber
 k0 = 4 * np.pi
 
-# approximation space polynomial degree
+# Approximation space polynomial degree
 deg = 1
 
-# number of elements in each direction of msh
+# Number of elements in each direction of the mesh
 n_elem = 128
 
 msh = create_unit_square(MPI.COMM_WORLD, n_elem, n_elem)
 n = ufl.FacetNormal(msh)
 
 # Source amplitude
-if np.issubdtype(PETSc.ScalarType, np.complexfloating):
-    A = PETSc.ScalarType(1 + 1j)
+if np.issubdtype(PETSc.ScalarType, np.complexfloating):  # type: ignore
+    A = PETSc.ScalarType(1 + 1j)  # type: ignore
 else:
     A = 1
 
 # Test and trial function space
-V = FunctionSpace(msh, ("Lagrange", deg))
+V = functionspace(msh, ("Lagrange", deg))
 
 # Define variational problem
-u = ufl.TrialFunction(V)
-v = ufl.TestFunction(V)
+u, v = ufl.TrialFunction(V), ufl.TestFunction(V)
 f = Function(V)
 f.interpolate(lambda x: A * k0**2 * np.cos(k0 * x[0]) * np.cos(k0 * x[1]))
 a = inner(grad(u), grad(v)) * dx - k0**2 * inner(u, v) * dx
@@ -73,13 +72,13 @@ with XDMFFile(MPI.COMM_WORLD, "out_helmholtz/plane_wave.xdmf", "w", encoding=XDM
     file.write_function(uh)
 # -
 
-# Calculate $L_2$ and $H^1$ errors of FEM solution and best approximation.
-# This demonstrates the error bounds given in Ihlenburg. Pollution errors are
-# evident for high wavenumbers.
+# Calculate $L_2$ and $H^1$ errors of FEM solution and best
+# approximation. This demonstrates the error bounds given in Ihlenburg.
+# Pollution errors are evident for high wavenumbers.
 
 # +
 # Function space for exact solution - need it to be higher than deg
-V_exact = FunctionSpace(msh, ("Lagrange", deg + 3))
+V_exact = functionspace(msh, ("Lagrange", deg + 3))
 u_exact = Function(V_exact)
 u_exact.interpolate(lambda x: A * np.cos(k0 * x[0]) * np.cos(k0 * x[1]))
 
