@@ -8,69 +8,75 @@
 #       jupytext_version: 1.13.6
 # ---
 
-# Copyright (C) 2022 Garth N. Wells
-#
-# This file is part of DOLFINx (<https://www.fenicsproject.org>)
-#
-# SPDX-License-Identifier:    LGPL-3.0-or-later
-#
 # # Interpolation and IO
 #
-# This demo shows the interpolation of functions into vector-element
-# $H(\mathrm{curl})$ finite element spaces, and the interpolation of these
-# special finite elements in discontinuous Lagrange spaces for
-# artifact-free visualisation.
+# Copyright (C) 2022 Garth N. Wells
+#
+# This demo ({download}`demo_interpolation-io.py`) shows the
+# interpolation of functions into vector-element $H(\mathrm{curl})$
+# finite element spaces, and the interpolation of these special finite
+# elements in discontinuous Lagrange spaces for artifact-free
+# visualisation.
+
+from mpi4py import MPI
 
 # +
 import numpy as np
 
-from dolfinx import plot
-from dolfinx.fem import Function, FunctionSpace, VectorFunctionSpace
+from dolfinx import default_scalar_type, plot
+from dolfinx.fem import Function, functionspace
 from dolfinx.mesh import CellType, create_rectangle, locate_entities
 
-from mpi4py import MPI
-from petsc4py.PETSc import ScalarType
+# -
+
+# Create a mesh. For later in the demo we need to ensure that a boundary
+# between cells is located at $x_0=0.5$.
 
-# Create a mesh. For what comes later in this demo we need to ensure
-# that a boundary between cells is located at x0=0.5
 msh = create_rectangle(MPI.COMM_WORLD, ((0.0, 0.0), (1.0, 1.0)), (16, 16), CellType.triangle)
 
-# Create Nedelec function space and finite element Function
-V = FunctionSpace(msh, ("Nedelec 1st kind H(curl)", 1))
-u = Function(V, dtype=ScalarType)
+# Create a Nédélec function space and finite element Function
+
+V = functionspace(msh, ("Nedelec 1st kind H(curl)", 1))
+u = Function(V, dtype=default_scalar_type)
+
+# Find cells with *all* vertices (0) $x_0 <= 0.5$ or (1) $x_0 >= 0.5$:
 
-# Find cells with *all* vertices (0) <= 0.5 or (1) >= 0.5
 tdim = msh.topology.dim
 cells0 = locate_entities(msh, tdim, lambda x: x[0] <= 0.5)
 cells1 = locate_entities(msh, tdim, lambda x: x[0] >= 0.5)
 
-# Interpolate in the Nedelec/H(curl) space a vector-valued expression
-# ``f``, where f \dot e_0 is discontinuous at x0 = 0.5 and  f \dot e_1
-# is continuous.
+# Interpolate in the Nédélec/H(curl) space a vector-valued expression
+# `f`, where $f \cdot n$ is discontinuous at $x_0 = 0.5$ and  $f \cdot
+# e$ is continuous.
+
 u.interpolate(lambda x: np.vstack((x[0], x[1])), cells0)
 u.interpolate(lambda x: np.vstack((x[0] + 1, x[1])), cells1)
 
 # Create a vector-valued discontinuous Lagrange space and function, and
-# interpolate the H(curl) function `u`
-V0 = VectorFunctionSpace(msh, ("Discontinuous Lagrange", 1))
-u0 = Function(V0, dtype=ScalarType)
+# interpolate the $H({\rm curl})$ function `u`
+
+gdim = msh.geometry.dim
+V0 = functionspace(msh, ("Discontinuous Lagrange", 1, (gdim,)))
+u0 = Function(V0, dtype=default_scalar_type)
 u0.interpolate(u)
 
+# We save the interpolated function `u0` in VTX format. When visualising
+# the field, at $x_0 = 0.5$ the $x_0$-component should appear
+# discontinuous and the $x_1$-component should appear continuous.
+
 try:
-    # Save the interpolated function u0 in VTX format. It should be seen
-    # when visualising that the x0-component is discontinuous across
-    # x0=0.5 and the x0-component is continuous across x0=0.5
     from dolfinx.io import VTXWriter
-    with VTXWriter(msh.comm, "output_nedelec.bp", u0) as f:
+    with VTXWriter(msh.comm, "output_nedelec.bp", u0, "bp4") as f:
         f.write(0.0)
 except ImportError:
-    print("ADIOS2 required for VTK output")
+    print("ADIOS2 required for VTX output")
 
 
-# Plot solution
+# Plot the functions
+
 try:
     import pyvista
-    cells, types, x = plot.create_vtk_mesh(V0)
+    cells, types, x = plot.vtk_mesh(V0)
     grid = pyvista.UnstructuredGrid(cells, types, x)
     values = np.zeros((x.shape[0], 3), dtype=np.float64)
     values[:, :msh.topology.dim] = u0.x.array.reshape(x.shape[0], msh.topology.dim).real
