@@ -3,7 +3,7 @@
 # This file is part of DOLFINx (https://www.fenicsproject.org)
 #
 # SPDX-License-Identifier:    LGPL-3.0-or-later
-"""General tools for timing and configuration"""
+"""General tools for timing and configuration."""
 
 import functools
 import typing
