@@ -9,6 +9,7 @@ Functions in this module generally apply functions in :mod:`dolfinx.fem`
 to PETSc linear algebra objects and handle any PETSc-specific
 preparation."""
 
+# mypy: ignore-errors
 
 from __future__ import annotations
 
@@ -17,26 +18,33 @@ import functools
 import os
 import typing
 
+import petsc4py
+import petsc4py.lib
+from petsc4py import PETSc
+
 import ufl
 from dolfinx import cpp as _cpp
 from dolfinx import la
 from dolfinx.cpp.fem import pack_coefficients as _pack_coefficients
 from dolfinx.cpp.fem import pack_constants as _pack_constants
 from dolfinx.fem import assemble
-from dolfinx.fem.bcs import DirichletBCMetaClass
+from dolfinx.fem.bcs import DirichletBC
 from dolfinx.fem.bcs import bcs_by_block as _bcs_by_block
-from dolfinx.fem.forms import FormMetaClass
+from dolfinx.fem.forms import Form
 from dolfinx.fem.forms import extract_function_spaces as _extract_spaces
 from dolfinx.fem.forms import form as _create_form
-from dolfinx.fem.forms import form_types
 from dolfinx.fem.function import Function as _Function
+from dolfinx.la import create_petsc_vector
 
-import petsc4py
-import petsc4py.lib
-from petsc4py import PETSc
+__all__ = ["create_vector", "create_vector_block", "create_vector_nest",
+           "create_matrix", "create_matrix_block", "create_matrix_nest",
+           "assemble_vector", "assemble_vector_nest", "assemble_vector_block",
+           "assemble_matrix", "assemble_matrix_nest", "assemble_matrix_block",
+           "apply_lifting", "apply_lifting_nest", "set_bc", "set_bc_nest",
+           "LinearProblem", "NonlinearProblem"]
 
 
-def _extract_function_spaces(a: typing.List[typing.List[FormMetaClass]]):
+def _extract_function_spaces(a: typing.List[typing.List[Form]]):
     """From a rectangular array of bilinear forms, extract the function
     spaces for each block row and block column.
 
@@ -69,28 +77,28 @@ def _extract_function_spaces(a: typing.List[typing.List[FormMetaClass]]):
 # -- Vector instantiation ----------------------------------------------------
 
 
-def create_vector(L: form_types) -> PETSc.Vec:
+def create_vector(L: Form) -> PETSc.Vec:
     """Create a PETSc vector that is compaible with a linear form.
 
     Args:
         L: A linear form.
 
     Returns:
-        A PETSc vector with a layout that is compatible with `L`.
+        A PETSc vector with a layout that is compatible with ``L``.
 
     """
     dofmap = L.function_spaces[0].dofmap
-    return la.create_petsc_vector(dofmap.index_map, dofmap.index_map_bs)
+    return create_petsc_vector(dofmap.index_map, dofmap.index_map_bs)
 
 
-def create_vector_block(L: typing.List[form_types]) -> PETSc.Vec:
+def create_vector_block(L: typing.List[Form]) -> PETSc.Vec:
     """Create a PETSc vector (blocked) that is compaible with a list of linear forms.
 
     Args:
         L: List of linear forms.
 
     Returns:
-        A PETSc vector with a layout that is compatible with `L`.
+        A PETSc vector with a layout that is compatible with ``L``.
 
     """
     maps = [(form.function_spaces[0].dofmap.index_map,
@@ -98,15 +106,15 @@ def create_vector_block(L: typing.List[form_types]) -> PETSc.Vec:
     return _cpp.fem.petsc.create_vector_block(maps)
 
 
-def create_vector_nest(L: typing.List[form_types]) -> PETSc.Vec:
-    """Create a PETSc netsted vector (``VecNest``) that is compaible with a list of linear forms.
+def create_vector_nest(L: typing.List[Form]) -> PETSc.Vec:
+    """Create a PETSc nested vector (``VecNest``) that is compatible with a list of linear forms.
 
     Args:
         L: List of linear forms.
 
     Returns:
         A PETSc nested vector (``VecNest``) with a layout that is
-        compatible with `L`.
+        compatible with ``L``.
 
     """
     maps = [(form.function_spaces[0].dofmap.index_map,
@@ -116,58 +124,56 @@ def create_vector_nest(L: typing.List[form_types]) -> PETSc.Vec:
 
 # -- Matrix instantiation ----------------------------------------------------
 
-def create_matrix(a: form_types, mat_type=None) -> PETSc.Mat:
-    """Create a PETSc matrix that is compaible with a bilinear form.
+def create_matrix(a: Form, mat_type=None) -> PETSc.Mat:
+    """Create a PETSc matrix that is compatible with a bilinear form.
 
     Args:
         a: A bilinear form.
         mat_type: The PETSc matrix type (``MatType``).
 
     Returns:
-        A PETSc matrix with a layout that is compatible with `a`.
+        A PETSc matrix with a layout that is compatible with ``a``.
 
     """
     if mat_type is None:
-        return _cpp.fem.petsc.create_matrix(a)
+        return _cpp.fem.petsc.create_matrix(a._cpp_object)
     else:
-        return _cpp.fem.petsc.create_matrix(a, mat_type)
+        return _cpp.fem.petsc.create_matrix(a._cpp_object, mat_type)
 
 
-def create_matrix_block(a: typing.List[typing.List[form_types]]) -> PETSc.Mat:
-    """Create a PETSc matrix that is compaible with a rectangular array of bilinear forms.
+def create_matrix_block(a: typing.List[typing.List[Form]]) -> PETSc.Mat:
+    """Create a PETSc matrix that is compatible with a rectangular array of bilinear forms.
 
     Args:
-        a: A rectangular array of bilinear forms.
+        a: Rectangular array of bilinear forms.
 
     Returns:
-        A PETSc matrix with a blocked layout that is compatible with `a`.
+        A PETSc matrix with a blocked layout that is compatible with
+        ``a``.
 
     """
-    return _cpp.fem.petsc.create_matrix_block(a)
+    _a = [[None if form is None else form._cpp_object for form in arow] for arow in a]
+    return _cpp.fem.petsc.create_matrix_block(_a)
 
 
-def create_matrix_nest(a: typing.List[typing.List[form_types]]) -> PETSc.Mat:
-    """Create a PETSc matrix (``MatNest``) that is compaible with a rectangular array of bilinear forms.
+def create_matrix_nest(a: typing.List[typing.List[Form]]) -> PETSc.Mat:
+    """Create a PETSc matrix (``MatNest``) that is compatible with a rectangular array of bilinear forms.
 
     Args:
-        a: A rectangular array of bilinear forms.
+        a: Rectangular array of bilinear forms.
 
     Returns:
-        A PETSc matrix ('MatNest``) that is compatible with `a`.
+        A PETSc matrix (``MatNest``) that is compatible with ``a``.
 
     """
-    return _cpp.fem.petsc.create_matrix_nest(a)
+    _a = [[None if form is None else form._cpp_object for form in arow] for arow in a]
+    return _cpp.fem.petsc.create_matrix_nest(_a)
 
 
 # -- Vector assembly ---------------------------------------------------------
 
 @functools.singledispatch
 def assemble_vector(L: typing.Any, constants=None, coeffs=None) -> PETSc.Vec:
-    return _assemble_vector_form(L, constants, coeffs)
-
-
-@assemble_vector.register(FormMetaClass)
-def _assemble_vector_form(L: form_types, constants=None, coeffs=None) -> PETSc.Vec:
     """Assemble linear form into a new PETSc vector.
 
     Note:
@@ -181,15 +187,15 @@ def _assemble_vector_form(L: form_types, constants=None, coeffs=None) -> PETSc.V
         An assembled vector.
 
     """
-    b = la.create_petsc_vector(L.function_spaces[0].dofmap.index_map,
-                               L.function_spaces[0].dofmap.index_map_bs)
+    b = create_petsc_vector(L.function_spaces[0].dofmap.index_map,
+                            L.function_spaces[0].dofmap.index_map_bs)
     with b.localForm() as b_local:
         assemble._assemble_vector_array(b_local.array_w, L, constants, coeffs)
     return b
 
 
 @assemble_vector.register(PETSc.Vec)
-def _assemble_vector_vec(b: PETSc.Vec, L: form_types, constants=None, coeffs=None) -> PETSc.Vec:
+def _assemble_vector_vec(b: PETSc.Vec, L: Form, constants=None, coeffs=None) -> PETSc.Vec:
     """Assemble linear form into an existing PETSc vector.
 
     Note:
@@ -199,7 +205,7 @@ def _assemble_vector_vec(b: PETSc.Vec, L: form_types, constants=None, coeffs=Non
 
     Args:
         b: Vector to assemble the contribution of the linear form into.
-        L: A linear form to assemble into `b`.
+        L: A linear form to assemble into ``b``.
 
     Returns:
         An assembled vector.
@@ -212,12 +218,7 @@ def _assemble_vector_vec(b: PETSc.Vec, L: form_types, constants=None, coeffs=Non
 
 @functools.singledispatch
 def assemble_vector_nest(L: typing.Any, constants=None, coeffs=None) -> PETSc.Vec:
-    return _assemble_vector_nest_forms(L, constants, coeffs)
-
-
-@assemble_vector_nest.register(list)
-def _assemble_vector_nest_forms(L: typing.List[form_types], constants=None, coeffs=None) -> PETSc.Vec:
-    """Assemble linear forms into a new nested PETSc (VecNest) vector.
+    """Assemble linear forms into a new nested PETSc (``VecNest``) vector.
     The returned vector is not finalised, i.e. ghost values are not
     accumulated on the owning processes.
 
@@ -232,8 +233,8 @@ def _assemble_vector_nest_forms(L: typing.List[form_types], constants=None, coef
 
 
 @assemble_vector_nest.register
-def _assemble_vector_nest_vec(b: PETSc.Vec, L: typing.List[form_types], constants=None, coeffs=None) -> PETSc.Vec:
-    """Assemble linear forms into a nested PETSc (VecNest) vector. The
+def _assemble_vector_nest_vec(b: PETSc.Vec, L: typing.List[Form], constants=None, coeffs=None) -> PETSc.Vec:
+    """Assemble linear forms into a nested PETSc (``VecNest``) vector. The
     vector is not zeroed before assembly and it is not finalised, i.e.
     ghost values are not accumulated on the owning processes.
 
@@ -248,24 +249,13 @@ def _assemble_vector_nest_vec(b: PETSc.Vec, L: typing.List[form_types], constant
 
 # FIXME: Revise this interface
 @functools.singledispatch
-def assemble_vector_block(L: typing.Any,
-                          a: typing.List[typing.List[form_types]],
-                          bcs: typing.List[DirichletBCMetaClass] = [],
+def assemble_vector_block(L: typing.List[Form],
+                          a: typing.List[typing.List[Form]],
+                          bcs: typing.List[DirichletBC] = [],
                           x0: typing.Optional[PETSc.Vec] = None,
                           scale: float = 1.0,
                           constants_L=None, coeffs_L=None,
                           constants_a=None, coeffs_a=None) -> PETSc.Vec:
-    return _assemble_vector_block_form(L, a, bcs, x0, scale, constants_L, coeffs_L, constants_a, coeffs_a)
-
-
-@assemble_vector_block.register(list)
-def _assemble_vector_block_form(L: typing.List[form_types],
-                                a: typing.List[typing.List[form_types]],
-                                bcs: typing.List[DirichletBCMetaClass] = [],
-                                x0: typing.Optional[PETSc.Vec] = None,
-                                scale: float = 1.0,
-                                constants_L=None, coeffs_L=None,
-                                constants_a=None, coeffs_a=None) -> PETSc.Vec:
     """Assemble linear forms into a monolithic vector. The vector is not
     finalised, i.e. ghost values are not accumulated.
 
@@ -281,9 +271,9 @@ def _assemble_vector_block_form(L: typing.List[form_types],
 
 @assemble_vector_block.register
 def _assemble_vector_block_vec(b: PETSc.Vec,
-                               L: typing.List[form_types],
-                               a: typing.List[typing.List[form_types]],
-                               bcs: typing.List[DirichletBCMetaClass] = [],
+                               L: typing.List[Form],
+                               a: typing.List[typing.List[Form]],
+                               bcs: typing.List[DirichletBC] = [],
                                x0: typing.Optional[PETSc.Vec] = None,
                                scale: float = 1.0,
                                constants_L=None, coeffs_L=None,
@@ -302,26 +292,28 @@ def _assemble_vector_block_vec(b: PETSc.Vec,
         x0_local = []
         x0_sub = [None] * len(maps)
 
-    constants_L = [form and _pack_constants(form) for form in L] if constants_L is None else constants_L
+    constants_L = [form and _pack_constants(form._cpp_object) for form in L] if constants_L is None else constants_L
     coeffs_L = [{} if form is None else _pack_coefficients(
-        form) for form in L] if coeffs_L is None else coeffs_L
-    constants_a = [[form and _pack_constants(form) for form in forms]
+        form._cpp_object) for form in L] if coeffs_L is None else coeffs_L
+    constants_a = [[form and _pack_constants(form._cpp_object) for form in forms]
                    for forms in a] if constants_a is None else constants_a
     coeffs_a = [[{} if form is None else _pack_coefficients(
-        form) for form in forms] for forms in a] if coeffs_a is None else coeffs_a
+        form._cpp_object) for form in forms] for forms in a] if coeffs_a is None else coeffs_a
 
-    bcs1 = _bcs_by_block(_extract_spaces(a, 1), bcs)
+    _bcs = [bc._cpp_object for bc in bcs]
+    bcs1 = _bcs_by_block(_extract_spaces(a, 1), _bcs)
     b_local = _cpp.la.petsc.get_local_vectors(b, maps)
     for b_sub, L_sub, a_sub, const_L, coeff_L, const_a, coeff_a in zip(b_local, L, a,
                                                                        constants_L, coeffs_L,
                                                                        constants_a, coeffs_a):
-        _cpp.fem.assemble_vector(b_sub, L_sub, const_L, coeff_L)
-        _cpp.fem.apply_lifting(b_sub, a_sub, const_a, coeff_a, bcs1, x0_local, scale)
+        _cpp.fem.assemble_vector(b_sub, L_sub._cpp_object, const_L, coeff_L)
+        _a_sub = [None if form is None else form._cpp_object for form in a_sub]
+        _cpp.fem.apply_lifting(b_sub, _a_sub, const_a, coeff_a, bcs1, x0_local, scale)
 
     _cpp.la.petsc.scatter_local_vectors(b, b_local, maps)
     b.ghostUpdate(addv=PETSc.InsertMode.ADD, mode=PETSc.ScatterMode.REVERSE)
 
-    bcs0 = _bcs_by_block(_extract_spaces(L), bcs)
+    bcs0 = _bcs_by_block(_extract_spaces(L), _bcs)
     offset = 0
     b_array = b.getArray(readonly=False)
     for submap, bc, _x0 in zip(maps, bcs0, x0_sub):
@@ -334,76 +326,108 @@ def _assemble_vector_block_vec(b: PETSc.Vec,
 
 # -- Matrix assembly ---------------------------------------------------------
 @functools.singledispatch
-def assemble_matrix(a: typing.Any, bcs: typing.List[DirichletBCMetaClass] = [],
-                    diagonal: float = 1.0,
-                    constants=None, coeffs=None) -> PETSc.Mat:
-    return _assemble_matrix_form(a, bcs, diagonal, constants, coeffs)
-
-
-@assemble_matrix.register(FormMetaClass)
-def _assemble_matrix_form(a: form_types, bcs: typing.List[DirichletBCMetaClass] = [],
-                          diagonal: float = 1.0,
-                          constants=None, coeffs=None) -> PETSc.Mat:
+def assemble_matrix(a: typing.Any, bcs: typing.List[DirichletBC] = [],
+                    diagonal: float = 1.0, constants=None, coeffs=None):
     """Assemble bilinear form into a matrix. The returned matrix is not
     finalised, i.e. ghost values are not accumulated.
 
+    Note:
+        The returned matrix is not 'assembled', i.e. ghost contributions
+        have not been communicated.
+
+    Args:
+        a: Bilinear form to assembled into a matrix.
+        bc: Dirichlet boundary conditions applied to the system.
+        diagonal: Value to set on matrix diagonal for Dirichlet boundary
+            condition constrained degrees-of-freedom.
+        constants: Constants appearing the in the form.
+        coeffs: Coefficients appearing the in the form.
+
+    Returns:
+        Matrix representing the bilinear form.
+
     """
-    A = _cpp.fem.petsc.create_matrix(a)
-    _assemble_matrix_mat(A, a, bcs, diagonal, constants, coeffs)
+    A = _cpp.fem.petsc.create_matrix(a._cpp_object)
+    assemble_matrix_mat(A, a, bcs, diagonal, constants, coeffs)
     return A
 
 
 @assemble_matrix.register
-def _assemble_matrix_mat(A: PETSc.Mat, a: form_types, bcs: typing.List[DirichletBCMetaClass] = [],
-                         diagonal: float = 1.0, constants=None, coeffs=None) -> PETSc.Mat:
+def assemble_matrix_mat(A: PETSc.Mat, a: Form, bcs: typing.List[DirichletBC] = [],
+                        diagonal: float = 1.0, constants=None, coeffs=None) -> PETSc.Mat:
     """Assemble bilinear form into a matrix. The returned matrix is not
     finalised, i.e. ghost values are not accumulated.
 
     """
-    constants = _pack_constants(a) if constants is None else constants
-    coeffs = _pack_coefficients(a) if coeffs is None else coeffs
-    _cpp.fem.petsc.assemble_matrix(A, a, constants, coeffs, bcs)
+    constants = _pack_constants(a._cpp_object) if constants is None else constants
+    coeffs = _pack_coefficients(a._cpp_object) if coeffs is None else coeffs
+    _bcs = [bc._cpp_object for bc in bcs]
+    _cpp.fem.petsc.assemble_matrix(A, a._cpp_object, constants, coeffs, _bcs)
     if a.function_spaces[0] is a.function_spaces[1]:
         A.assemblyBegin(PETSc.Mat.AssemblyType.FLUSH)
         A.assemblyEnd(PETSc.Mat.AssemblyType.FLUSH)
-        _cpp.fem.petsc.insert_diagonal(A, a.function_spaces[0], bcs, diagonal)
+        _cpp.fem.petsc.insert_diagonal(A, a.function_spaces[0], _bcs, diagonal)
     return A
 
 
 # FIXME: Revise this interface
 @functools.singledispatch
-def assemble_matrix_nest(a: typing.Any,
-                         bcs: typing.List[DirichletBCMetaClass] = [], mat_types=[],
-                         diagonal: float = 1.0,
-                         constants=None, coeffs=None) -> PETSc.Mat:
-    return _assemble_matrix_nest_form(a, bcs, diagonal, constants, coeffs)
+def assemble_matrix_nest(a: typing.List[typing.List[Form]],
+                         bcs: typing.List[DirichletBC] = [], mat_types=[],
+                         diagonal: float = 1.0, constants=None, coeffs=None) -> PETSc.Mat:
+    """Create a nested matrix and assembled bilinear forms into the matrix.
 
+    Args:
+        a: Rectangular (list-of-lists) array for bilinear forms.
+        bcs: Dirichlet boundary conditions.
+        mat_types: PETSc matrix type for each matrix block.
+        diagonal: Value to set on matrix diagonal for Dirichlet boundary
+            condition constrained degrees-of-freedom.
+        constants: Constants appearing the in the form.
+        coeffs: Coefficients appearing the in the form.
 
-@assemble_matrix_nest.register(list)
-def _assemble_matrix_nest_form(a: typing.List[typing.List[form_types]],
-                               bcs: typing.List[DirichletBCMetaClass] = [], mat_types=[],
-                               diagonal: float = 1.0,
-                               constants=None, coeffs=None) -> PETSc.Mat:
-    """Assemble bilinear forms into matrix"""
-    A = _cpp.fem.petsc.create_matrix_nest(a, mat_types)
+    Returns:
+        PETSc matrix (``MatNest``) representing the block of bilinear
+        forms.
+
+    """
+    _a = [[None if form is None else form._cpp_object for form in arow] for arow in a]
+    A = _cpp.fem.petsc.create_matrix_nest(_a, mat_types)
     _assemble_matrix_nest_mat(A, a, bcs, diagonal, constants, coeffs)
     return A
 
 
 @assemble_matrix_nest.register
-def _assemble_matrix_nest_mat(A: PETSc.Mat, a: typing.List[typing.List[form_types]],
-                              bcs: typing.List[DirichletBCMetaClass] = [], diagonal: float = 1.0,
+def _assemble_matrix_nest_mat(A: PETSc.Mat, a: typing.List[typing.List[Form]],
+                              bcs: typing.List[DirichletBC] = [], diagonal: float = 1.0,
                               constants=None, coeffs=None) -> PETSc.Mat:
-    """Assemble bilinear forms into matrix"""
-    constants = [[form and _pack_constants(form) for form in forms]
+    """Assemble bilinear forms into a nested matrix
+
+    Args:
+        A: PETSc ``MatNest`` matrix. Matrix must have been correctly
+            initialized for the bilinear forms.
+        a: Rectangular (list-of-lists) array for bilinear forms.
+        bcs: Dirichlet boundary conditions.
+        mat_types: PETSc matrix type for each matrix block.
+        diagonal: Value to set on matrix diagonal for Dirichlet boundary
+            condition constrained degrees-of-freedom.
+        constants: Constants appearing the in the form.
+        coeffs: Coefficients appearing the in the form.
+
+    Returns:
+        PETSc matrix (``MatNest``) representing the block of bilinear
+        forms.
+
+    """
+    constants = [[form and _pack_constants(form._cpp_object) for form in forms]
                  for forms in a] if constants is None else constants
     coeffs = [[{} if form is None else _pack_coefficients(
-        form) for form in forms] for forms in a] if coeffs is None else coeffs
+        form._cpp_object) for form in forms] for forms in a] if coeffs is None else coeffs
     for i, (a_row, const_row, coeff_row) in enumerate(zip(a, constants, coeffs)):
         for j, (a_block, const, coeff) in enumerate(zip(a_row, const_row, coeff_row)):
             if a_block is not None:
                 Asub = A.getNestSubMatrix(i, j)
-                _assemble_matrix_mat(Asub, a_block, bcs, diagonal, const, coeff)
+                assemble_matrix_mat(Asub, a_block, bcs, diagonal, const, coeff)
             elif i == j:
                 for bc in bcs:
                     row_forms = [row_form for row_form in a_row if row_form is not None]
@@ -417,44 +441,38 @@ def _assemble_matrix_nest_mat(A: PETSc.Mat, a: typing.List[typing.List[form_type
 
 # FIXME: Revise this interface
 @functools.singledispatch
-def assemble_matrix_block(a: typing.Any,
-                          bcs: typing.List[DirichletBCMetaClass] = [],
+def assemble_matrix_block(a: typing.List[typing.List[Form]],
+                          bcs: typing.List[DirichletBC] = [],
                           diagonal: float = 1.0,
-                          constants=None, coeffs=None) -> PETSc.Mat:
-    return _assemble_matrix_block_form(a, bcs, diagonal, constants, coeffs)
-
-
-@assemble_matrix_block.register(list)
-def _assemble_matrix_block_form(a: typing.List[typing.List[form_types]],
-                                bcs: typing.List[DirichletBCMetaClass] = [],
-                                diagonal: float = 1.0,
-                                constants=None, coeffs=None) -> PETSc.Mat:
-    """Assemble bilinear forms into matrix"""
-    A = _cpp.fem.petsc.create_matrix_block(a)
+                          constants=None, coeffs=None) -> PETSc.Mat:  # type: ignore
+    """Assemble bilinear forms into a blocked matrix."""
+    _a = [[None if form is None else form._cpp_object for form in arow] for arow in a]
+    A = _cpp.fem.petsc.create_matrix_block(_a)
     return _assemble_matrix_block_mat(A, a, bcs, diagonal, constants, coeffs)
 
 
 @assemble_matrix_block.register
-def _assemble_matrix_block_mat(A: PETSc.Mat, a: typing.List[typing.List[form_types]],
-                               bcs: typing.List[DirichletBCMetaClass] = [], diagonal: float = 1.0,
+def _assemble_matrix_block_mat(A: PETSc.Mat, a: typing.List[typing.List[Form]],
+                               bcs: typing.List[DirichletBC] = [], diagonal: float = 1.0,
                                constants=None, coeffs=None) -> PETSc.Mat:
-    """Assemble bilinear forms into matrix"""
+    """Assemble bilinear forms into a blocked matrix."""
 
-    constants = [[form and _pack_constants(form) for form in forms]
+    constants = [[form and _pack_constants(form._cpp_object) for form in forms]
                  for forms in a] if constants is None else constants
     coeffs = [[{} if form is None else _pack_coefficients(
-        form) for form in forms] for forms in a] if coeffs is None else coeffs
+        form._cpp_object) for form in forms] for forms in a] if coeffs is None else coeffs
 
     V = _extract_function_spaces(a)
     is_rows = _cpp.la.petsc.create_index_sets([(Vsub.dofmap.index_map, Vsub.dofmap.index_map_bs) for Vsub in V[0]])
     is_cols = _cpp.la.petsc.create_index_sets([(Vsub.dofmap.index_map, Vsub.dofmap.index_map_bs) for Vsub in V[1]])
 
     # Assemble form
+    _bcs = [bc._cpp_object for bc in bcs]
     for i, a_row in enumerate(a):
         for j, a_sub in enumerate(a_row):
             if a_sub is not None:
                 Asub = A.getLocalSubMatrix(is_rows[i], is_cols[j])
-                _cpp.fem.petsc.assemble_matrix(Asub, a_sub, constants[i][j], coeffs[i][j], bcs, True)
+                _cpp.fem.petsc.assemble_matrix(Asub, a_sub._cpp_object, constants[i][j], coeffs[i][j], _bcs, True)
                 A.restoreLocalSubMatrix(is_rows[i], is_cols[j], Asub)
             elif i == j:
                 for bc in bcs:
@@ -474,7 +492,7 @@ def _assemble_matrix_block_mat(A: PETSc.Mat, a: typing.List[typing.List[form_typ
             if a_sub is not None:
                 Asub = A.getLocalSubMatrix(is_rows[i], is_cols[j])
                 if a_sub.function_spaces[0] is a_sub.function_spaces[1]:
-                    _cpp.fem.petsc.insert_diagonal(Asub, a_sub.function_spaces[0], bcs, diagonal)
+                    _cpp.fem.petsc.insert_diagonal(Asub, a_sub.function_spaces[0], _bcs, diagonal)
                 A.restoreLocalSubMatrix(is_rows[i], is_cols[j], Asub)
 
     return A
@@ -482,8 +500,8 @@ def _assemble_matrix_block_mat(A: PETSc.Mat, a: typing.List[typing.List[form_typ
 
 # -- Modifiers for Dirichlet conditions ---------------------------------------
 
-def apply_lifting(b: PETSc.Vec, a: typing.List[form_types],
-                  bcs: typing.List[typing.List[DirichletBCMetaClass]],
+def apply_lifting(b: PETSc.Vec, a: typing.List[Form],
+                  bcs: typing.List[typing.List[DirichletBC]],
                   x0: typing.List[PETSc.Vec] = [],
                   scale: float = 1.0, constants=None, coeffs=None) -> None:
     """Apply the function :func:`dolfinx.fem.apply_lifting` to a PETSc Vector."""
@@ -494,8 +512,8 @@ def apply_lifting(b: PETSc.Vec, a: typing.List[form_types],
         assemble.apply_lifting(b_local.array_w, a, bcs, x0_r, scale, constants, coeffs)
 
 
-def apply_lifting_nest(b: PETSc.Vec, a: typing.List[typing.List[form_types]],
-                       bcs: typing.List[DirichletBCMetaClass],
+def apply_lifting_nest(b: PETSc.Vec, a: typing.List[typing.List[Form]],
+                       bcs: typing.List[DirichletBC],
                        x0: typing.Optional[PETSc.Vec] = None,
                        scale: float = 1.0, constants=None, coeffs=None) -> PETSc.Vec:
     """Apply the function :func:`dolfinx.fem.apply_lifting` to each sub-vector in a nested PETSc Vector."""
@@ -503,16 +521,16 @@ def apply_lifting_nest(b: PETSc.Vec, a: typing.List[typing.List[form_types]],
     x0 = [] if x0 is None else x0.getNestSubVecs()
     bcs1 = _bcs_by_block(_extract_spaces(a, 1), bcs)
 
-    constants = [[form and _pack_constants(form) for form in forms]
+    constants = [[form and _pack_constants(form._cpp_object) for form in forms]
                  for forms in a] if constants is None else constants
     coeffs = [[{} if form is None else _pack_coefficients(
-        form) for form in forms] for forms in a] if coeffs is None else coeffs
+        form._cpp_object) for form in forms] for forms in a] if coeffs is None else coeffs
     for b_sub, a_sub, const, coeff in zip(b.getNestSubVecs(), a, constants, coeffs):
         apply_lifting(b_sub, a_sub, bcs1, x0, scale, const, coeff)
     return b
 
 
-def set_bc(b: PETSc.Vec, bcs: typing.List[DirichletBCMetaClass],
+def set_bc(b: PETSc.Vec, bcs: typing.List[DirichletBC],
            x0: typing.Optional[PETSc.Vec] = None, scale: float = 1.0) -> None:
     """Apply the function :func:`dolfinx.fem.set_bc` to a PETSc Vector."""
     if x0 is not None:
@@ -520,7 +538,7 @@ def set_bc(b: PETSc.Vec, bcs: typing.List[DirichletBCMetaClass],
     assemble.set_bc(b.array_w, bcs, x0, scale)
 
 
-def set_bc_nest(b: PETSc.Vec, bcs: typing.List[typing.List[DirichletBCMetaClass]],
+def set_bc_nest(b: PETSc.Vec, bcs: typing.List[typing.List[DirichletBC]],
                 x0: typing.Optional[PETSc.Vec] = None, scale: float = 1.0) -> None:
     """Apply the function :func:`dolfinx.fem.set_bc` to each sub-vector of a nested PETSc Vector."""
     _b = b.getNestSubVecs()
@@ -535,13 +553,18 @@ class LinearProblem:
 
     """
 
-    def __init__(self, a: ufl.Form, L: ufl.Form, bcs: typing.List[DirichletBCMetaClass] = [],
-                 u: typing.Optional[_Function] = None, petsc_options={}, form_compiler_options={}, jit_options={}):
+    def __init__(self, a: ufl.Form, L: ufl.Form, bcs: typing.List[DirichletBC] = [],
+                 u: typing.Optional[_Function] = None,
+                 petsc_options: typing.Optional[dict] = None,
+                 form_compiler_options: typing.Optional[dict] = None,
+                 jit_options: typing.Optional[dict] = None):
         """Initialize solver for a linear variational problem.
 
         Args:
-            a: A bilinear UFL form, the left hand side of the variational problem.
-            L: A linear UFL form, the right hand side of the variational problem.
+            a: A bilinear UFL form, the left hand side of the
+                variational problem.
+            L: A linear UFL form, the right hand side of the variational
+                problem.
             bcs: A list of Dirichlet boundary conditions.
             u: The solution function. It will be created if not provided.
             petsc_options: Options that are passed to the linear
@@ -558,14 +581,12 @@ class LinearProblem:
 
         Example::
 
-            problem = LinearProblem(a, L, [bc0, bc1],
-                                    petsc_options={"ksp_type": "preonly",
-                                                   "pc_type": "lu",
-                                                   "pc_factor_mat_solver_type": "mumps"})
+            problem = LinearProblem(a, L, [bc0, bc1], petsc_options={"ksp_type": "preonly",
+                                                                     "pc_type": "lu",
+                                                                     "pc_factor_mat_solver_type": "mumps"})
         """
         self._a = _create_form(a, form_compiler_options=form_compiler_options, jit_options=jit_options)
         self._A = create_matrix(self._a)
-
         self._L = _create_form(L, form_compiler_options=form_compiler_options, jit_options=jit_options)
         self._b = create_vector(self._L)
 
@@ -577,7 +598,7 @@ class LinearProblem:
         else:
             self.u = u
 
-        self._x = _cpp.la.petsc.create_vector_wrap(self.u.x)
+        self._x = la.create_petsc_vector_wrap(self.u.x)
         self.bcs = bcs
 
         self._solver = PETSc.KSP().create(self.u.function_space.mesh.comm)
@@ -590,8 +611,9 @@ class LinearProblem:
         # Set PETSc options
         opts = PETSc.Options()
         opts.prefixPush(problem_prefix)
-        for k, v in petsc_options.items():
-            opts[k] = v
+        if petsc_options is not None:
+            for k, v in petsc_options.items():
+                opts[k] = v
         opts.prefixPop()
         self._solver.setFromOptions()
 
@@ -601,12 +623,18 @@ class LinearProblem:
         self._b.setOptionsPrefix(problem_prefix)
         self._b.setFromOptions()
 
+    def __del__(self):
+        self._solver.destroy()
+        self._A.destroy()
+        self._b.destroy()
+        self._x.destroy()
+
     def solve(self) -> _Function:
         """Solve the problem."""
 
         # Assemble lhs
         self._A.zeroEntries()
-        _assemble_matrix_mat(self._A, self._a, bcs=self.bcs)
+        assemble_matrix_mat(self._A, self._a, bcs=self.bcs)
         self._A.assemble()
 
         # Assemble rhs
@@ -626,12 +654,12 @@ class LinearProblem:
         return self.u
 
     @property
-    def L(self) -> FormMetaClass:
+    def L(self) -> Form:
         """The compiled linear form"""
         return self._L
 
     @property
-    def a(self) -> FormMetaClass:
+    def a(self) -> Form:
         """The compiled bilinear form"""
         return self._a
 
@@ -658,9 +686,10 @@ class NonlinearProblem:
 
     """
 
-    def __init__(self, F: ufl.form.Form, u: _Function, bcs: typing.List[DirichletBCMetaClass] = [],
-                 J: ufl.form.Form = None, form_compiler_options={}, jit_options={}):
-        """Initialize solver for solving a non-linear problem using Newton's method, :math:`dF/du(u) du = -F(u)`.
+    def __init__(self, F: ufl.form.Form, u: _Function, bcs: typing.List[DirichletBC] = [],
+                 J: ufl.form.Form = None, form_compiler_options: typing.Optional[dict] = None,
+                 jit_options: typing.Optional[dict] = None):
+        """Initialize solver for solving a non-linear problem using Newton's method, :math:`(dF/du)(u) du = -F(u)`.
 
         Args:
             F: The PDE residual F(u, v)
@@ -694,16 +723,16 @@ class NonlinearProblem:
         self.bcs = bcs
 
     @property
-    def L(self) -> FormMetaClass:
+    def L(self) -> Form:
         """Compiled linear form (the residual form)"""
         return self._L
 
     @property
-    def a(self) -> FormMetaClass:
+    def a(self) -> Form:
         """Compiled bilinear form (the Jacobian form)"""
         return self._a
 
-    def form(self, x: PETSc.Vec):
+    def form(self, x: PETSc.Vec) -> None:
         """This function is called before the residual or Jacobian is
         computed. This is usually used to update ghost values.
 
@@ -713,7 +742,7 @@ class NonlinearProblem:
         """
         x.ghostUpdate(addv=PETSc.InsertMode.INSERT, mode=PETSc.ScatterMode.FORWARD)
 
-    def F(self, x: PETSc.Vec, b: PETSc.Vec):
+    def F(self, x: PETSc.Vec, b: PETSc.Vec) -> None:
         """Assemble the residual F into the vector b.
 
         Args:
@@ -731,7 +760,7 @@ class NonlinearProblem:
         b.ghostUpdate(addv=PETSc.InsertMode.ADD, mode=PETSc.ScatterMode.REVERSE)
         set_bc(b, self.bcs, x, -1.0)
 
-    def J(self, x: PETSc.Vec, A: PETSc.Mat):
+    def J(self, x: PETSc.Vec, A: PETSc.Mat) -> None:
         """Assemble the Jacobian matrix.
 
         Args:
@@ -739,19 +768,19 @@ class NonlinearProblem:
 
         """
         A.zeroEntries()
-        _assemble_matrix_mat(A, self._a, self.bcs)
+        assemble_matrix_mat(A, self._a, self.bcs)
         A.assemble()
 
 
 def load_petsc_lib(loader: typing.Callable[[str], typing.Any]) -> typing.Any:
-    """
-    Load PETSc shared library using loader callable, e.g. ctypes.CDLL.
+    """Load PETSc shared library using loader callable, e.g. ctypes.CDLL.
 
     Args:
         loader: A callable that accepts a library path and returns a wrapped library.
 
     Returns:
         A wrapped library of the type returned by the callable.
+
     """
     petsc_lib = None
 
