@@ -11,13 +11,13 @@ import os
 from pathlib import Path
 from typing import Optional
 
+from mpi4py import MPI
+
 import ffcx
 import ffcx.codegeneration.jit
 import ufl
 
-from mpi4py import MPI
-
-__all__ = ["ffcx_jit", "get_options"]
+__all__ = ["ffcx_jit", "get_options", "mpi_jit_decorator"]
 
 DOLFINX_DEFAULT_JIT_OPTIONS = {
     "cache_dir":
@@ -105,14 +105,14 @@ def _load_options():
         with open(user_config_file) as f:
             user_options = json.load(f)
     except FileNotFoundError:
-        user_options = {}
+        user_options = dict()
 
     pwd_config_file = Path.cwd().joinpath("dolfinx_jit_options.json")
     try:
         with open(pwd_config_file) as f:
             pwd_options = json.load(f)
     except FileNotFoundError:
-        pwd_options = {}
+        pwd_options = dict()
 
     return (user_options, pwd_options)
 
@@ -121,17 +121,17 @@ def get_options(priority_options: Optional[dict] = None) -> dict:
     """Return a copy of the merged JIT option values for DOLFINx.
 
     Args:
-        priority_options: take priority over all other option values (see notes)
+        priority_options: Take priority over all other option values
+            (see notes).
 
     Returns:
-        dict: merged option values
+        dict: Merged option values.
 
-    Notes:
-        See ffcx_jit for user facing documentation.
+    Note:
+        See :func:`ffcx_jit` for user facing documentation.
 
     """
-    options = {}
-
+    options = dict()
     for param, (value, _) in DOLFINX_DEFAULT_JIT_OPTIONS.items():
         options[param] = value
 
@@ -149,24 +149,25 @@ def get_options(priority_options: Optional[dict] = None) -> dict:
 
 
 @mpi_jit_decorator
-def ffcx_jit(ufl_object, form_compiler_options={}, jit_options={}):
+def ffcx_jit(ufl_object, form_compiler_options: Optional[dict] = None,
+             jit_options: Optional[dict] = None):
     """Compile UFL object with FFCx and CFFI.
 
     Args:
-        ufl_object: Object to compile, e.g. ufl.Form
+        ufl_object: Object to compile, e.g. ``ufl.Form``.
         form_compiler_options: Options used in FFCx compilation of
-            this form. Run `ffcx --help` at the commandline to see all
-            available options. Takes priority over all other option
+            this form. Run ``ffcx --help`` at the command line to see
+            all available options. Takes priority over all other option
             values.
         jit_options: Options used in CFFI JIT compilation of C code
-            generated by FFCx. See `python/dolfinx/jit.py` for all
-            available options. Takes priority over all other
-            option values.
+            generated by FFCx. See ``python/dolfinx/jit.py`` for all
+            available options. Takes priority over all other option
+            values.
 
     Returns:
         (compiled object, module, (header code, implementation code))
 
-    Notes:
+    Note:
         Priority ordering of options controlling DOLFINx JIT
         compilation from highest to lowest is:
 
@@ -195,7 +196,6 @@ def ffcx_jit(ufl_object, form_compiler_options={}, jit_options={}):
             **{ "cffi_extra_compile_args": ["-O2", "-march=native" ],  "cffi_verbose": True }**
 
     """
-    # Prepare form compiler options with priority options
     p_ffcx = ffcx.get_options(form_compiler_options)
     p_jit = get_options(jit_options)
 
@@ -205,8 +205,7 @@ def ffcx_jit(ufl_object, form_compiler_options={}, jit_options={}):
     elif isinstance(ufl_object, ufl.FiniteElementBase):
         r = ffcx.codegeneration.jit.compile_elements([ufl_object], options=p_ffcx, **p_jit)
     elif isinstance(ufl_object, ufl.Mesh):
-        r = ffcx.codegeneration.jit.compile_coordinate_maps(
-            [ufl_object], options=p_ffcx, **p_jit)
+        r = ffcx.codegeneration.jit.compile_coordinate_maps([ufl_object], options=p_ffcx, **p_jit)
     elif isinstance(ufl_object, tuple) and isinstance(ufl_object[0], ufl.core.expr.Expr):
         r = ffcx.codegeneration.jit.compile_expressions([ufl_object], options=p_ffcx, **p_jit)
     else:
