@@ -4,7 +4,7 @@
 #
 # SPDX-License-Identifier:    LGPL-3.0-or-later
 
-"""Logging module"""
+"""Logging module."""
 
 # Import pybind11 wrapped code intp dolfinx.log
 from dolfinx.cpp.log import (LogLevel, get_log_level, log,  # noqa
