@@ -42,14 +42,16 @@ void declare_adjacency_list(py::module& m, std::string type)
              std::shared_ptr<dolfinx::graph::AdjacencyList<T>>>(
       m, pyclass_name.c_str(), "Adjacency List")
       .def(py::init(
-          [](const py::array_t<T, py::array::c_style>& adj)
-          {
-            if (adj.ndim() > 2)
-              throw std::runtime_error("Incorrect array dimension.");
-            const std::size_t dim = adj.ndim() < 2 ? 1 : adj.shape(1);
-            std::vector<T> data(adj.data(), adj.data() + adj.size());
-            return dolfinx::graph::regular_adjacency_list(std::move(data), dim);
-          }))
+               [](const py::array_t<T, py::array::c_style>& adj)
+               {
+                 if (adj.ndim() > 2)
+                   throw std::runtime_error("Incorrect array dimension.");
+                 const std::size_t dim = adj.ndim() < 2 ? 1 : adj.shape(1);
+                 std::vector<T> data(adj.data(), adj.data() + adj.size());
+                 return dolfinx::graph::regular_adjacency_list(std::move(data),
+                                                               dim);
+               }),
+           py::arg("adj"))
       .def(py::init(
                [](const py::array_t<T, py::array::c_style>& array,
                   const py::array_t<std::int32_t, py::array::c_style>& displ)
@@ -65,10 +67,10 @@ void declare_adjacency_list(py::module& m, std::string type)
           "links",
           [](const dolfinx::graph::AdjacencyList<T>& self, int i)
           {
-            xtl::span<const T> link = self.links(i);
+            std::span<const T> link = self.links(i);
             return py::array_t<T>(link.size(), link.data(), py::cast(self));
           },
-          "Links (edges) of a node")
+          py::arg("i"), "Links (edges) of a node")
       .def_property_readonly("array",
                              [](const dolfinx::graph::AdjacencyList<T>& self)
                              {
@@ -142,6 +144,6 @@ void graph(py::module& m)
       py::arg("suppress_output") = true, "KaHIP graph partitioner");
 #endif
 
-  m.def("reorder_gps", &dolfinx::graph::reorder_gps);
+  m.def("reorder_gps", &dolfinx::graph::reorder_gps, py::arg("graph"));
 }
 } // namespace dolfinx_wrappers
