@@ -6,13 +6,18 @@
 
 from pathlib import Path
 
+from mpi4py import MPI
+from petsc4py import PETSc
+
 import numpy as np
 import pytest
 
+import basix
 import ufl
-from dolfinx.fem import (Function, FunctionSpace, VectorFunctionSpace,
-                         assemble_scalar, dirichletbc, form,
-                         locate_dofs_topological)
+from basix.ufl import element, mixed_element
+from dolfinx import default_real_type
+from dolfinx.fem import (Function, FunctionSpace, assemble_scalar, dirichletbc,
+                         form, locate_dofs_topological)
 from dolfinx.fem.petsc import (apply_lifting, assemble_matrix, assemble_vector,
                                set_bc)
 from dolfinx.io import XDMFFile
@@ -22,9 +27,6 @@ from dolfinx.mesh import (CellType, create_rectangle, create_unit_cube,
 from ufl import (CellDiameter, FacetNormal, SpatialCoordinate, TestFunction,
                  TrialFunction, avg, div, ds, dS, dx, grad, inner, jump)
 
-from mpi4py import MPI
-from petsc4py import PETSc
-
 
 def run_scalar_test(mesh, V, degree):
     """Manufactured Poisson problem, solving u = x[1]**p, where p is the
@@ -80,7 +82,11 @@ def run_scalar_test(mesh, V, degree):
     M = (u_exact - uh)**2 * dx
     M = form(M)
     error = mesh.comm.allreduce(assemble_scalar(M), op=MPI.SUM)
-    assert np.absolute(error) < 1.0e-14
+    assert np.abs(error) < 1.0e-12
+
+    solver.destroy()
+    A.destroy()
+    b.destroy()
 
 
 def run_vector_test(mesh, V, degree):
@@ -117,7 +123,11 @@ def run_vector_test(mesh, V, degree):
     M = form(M)
 
     error = mesh.comm.allreduce(assemble_scalar(M), op=MPI.SUM)
-    assert np.absolute(error) < 1.0e-14
+    assert np.abs(error) < 1.0e-14
+
+    solver.destroy()
+    A.destroy()
+    b.destroy()
 
 
 def run_dg_test(mesh, V, degree):
@@ -187,7 +197,11 @@ def run_dg_test(mesh, V, degree):
     M = form(M)
 
     error = mesh.comm.allreduce(assemble_scalar(M), op=MPI.SUM)
-    assert np.absolute(error) < 1.0e-14
+    assert np.abs(error) < 1.0e-14
+
+    solver.destroy()
+    A.destroy()
+    b.destroy()
 
 
 @pytest.mark.parametrize("family", ["N1curl", "N2curl"])
@@ -204,8 +218,8 @@ def test_curl_curl_eigenvalue(family, order):
     mesh = create_rectangle(MPI.COMM_WORLD, [np.array([0.0, 0.0]),
                                              np.array([np.pi, np.pi])], [24, 24], CellType.triangle)
 
-    element = ufl.FiniteElement(family, ufl.triangle, order)
-    V = FunctionSpace(mesh, element)
+    e = element(family, basix.CellType.triangle, order)
+    V = FunctionSpace(mesh, e)
 
     u = ufl.TrialFunction(V)
     v = ufl.TestFunction(V)
@@ -251,8 +265,12 @@ def test_curl_curl_eigenvalue(family, order):
     eigenvalues_exact = np.array([1.0, 1.0, 2.0, 4.0, 4.0, 5.0, 5.0, 8.0, 9.0])
     assert np.isclose(eigenvalues_sorted[0:eigenvalues_exact.shape[0]], eigenvalues_exact, rtol=1E-2).all()
 
+    eps.destroy()
+    A.destroy()
+    B.destroy()
 
-@pytest.mark.skipif(np.issubdtype(PETSc.ScalarType, np.complexfloating),
+
+@pytest.mark.skipif(np.issubdtype(PETSc.ScalarType, np.complexfloating),  # type: ignore
                     reason="This test does not work in complex mode.")
 @pytest.mark.parametrize("family", ["HHJ", "Regge"])
 def test_biharmonic(family):
@@ -261,12 +279,12 @@ def test_biharmonic(family):
     Solved using rotated Regge or the Hellan-Herrmann-Johnson (HHJ) mixed
     finite element method in two-dimensions."""
     mesh = create_rectangle(MPI.COMM_WORLD, [np.array([0.0, 0.0]),
-                                             np.array([1.0, 1.0])], [32, 32], CellType.triangle)
+                                             np.array([1.0, 1.0])], [16, 16], CellType.triangle)
 
-    element = ufl.MixedElement([ufl.FiniteElement(family, ufl.triangle, 1),
-                                ufl.FiniteElement("Lagrange", ufl.triangle, 2)])
+    e = mixed_element([element(family, basix.CellType.triangle, 1),
+                       element(basix.ElementFamily.P, basix.CellType.triangle, 2)])
 
-    V = FunctionSpace(mesh, element)
+    V = FunctionSpace(mesh, e)
     sigma, u = ufl.TrialFunctions(V)
     tau, v = ufl.TestFunctions(V)
 
@@ -308,11 +326,11 @@ def test_biharmonic(family):
 
     V_1 = V.sub(1).collapse()[0]
     zero_u = Function(V_1)
-    zero_u.x.array[:] = 0.0
+    zero_u.x.array[:] = 0
 
     # Strong (Dirichlet) boundary condition
-    boundary_facets = locate_entities_boundary(
-        mesh, mesh.topology.dim - 1, lambda x: np.full(x.shape[1], True, dtype=bool))
+    boundary_facets = locate_entities_boundary(mesh, mesh.topology.dim - 1,
+                                               lambda x: np.full(x.shape[1], True, dtype=bool))
     boundary_dofs = locate_dofs_topological((V.sub(1), V_1), mesh.topology.dim - 1, boundary_facets)
 
     bcs = [dirichletbc(zero_u, boundary_dofs, V.sub(1))]
@@ -326,8 +344,7 @@ def test_biharmonic(family):
 
     # Solve
     solver = PETSc.KSP().create(MPI.COMM_WORLD)
-    solver.setTolerances(rtol=1e-10)
-    solver.setType("bcgs")
+    solver.setTolerances(rtol=1e-12)
     solver.setOperators(A)
 
     x_h = Function(V)
@@ -336,11 +353,10 @@ def test_biharmonic(family):
 
     # Recall that x_h has flattened indices
     u_error_numerator = np.sqrt(mesh.comm.allreduce(assemble_scalar(
-        form(inner(u_exact - x_h[4], u_exact - x_h[4]) * dx(mesh, metadata={"quadrature_degree": 5}))), op=MPI.SUM))
+        form(inner(u_exact - x_h[4], u_exact - x_h[4]) * dx(mesh, metadata={"quadrature_degree": 6}))), op=MPI.SUM))
     u_error_denominator = np.sqrt(mesh.comm.allreduce(assemble_scalar(
-        form(inner(u_exact, u_exact) * dx(mesh, metadata={"quadrature_degree": 5}))), op=MPI.SUM))
-
-    assert np.absolute(u_error_numerator / u_error_denominator) < 0.05
+        form(inner(u_exact, u_exact) * dx(mesh, metadata={"quadrature_degree": 6}))), op=MPI.SUM))
+    assert np.abs(u_error_numerator / u_error_denominator) < 0.05
 
     # Reconstruct tensor from flattened indices.
     # Apply inverse transform. In 2D we have S^{-1} = S.
@@ -352,12 +368,15 @@ def test_biharmonic(family):
         raise ValueError(f"Family {family} not supported.")
 
     sigma_error_numerator = np.sqrt(mesh.comm.allreduce(assemble_scalar(
-        form(inner(sigma_exact - sigma_h, sigma_exact - sigma_h) * dx(mesh, metadata={"quadrature_degree": 5}))),
+        form(inner(sigma_exact - sigma_h, sigma_exact - sigma_h) * dx(mesh, metadata={"quadrature_degree": 6}))),
         op=MPI.SUM))
     sigma_error_denominator = np.sqrt(mesh.comm.allreduce(assemble_scalar(
-        form(inner(sigma_exact, sigma_exact) * dx(mesh, metadata={"quadrature_degree": 5}))), op=MPI.SUM))
+        form(inner(sigma_exact, sigma_exact) * dx(mesh, metadata={"quadrature_degree": 6}))), op=MPI.SUM))
+    assert np.abs(sigma_error_numerator / sigma_error_denominator) < 0.05
 
-    assert np.absolute(sigma_error_numerator / sigma_error_denominator) < 0.005
+    solver.destroy()
+    A.destroy()
+    b.destroy()
 
 
 def get_mesh(cell_type, datadir):
@@ -383,6 +402,7 @@ parametrize_cell_types_hex = pytest.mark.parametrize("cell_type", [CellType.hexa
 
 
 # Run tests on all spaces in periodic table on triangles and tetrahedra
+@pytest.mark.skipif(default_real_type != np.float64, reason="float32 not supported yet")
 @parametrize_cell_types_simplex
 @pytest.mark.parametrize("family", ["Lagrange"])
 @pytest.mark.parametrize("degree", [2, 3, 4])
@@ -406,6 +426,7 @@ def test_P_simplex_built_in(family, degree, cell_type, datadir):
     run_scalar_test(mesh, V, degree)
 
 
+@pytest.mark.skipif(default_real_type != np.float64, reason="float32 not supported yet")
 @parametrize_cell_types_simplex
 @pytest.mark.parametrize("family", ["Lagrange"])
 @pytest.mark.parametrize("degree", [2, 3, 4])
@@ -413,10 +434,12 @@ def test_vector_P_simplex(family, degree, cell_type, datadir):
     if cell_type == CellType.tetrahedron and degree == 4:
         pytest.skip("Skip expensive test on tetrahedron")
     mesh = get_mesh(cell_type, datadir)
-    V = VectorFunctionSpace(mesh, (family, degree))
+    gdim = mesh.geometry.dim
+    V = FunctionSpace(mesh, (family, degree, (gdim,)))
     run_vector_test(mesh, V, degree)
 
 
+@pytest.mark.skipif(default_real_type != np.float64, reason="float32 not supported yet")
 @parametrize_cell_types_simplex
 @pytest.mark.parametrize("family", ["DG"])
 @pytest.mark.parametrize("degree", [2, 3])
@@ -426,6 +449,7 @@ def test_dP_simplex(family, degree, cell_type, datadir):
     run_dg_test(mesh, V, degree)
 
 
+@pytest.mark.skipif(default_real_type != np.float64, reason="float32 not supported yet")
 @parametrize_cell_types_simplex
 @pytest.mark.parametrize("family", ["RT", "N1curl"])
 @pytest.mark.parametrize("degree", [1, 2, 3, 4])
@@ -437,6 +461,7 @@ def test_RT_N1curl_simplex(family, degree, cell_type, datadir):
     run_vector_test(mesh, V, degree - 1)
 
 
+@pytest.mark.skipif(default_real_type != np.float64, reason="float32 not supported yet")
 @parametrize_cell_types_simplex
 @pytest.mark.parametrize("family", ["Discontinuous Raviart-Thomas"])
 @pytest.mark.parametrize("degree", [1, 2, 3, 4])
@@ -448,6 +473,7 @@ def test_discontinuous_RT(family, degree, cell_type, datadir):
     run_vector_test(mesh, V, degree - 1)
 
 
+@pytest.mark.skipif(default_real_type != np.float64, reason="float32 not supported yet")
 @parametrize_cell_types_simplex
 @pytest.mark.parametrize("family", ["BDM", "N2curl"])
 @pytest.mark.parametrize("degree", [1, 2])
@@ -459,6 +485,7 @@ def test_BDM_N2curl_simplex(family, degree, cell_type, datadir):
 
 # Skip slowest test in complex to stop CI timing out
 # @skip_if_complex
+@pytest.mark.skipif(default_real_type != np.float64, reason="float32 not supported yet")
 @parametrize_cell_types_simplex
 @pytest.mark.parametrize("family", ["BDM", "N2curl"])
 @pytest.mark.parametrize("degree", [3])
@@ -470,6 +497,7 @@ def test_BDM_N2curl_simplex_highest_order(family, degree, cell_type, datadir):
 
 # Run tests on all spaces in periodic table on quadrilaterals and
 # hexahedra
+@pytest.mark.skipif(default_real_type != np.float64, reason="float32 not supported yet")
 @parametrize_cell_types_tp
 @pytest.mark.parametrize("family", ["Q"])
 @pytest.mark.parametrize("degree", [2, 3, 4])
@@ -479,6 +507,7 @@ def test_P_tp(family, degree, cell_type, datadir):
     run_scalar_test(mesh, V, degree)
 
 
+@pytest.mark.skipif(default_real_type != np.float64, reason="float32 not supported yet")
 @parametrize_cell_types_tp
 @pytest.mark.parametrize("family", ["Q"])
 @pytest.mark.parametrize("degree", [2, 3, 4])
@@ -492,6 +521,7 @@ def test_P_tp_built_in_mesh(family, degree, cell_type, datadir):
     run_scalar_test(mesh, V, degree)
 
 
+@pytest.mark.skipif(default_real_type != np.float64, reason="float32 not supported yet")
 @parametrize_cell_types_tp
 @pytest.mark.parametrize("family", ["Q"])
 @pytest.mark.parametrize("degree", [2, 3, 4])
@@ -499,10 +529,12 @@ def test_vector_P_tp(family, degree, cell_type, datadir):
     if cell_type == CellType.hexahedron and degree == 4:
         pytest.skip("Skip expensive test on hexahedron")
     mesh = get_mesh(cell_type, datadir)
-    V = VectorFunctionSpace(mesh, (family, degree))
+    gdim = mesh.geometry.dim
+    V = FunctionSpace(mesh, (family, degree, (gdim,)))
     run_vector_test(mesh, V, degree)
 
 
+@pytest.mark.skipif(default_real_type != np.float64, reason="float32 not supported yet")
 @parametrize_cell_types_quad
 @pytest.mark.parametrize("family", ["DQ"])
 @pytest.mark.parametrize("degree", [1, 2, 3])
@@ -512,6 +544,7 @@ def test_dP_quad(family, degree, cell_type, datadir):
     run_dg_test(mesh, V, degree)
 
 
+@pytest.mark.skipif(default_real_type != np.float64, reason="float32 not supported yet")
 @parametrize_cell_types_hex
 @pytest.mark.parametrize("family", ["DQ"])
 @pytest.mark.parametrize("degree", [1, 2])
@@ -521,6 +554,7 @@ def test_dP_hex(family, degree, cell_type, datadir):
     run_dg_test(mesh, V, degree)
 
 
+@pytest.mark.skipif(default_real_type != np.float64, reason="float32 not supported yet")
 @parametrize_cell_types_tp
 @pytest.mark.parametrize("family", ["S"])
 @pytest.mark.parametrize("degree", [2, 3, 4])
@@ -530,6 +564,7 @@ def test_S_tp(family, degree, cell_type, datadir):
     run_scalar_test(mesh, V, degree // 2)
 
 
+@pytest.mark.skipif(default_real_type != np.float64, reason="float32 not supported yet")
 @parametrize_cell_types_tp
 @pytest.mark.parametrize("family", ["S"])
 @pytest.mark.parametrize("degree", [2, 3, 4])
@@ -543,6 +578,7 @@ def test_S_tp_built_in_mesh(family, degree, cell_type, datadir):
     run_scalar_test(mesh, V, degree // 2)
 
 
+@pytest.mark.skipif(default_real_type != np.float64, reason="float32 not supported yet")
 @parametrize_cell_types_tp
 @pytest.mark.parametrize("family", ["S"])
 @pytest.mark.parametrize("degree", [2, 3, 4])
@@ -550,10 +586,12 @@ def test_vector_S_tp(family, degree, cell_type, datadir):
     if cell_type == CellType.hexahedron and degree == 4:
         pytest.skip("Skip expensive test on hexahedron")
     mesh = get_mesh(cell_type, datadir)
-    V = VectorFunctionSpace(mesh, (family, degree))
+    gdim = mesh.geometry.dim
+    V = FunctionSpace(mesh, (family, degree, (gdim,)))
     run_vector_test(mesh, V, degree // 2)
 
 
+@pytest.mark.skipif(default_real_type != np.float64, reason="float32 not supported yet")
 @parametrize_cell_types_quad
 @pytest.mark.parametrize("family", ["DPC"])
 @pytest.mark.parametrize("degree", [2, 3, 4])
@@ -563,6 +601,7 @@ def test_DPC_quad(family, degree, cell_type, datadir):
     run_dg_test(mesh, V, degree // 2)
 
 
+@pytest.mark.skipif(default_real_type != np.float64, reason="float32 not supported yet")
 @parametrize_cell_types_hex
 @pytest.mark.parametrize("family", ["DPC"])
 @pytest.mark.parametrize("degree", [2])
@@ -572,6 +611,7 @@ def test_DPC_hex(family, degree, cell_type, datadir):
     run_dg_test(mesh, V, degree // 2)
 
 
+@pytest.mark.skipif(default_real_type != np.float64, reason="float32 not supported yet")
 @parametrize_cell_types_quad
 @pytest.mark.parametrize("family", ["RTCE", "RTCF"])
 @pytest.mark.parametrize("degree", [1, 2, 3])
@@ -581,6 +621,7 @@ def test_RTC_quad(family, degree, cell_type, datadir):
     run_vector_test(mesh, V, degree - 1)
 
 
+@pytest.mark.skipif(default_real_type != np.float64, reason="float32 not supported yet")
 @parametrize_cell_types_hex
 @pytest.mark.parametrize("family", ["NCE", "NCF"])
 @pytest.mark.parametrize("degree", [1, 2, 3])
@@ -590,6 +631,7 @@ def test_NC_hex(family, degree, cell_type, datadir):
     run_vector_test(mesh, V, degree - 1)
 
 
+@pytest.mark.skipif(default_real_type != np.float64, reason="float32 not supported yet")
 @parametrize_cell_types_quad
 @pytest.mark.parametrize("family", ["BDMCE", "BDMCF"])
 @pytest.mark.parametrize("degree", [1, 2, 3, 4])
@@ -599,6 +641,7 @@ def test_BDM_quad(family, degree, cell_type, datadir):
     run_vector_test(mesh, V, (degree - 1) // 2)
 
 
+@pytest.mark.skipif(default_real_type != np.float64, reason="float32 not supported yet")
 @parametrize_cell_types_hex
 @pytest.mark.parametrize("family", ["AAE", "AAF"])
 @pytest.mark.parametrize("degree", [1, 2, 3])
