@@ -7,19 +7,22 @@
 
 import random
 
-import numba
+from mpi4py import MPI
+
 import numpy as np
 import pytest
 
 import basix
-import basix.ufl_wrapper
 import ufl
-from dolfinx.fem import (Expression, Function, FunctionSpace,
-                         VectorFunctionSpace, assemble_scalar, form)
-from dolfinx.mesh import (CellType, create_mesh, create_unit_cube,
-                          create_unit_square, locate_entities, meshtags)
-
-from mpi4py import MPI
+from basix.ufl import (blocked_element, custom_element, element,
+                       enriched_element, mixed_element)
+from dolfinx import default_real_type
+from dolfinx.fem import (Expression, Function, FunctionSpace, assemble_scalar,
+                         create_nonmatching_meshes_interpolation_data, form)
+from dolfinx.geometry import bb_tree, compute_collisions_points
+from dolfinx.mesh import (CellType, create_mesh, create_rectangle,
+                          create_unit_cube, create_unit_square,
+                          locate_entities, locate_entities_boundary, meshtags)
 
 parametrize_cell_types = pytest.mark.parametrize(
     "cell_type", [
@@ -58,7 +61,8 @@ def random_point_in_reference(cell_type):
 
 
 def random_point_in_cell(mesh):
-    cell_type = mesh.topology.cell_type
+    assert len(mesh.topology.cell_types) == 1
+    cell_type = mesh.topology.cell_types[0]
     point = random_point_in_reference(cell_type)
 
     if cell_type == CellType.interval:
@@ -82,40 +86,39 @@ def random_point_in_cell(mesh):
 
 def one_cell_mesh(cell_type):
     if cell_type == CellType.interval:
-        points = np.array([[-1.], [2.]])
+        points = np.array([[-1.], [2.]], dtype=default_real_type)
     if cell_type == CellType.triangle:
-        points = np.array([[-1., -1.], [2., 0.], [0., 0.5]])
+        points = np.array([[-1., -1.], [2., 0.], [0., 0.5]], dtype=default_real_type)
     elif cell_type == CellType.tetrahedron:
-        points = np.array([[-1., -1., -1.], [2., 0., 0.], [0., 0.5, 0.], [0., 0., 1.]])
+        points = np.array([[-1., -1., -1.], [2., 0., 0.], [0., 0.5, 0.], [0., 0., 1.]], dtype=default_real_type)
     elif cell_type == CellType.quadrilateral:
-        points = np.array([[-1., 0.], [1., 0.], [-1., 1.5], [1., 1.5]])
+        points = np.array([[-1., 0.], [1., 0.], [-1., 1.5], [1., 1.5]], dtype=default_real_type)
     elif cell_type == CellType.hexahedron:
         points = np.array([[-1., -0.5, 0.], [1., -0.5, 0.], [-1., 1.5, 0.],
                            [1., 1.5, 0.], [0., -0.5, 1.], [1., -0.5, 1.],
-                           [-1., 1.5, 1.], [1., 1.5, 1.]])
+                           [-1., 1.5, 1.], [1., 1.5, 1.]], dtype=default_real_type)
     num_points = len(points)
 
     # Randomly number the points and create the mesh
     order = list(range(num_points))
     random.shuffle(order)
-    ordered_points = np.zeros(points.shape)
+    ordered_points = np.zeros(points.shape, dtype=default_real_type)
     for i, j in enumerate(order):
         ordered_points[j] = points[i]
     cells = np.array([order])
 
-    domain = ufl.Mesh(ufl.VectorElement("Lagrange", cell_type.name, 1))
+    domain = ufl.Mesh(element("Lagrange", cell_type.name, 1, shape=(ordered_points.shape[1],)))
     return create_mesh(MPI.COMM_WORLD, cells, ordered_points, domain)
 
 
 def two_cell_mesh(cell_type):
     if cell_type == CellType.interval:
-        points = np.array([[0.], [1.], [-1.]])
+        points = np.array([[0.], [1.], [-1.]], dtype=default_real_type)
         cells = [[0, 1], [0, 2]]
     if cell_type == CellType.triangle:
         # Define equilateral triangles with area 1
         root = 3 ** 0.25  # 4th root of 3
-        points = np.array([[0., 0.], [2 / root, 0.],
-                           [1 / root, root], [1 / root, -root]])
+        points = np.array([[0., 0.], [2 / root, 0.], [1 / root, root], [1 / root, -root]], dtype=default_real_type)
         cells = [[0, 1, 2], [1, 0, 3]]
     elif cell_type == CellType.tetrahedron:
         # Define regular tetrahedra with volume 1
@@ -123,21 +126,21 @@ def two_cell_mesh(cell_type):
         points = np.array([[0., 0., 0.], [s, 0., 0.],
                            [s / 2, s * np.sqrt(3) / 2, 0.],
                            [s / 2, s / 2 / np.sqrt(3), s * np.sqrt(2 / 3)],
-                           [s / 2, s / 2 / np.sqrt(3), -s * np.sqrt(2 / 3)]])
+                           [s / 2, s / 2 / np.sqrt(3), -s * np.sqrt(2 / 3)]], dtype=default_real_type)
         cells = [[0, 1, 2, 3], [0, 2, 1, 4]]
     elif cell_type == CellType.quadrilateral:
         # Define unit quadrilaterals (area 1)
-        points = np.array([[0., 0.], [1., 0.], [0., 1.], [1., 1.], [0., -1.], [1., -1.]])
+        points = np.array([[0., 0.], [1., 0.], [0., 1.], [1., 1.], [0., -1.], [1., -1.]], dtype=default_real_type)
         cells = [[0, 1, 2, 3], [5, 1, 4, 0]]
     elif cell_type == CellType.hexahedron:
         # Define unit hexahedra (volume 1)
         points = np.array([[0., 0., 0.], [1., 0., 0.], [0., 1., 0.],
                            [1., 1., 0.], [0., 0., 1.], [1., 0., 1.],
                            [0., 1., 1.], [1., 1., 1.], [0., 0., -1.],
-                           [1., 0., -1.], [0., 1., -1.], [1., 1., -1.]])
+                           [1., 0., -1.], [0., 1., -1.], [1., 1., -1.]], dtype=default_real_type)
         cells = [[0, 1, 2, 3, 4, 5, 6, 7], [9, 11, 8, 10, 1, 3, 0, 2]]
 
-    domain = ufl.Mesh(ufl.VectorElement("Lagrange", cell_type.name, 1))
+    domain = ufl.Mesh(element("Lagrange", cell_type.name, 1, shape=(points.shape[1],)))
     mesh = create_mesh(MPI.COMM_WORLD, cells, points, domain)
     return mesh
 
@@ -146,7 +149,6 @@ def run_scalar_test(V, poly_order):
     """Test that interpolation is correct in a scalar valued space."""
     random.seed(13)
     tdim = V.mesh.topology.dim
-
     if tdim == 1:
         def f(x):
             return x[0] ** poly_order
@@ -160,10 +162,11 @@ def run_scalar_test(V, poly_order):
     v = Function(V)
     v.interpolate(f)
     points = [random_point_in_cell(V.mesh) for count in range(5)]
+    points = np.asarray(points, dtype=default_real_type)
     cells = [0 for count in range(5)]
     values = v.eval(points, cells)
     for p, val in zip(points, values):
-        assert np.allclose(val, f(p))
+        assert np.allclose(val, f(p), atol=1.0e-5)
 
 
 def run_vector_test(V, poly_order):
@@ -187,7 +190,7 @@ def run_vector_test(V, poly_order):
     cells = [0 for count in range(5)]
     values = v.eval(points, cells)
     for p, val in zip(points, values):
-        assert np.allclose(val, f(p))
+        assert np.allclose(val, f(p), atol=1.0e-5)
 
 
 @pytest.mark.skip_in_parallel
@@ -214,9 +217,10 @@ def test_serendipity_interpolation(cell_type, order):
 @parametrize_cell_types
 @pytest.mark.parametrize('order', range(1, 5))
 def test_vector_interpolation(cell_type, order):
-    """Test that interpolation is correct in a VectorFunctionSpace."""
+    """Test that interpolation is correct in a blocked (vector) FunctionSpace."""
     mesh = one_cell_mesh(cell_type)
-    V = VectorFunctionSpace(mesh, ("Lagrange", order))
+    gdim = mesh.geometry.dim
+    V = FunctionSpace(mesh, ("Lagrange", order, (gdim,)))
     run_vector_test(V, order)
 
 
@@ -266,9 +270,9 @@ def test_mixed_sub_interpolation():
     def f(x):
         return np.vstack((10 + x[0], -10 - x[1], 25 + x[0]))
 
-    P2 = ufl.VectorElement("Lagrange", mesh.ufl_cell(), 2)
-    P1 = ufl.FiniteElement("Lagrange", mesh.ufl_cell(), 1)
-    for i, P in enumerate((P2 * P1, P1 * P2)):
+    P2 = element("Lagrange", mesh.basix_cell(), 2, shape=(mesh.geometry.dim,))
+    P1 = element("Lagrange", mesh.basix_cell(), 1)
+    for i, P in enumerate((mixed_element([P2, P1]), mixed_element([P1, P2]))):
         W = FunctionSpace(mesh, P)
         U = Function(W)
         U.sub(i).interpolate(f)
@@ -278,28 +282,31 @@ def test_mixed_sub_interpolation():
         u, v = Function(V), Function(V)
         u.interpolate(U.sub(i))
         v.interpolate(f)
-        assert np.allclose(u.vector.array, v.vector.array)
+        assert np.allclose(u.x.array, v.x.array)
 
         # Same map, different elements
-        V = VectorFunctionSpace(mesh, ("Lagrange", 1))
+        gdim = mesh.geometry.dim
+        V = FunctionSpace(mesh, ("Lagrange", 1, (gdim,)))
         u, v = Function(V), Function(V)
         u.interpolate(U.sub(i))
         v.interpolate(f)
-        assert np.allclose(u.vector.array, v.vector.array)
+        assert np.allclose(u.x.array, v.x.array)
 
         # Different maps (0)
         V = FunctionSpace(mesh, ("N1curl", 1))
         u, v = Function(V), Function(V)
         u.interpolate(U.sub(i))
         v.interpolate(f)
-        assert np.allclose(u.vector.array, v.vector.array)
+        atol = 5 * np.finfo(u.x.array.dtype).resolution
+        assert np.allclose(u.x.array, v.x.array, atol=atol)
 
         # Different maps (1)
         V = FunctionSpace(mesh, ("RT", 2))
         u, v = Function(V), Function(V)
         u.interpolate(U.sub(i))
         v.interpolate(f)
-        assert np.allclose(u.vector.array, v.vector.array)
+        atol = 5 * np.finfo(u.x.array.dtype).resolution
+        assert np.allclose(u.x.array, v.x.array, atol=atol)
 
         # Test with wrong shape
         V0 = FunctionSpace(mesh, P.sub_elements()[0])
@@ -315,9 +322,9 @@ def test_mixed_sub_interpolation():
 def test_mixed_interpolation():
     """Test that mixed interpolation raised an exception."""
     mesh = one_cell_mesh(CellType.triangle)
-    A = ufl.FiniteElement("Lagrange", mesh.ufl_cell(), 1)
-    B = ufl.VectorElement("Lagrange", mesh.ufl_cell(), 1)
-    v = Function(FunctionSpace(mesh, ufl.MixedElement([A, B])))
+    A = element("Lagrange", mesh.basix_cell(), 1)
+    B = element("Lagrange", mesh.basix_cell(), 1, shape=(mesh.geometry.dim,))
+    v = Function(FunctionSpace(mesh, mixed_element([A, B])))
     with pytest.raises(RuntimeError):
         v.interpolate(lambda x: (x[1], 2 * x[0], 3 * x[1]))
 
@@ -334,14 +341,14 @@ def test_interpolation_nedelec(order1, order2):
     # space for order > 1
     u.interpolate(lambda x: x)
     v.interpolate(u)
-    assert np.isclose(assemble_scalar(form(ufl.inner(u - v, u - v) * ufl.dx)), 0)
+    assert assemble_scalar(form(ufl.inner(u - v, u - v) * ufl.dx)) == pytest.approx(0, abs=1.0e-10)
 
     # The target expression is also contained in N2curl space of any
     # order
     V2 = FunctionSpace(mesh, ("N2curl", 1))
     w = Function(V2)
     w.interpolate(u)
-    assert np.isclose(assemble_scalar(form(ufl.inner(u - w, u - w) * ufl.dx)), 0)
+    assert assemble_scalar(form(ufl.inner(u - w, u - w) * ufl.dx)) == pytest.approx(0, abs=1.0e-10)
 
 
 @pytest.mark.parametrize("tdim", [2, 3])
@@ -351,13 +358,12 @@ def test_interpolation_dg_to_n1curl(tdim, order):
         mesh = create_unit_square(MPI.COMM_WORLD, 5, 5)
     else:
         mesh = create_unit_cube(MPI.COMM_WORLD, 2, 2, 2)
-    V = VectorFunctionSpace(mesh, ("DG", order))
+    V = FunctionSpace(mesh, ("DG", order, (tdim,)))
     V1 = FunctionSpace(mesh, ("N1curl", order + 1))
     u, v = Function(V), Function(V1)
     u.interpolate(lambda x: x[:tdim] ** order)
     v.interpolate(u)
-    s = assemble_scalar(form(ufl.inner(u - v, u - v) * ufl.dx))
-    assert np.isclose(s, 0)
+    assert assemble_scalar(form(ufl.inner(u - v, u - v) * ufl.dx)) == pytest.approx(0.0, abs=1.0e-8)
 
 
 @pytest.mark.parametrize("tdim", [2, 3])
@@ -368,12 +374,11 @@ def test_interpolation_n1curl_to_dg(tdim, order):
     else:
         mesh = create_unit_cube(MPI.COMM_WORLD, 2, 2, 2)
     V = FunctionSpace(mesh, ("N1curl", order + 1))
-    V1 = VectorFunctionSpace(mesh, ("DG", order))
+    V1 = FunctionSpace(mesh, ("DG", order, (tdim,)))
     u, v = Function(V), Function(V1)
     u.interpolate(lambda x: x[:tdim] ** order)
     v.interpolate(u)
-    s = assemble_scalar(form(ufl.inner(u - v, u - v) * ufl.dx))
-    assert np.isclose(s, 0)
+    assert assemble_scalar(form(ufl.inner(u - v, u - v) * ufl.dx)) == pytest.approx(0.0, abs=1e-10)
 
 
 @pytest.mark.parametrize("tdim", [2, 3])
@@ -388,8 +393,7 @@ def test_interpolation_n2curl_to_bdm(tdim, order):
     u, v = Function(V), Function(V1)
     u.interpolate(lambda x: x[:tdim] ** order)
     v.interpolate(u)
-    s = assemble_scalar(form(ufl.inner(u - v, u - v) * ufl.dx))
-    assert np.isclose(s, 0)
+    assert assemble_scalar(form(ufl.inner(u - v, u - v) * ufl.dx)) == pytest.approx(0.0, abs=1.0e-10)
 
 
 @pytest.mark.parametrize("order1", [1, 2, 3, 4, 5])
@@ -399,39 +403,32 @@ def test_interpolation_p2p(order1, order2):
     V = FunctionSpace(mesh, ("Lagrange", order1))
     V1 = FunctionSpace(mesh, ("Lagrange", order2))
     u, v = Function(V), Function(V1)
-
     u.interpolate(lambda x: x[0])
     v.interpolate(u)
-
-    s = assemble_scalar(form(ufl.inner(u - v, u - v) * ufl.dx))
-    assert np.isclose(s, 0)
+    assert assemble_scalar(form(ufl.inner(u - v, u - v) * ufl.dx)) == pytest.approx(0.0, abs=1e-10)
 
     DG = FunctionSpace(mesh, ("DG", order2))
     w = Function(DG)
     w.interpolate(u)
-    s = assemble_scalar(form(ufl.inner(u - w, u - w) * ufl.dx))
-    assert np.isclose(s, 0)
+    assert assemble_scalar(form(ufl.inner(u - w, u - w) * ufl.dx)) == pytest.approx(0.0, abs=1e-10)
 
 
 @pytest.mark.parametrize("order1", [1, 2, 3])
 @pytest.mark.parametrize("order2", [1, 2])
 def test_interpolation_vector_elements(order1, order2):
     mesh = create_unit_cube(MPI.COMM_WORLD, 2, 2, 2)
-    V = VectorFunctionSpace(mesh, ("Lagrange", order1))
-    V1 = VectorFunctionSpace(mesh, ("Lagrange", order2))
+    gdim = mesh.geometry.dim
+    V = FunctionSpace(mesh, ("Lagrange", order1, (gdim,)))
+    V1 = FunctionSpace(mesh, ("Lagrange", order2, (gdim,)))
     u, v = Function(V), Function(V1)
-
     u.interpolate(lambda x: x)
     v.interpolate(u)
+    assert assemble_scalar(form(ufl.inner(u - v, u - v) * ufl.dx)) == pytest.approx(0)
 
-    s = assemble_scalar(form(ufl.inner(u - v, u - v) * ufl.dx))
-    assert np.isclose(s, 0)
-
-    DG = VectorFunctionSpace(mesh, ("DG", order2))
+    DG = FunctionSpace(mesh, ("DG", order2, (gdim,)))
     w = Function(DG)
     w.interpolate(u)
-    s = assemble_scalar(form(ufl.inner(u - w, u - w) * ufl.dx))
-    assert np.isclose(s, 0)
+    assert assemble_scalar(form(ufl.inner(u - w, u - w) * ufl.dx)) == pytest.approx(0)
 
 
 @pytest.mark.skip_in_parallel
@@ -442,19 +439,16 @@ def test_interpolation_non_affine():
                        [1, 0, 1.5], [0.5, 2, 0], [0, 2, 1.5], [1, 2, 1.5],
                        [0.5, 0, 3], [0, 1, 3], [1, 1, 3], [0.5, 2, 3],
                        [0.5, 1, 0], [0.5, 0, 1.5], [0, 1, 1.5], [1, 1, 1.5],
-                       [0.5, 2, 1.5], [0.5, 1, 3], [0.5, 1, 1.5]], dtype=np.float64)
-
+                       [0.5, 2, 1.5], [0.5, 1, 3], [0.5, 1, 1.5]], dtype=default_real_type)
     cells = np.array([range(len(points))], dtype=np.int32)
-    cell_type = CellType.hexahedron
-    domain = ufl.Mesh(ufl.VectorElement("Lagrange", cell_type.name, 2))
+    domain = ufl.Mesh(element("Lagrange", "hexahedron", 2, shape=(3,)))
     mesh = create_mesh(MPI.COMM_WORLD, cells, points, domain)
     W = FunctionSpace(mesh, ("NCE", 1))
     V = FunctionSpace(mesh, ("NCE", 2))
     w, v = Function(W), Function(V)
     w.interpolate(lambda x: x)
     v.interpolate(w)
-    s = assemble_scalar(form(ufl.inner(w - v, w - v) * ufl.dx))
-    assert np.isclose(s, 0)
+    assert assemble_scalar(form(ufl.inner(w - v, w - v) * ufl.dx)) == pytest.approx(0, abs=1e-10)
 
 
 @pytest.mark.skip_in_parallel
@@ -465,19 +459,17 @@ def test_interpolation_non_affine_nonmatching_maps():
                        [1, 0, 1.5], [0.5, 2, 0], [0, 2, 1.5], [1, 2, 1.5],
                        [0.5, 0, 3], [0, 1, 3], [1, 1, 3], [0.5, 2, 3],
                        [0.5, 1, 0], [0.5, -0.1, 1.5], [0, 1, 1.5], [1, 1, 1.5],
-                       [0.5, 2, 1.5], [0.5, 1, 3], [0.5, 1, 1.5]], dtype=np.float64)
-
+                       [0.5, 2, 1.5], [0.5, 1, 3], [0.5, 1, 1.5]], dtype=default_real_type)
     cells = np.array([range(len(points))], dtype=np.int32)
-    cell_type = CellType.hexahedron
-    domain = ufl.Mesh(ufl.VectorElement("Lagrange", cell_type.name, 2))
+    domain = ufl.Mesh(element("Lagrange", "hexahedron", 2, shape=(3,)))
     mesh = create_mesh(MPI.COMM_WORLD, cells, points, domain)
-    W = VectorFunctionSpace(mesh, ("DG", 1))
+    gdim = mesh.geometry.dim
+    W = FunctionSpace(mesh, ("DG", 1, (gdim,)))
     V = FunctionSpace(mesh, ("NCE", 4))
     w, v = Function(W), Function(V)
     w.interpolate(lambda x: x)
     v.interpolate(w)
-    s = assemble_scalar(form(ufl.inner(w - v, w - v) * ufl.dx))
-    assert np.isclose(s, 0)
+    assert assemble_scalar(form(ufl.inner(w - v, w - v) * ufl.dx)) == pytest.approx(0, abs=1e-10)
 
 
 @pytest.mark.parametrize("order", [2, 3, 4])
@@ -497,7 +489,7 @@ def test_nedelec_spatial(order, dim):
     f_ex = x
     f = Expression(f_ex, V.element.interpolation_points())
     u.interpolate(f)
-    assert np.isclose(np.abs(assemble_scalar(form(ufl.inner(u - f_ex, u - f_ex) * ufl.dx))), 0)
+    assert np.abs(assemble_scalar(form(ufl.inner(u - f_ex, u - f_ex) * ufl.dx))) == pytest.approx(0, abs=1e-10)
 
     # The target expression is also contained in N2curl space of any
     # order
@@ -505,7 +497,7 @@ def test_nedelec_spatial(order, dim):
     w = Function(V2)
     f2 = Expression(f_ex, V2.element.interpolation_points())
     w.interpolate(f2)
-    assert np.isclose(np.abs(assemble_scalar(form(ufl.inner(w - f_ex, w - f_ex) * ufl.dx))), 0)
+    assert np.abs(assemble_scalar(form(ufl.inner(w - f_ex, w - f_ex) * ufl.dx))) == pytest.approx(0)
 
 
 @pytest.mark.parametrize("order", [1, 2, 3, 4])
@@ -519,34 +511,30 @@ def test_vector_interpolation_spatial(order, dim, affine):
         ct = CellType.tetrahedron if affine else CellType.hexahedron
         mesh = create_unit_cube(MPI.COMM_WORLD, 3, 2, 2, ct)
 
-    V = VectorFunctionSpace(mesh, ("Lagrange", order))
+    V = FunctionSpace(mesh, ("Lagrange", order, (dim,)))
     u = Function(V)
     x = ufl.SpatialCoordinate(mesh)
 
     # The expression (x,y,z)^n is contained in space
     f = ufl.as_vector([x[i]**order for i in range(dim)])
     u.interpolate(Expression(f, V.element.interpolation_points()))
-    assert np.isclose(np.abs(assemble_scalar(form(ufl.inner(u - f, u - f) * ufl.dx))), 0)
+    assert np.abs(assemble_scalar(form(ufl.inner(u - f, u - f) * ufl.dx))) == pytest.approx(0)
 
 
 @pytest.mark.parametrize("order", [1, 2, 3, 4])
 def test_2D_lagrange_to_curl(order):
     mesh = create_unit_square(MPI.COMM_WORLD, 3, 4)
-    V = FunctionSpace(mesh, ("N1curl", order))
-    u = Function(V)
-
-    W = FunctionSpace(mesh, ("Lagrange", order))
-    u0 = Function(W)
+    V, W = FunctionSpace(mesh, ("N1curl", order)), FunctionSpace(mesh, ("Lagrange", order))
+    u, u0 = Function(V), Function(W)
     u0.interpolate(lambda x: -x[1])
     u1 = Function(W)
     u1.interpolate(lambda x: x[0])
-
     f = ufl.as_vector((u0, u1))
     f_expr = Expression(f, V.element.interpolation_points())
     u.interpolate(f_expr)
     x = ufl.SpatialCoordinate(mesh)
     f_ex = ufl.as_vector((-x[1], x[0]))
-    assert np.isclose(np.abs(assemble_scalar(form(ufl.inner(u - f_ex, u - f_ex) * ufl.dx))), 0)
+    assert np.abs(assemble_scalar(form(ufl.inner(u - f_ex, u - f_ex) * ufl.dx))) == pytest.approx(0)
 
 
 @pytest.mark.parametrize("order", [2, 3, 4])
@@ -555,15 +543,13 @@ def test_de_rahm_2D(order):
     W = FunctionSpace(mesh, ("Lagrange", order))
     w = Function(W)
     w.interpolate(lambda x: x[0] + x[0] * x[1] + 2 * x[1]**2)
-
     g = ufl.grad(w)
     Q = FunctionSpace(mesh, ("N2curl", order - 1))
     q = Function(Q)
     q.interpolate(Expression(g, Q.element.interpolation_points()))
-
     x = ufl.SpatialCoordinate(mesh)
     g_ex = ufl.as_vector((1 + x[1], 4 * x[1] + x[0]))
-    assert np.isclose(np.abs(assemble_scalar(form(ufl.inner(q - g_ex, q - g_ex) * ufl.dx))), 0)
+    assert np.abs(assemble_scalar(form(ufl.inner(q - g_ex, q - g_ex) * ufl.dx))) == pytest.approx(0, abs=1e-10)
 
     V = FunctionSpace(mesh, ("BDM", order - 1))
     v = Function(V)
@@ -573,13 +559,14 @@ def test_de_rahm_2D(order):
 
     v.interpolate(Expression(curl2D(ufl.grad(w)), V.element.interpolation_points()))
     h_ex = ufl.as_vector((1, -1))
-    assert np.isclose(np.abs(assemble_scalar(form(ufl.inner(v - h_ex, v - h_ex) * ufl.dx))), 0)
+    assert np.abs(assemble_scalar(form(ufl.inner(v - h_ex, v - h_ex) * ufl.dx))) == pytest.approx(0, abs=1.0e-6)
 
 
 @pytest.mark.parametrize("order", [1, 2, 3, 4])
 @pytest.mark.parametrize("dim", [2, 3])
 @pytest.mark.parametrize("affine", [True, False])
-def test_interpolate_subset(order, dim, affine):
+@pytest.mark.parametrize("callable_", [True, False])
+def test_interpolate_subset(order, dim, affine, callable_):
     if dim == 2:
         ct = CellType.triangle if affine else CellType.quadrilateral
         mesh = create_unit_square(MPI.COMM_WORLD, 3, 4, ct)
@@ -593,73 +580,84 @@ def test_interpolate_subset(order, dim, affine):
     cells = locate_entities(mesh, mesh.topology.dim, lambda x: x[1] <= 0.5 + 1e-10)
     num_local_cells = mesh.topology.index_map(mesh.topology.dim).size_local
     cells_local = cells[cells < num_local_cells]
-
     x = ufl.SpatialCoordinate(mesh)
     f = x[1]**order
-    expr = Expression(f, V.element.interpolation_points())
-    u.interpolate(expr, cells_local)
+    if not callable_:
+        expr = Expression(f, V.element.interpolation_points())
+        u.interpolate(expr, cells_local)
+    else:
+        u.interpolate(lambda x: x[1]**order, cells_local)
     mt = meshtags(mesh, mesh.topology.dim, cells_local, np.ones(cells_local.size, dtype=np.int32))
     dx = ufl.Measure("dx", domain=mesh, subdomain_data=mt)
-    assert np.isclose(np.abs(form(assemble_scalar(form(ufl.inner(u - f, u - f) * dx(1))))), 0)
+    assert np.abs(form(assemble_scalar(form(ufl.inner(u - f, u - f) * dx(1))))) == pytest.approx(0)
     integral = mesh.comm.allreduce(assemble_scalar(form(u * dx)), op=MPI.SUM)
-    assert np.isclose(integral, 1 / (order + 1) * 0.5**(order + 1), 0)
+    assert integral == pytest.approx(1 / (order + 1) * 0.5**(order + 1), abs=1.0e-6)
 
 
 def test_interpolate_callable():
     """Test interpolation with callables"""
+    numba = pytest.importorskip("numba")
     mesh = create_unit_square(MPI.COMM_WORLD, 2, 1)
     V = FunctionSpace(mesh, ("Lagrange", 2))
     u0, u1 = Function(V), Function(V)
 
-    @numba.njit
+    @ numba.njit
     def f(x):
         return x[0]
 
     u0.interpolate(lambda x: x[0])
     u1.interpolate(f)
     assert np.allclose(u0.x.array, u1.x.array)
-
     with pytest.raises(RuntimeError):
         u0.interpolate(lambda x: np.vstack([x[0], x[1]]))
 
 
+@pytest.mark.parametrize("bound", [1.5, 0.5])
+def test_interpolate_callable_subset(bound):
+    """Test interpolation on subsets with callables"""
+    mesh = create_unit_square(MPI.COMM_WORLD, 3, 4)
+    cells = locate_entities(mesh, mesh.topology.dim, lambda x: x[1] <= bound + 1e-10)
+    num_local_cells = mesh.topology.index_map(mesh.topology.dim).size_local
+    cells_local = cells[cells < num_local_cells]
+    V = FunctionSpace(mesh, ("DG", 2))
+    u0, u1 = Function(V), Function(V)
+    x = ufl.SpatialCoordinate(mesh)
+    f = x[0]
+    expr = Expression(f, V.element.interpolation_points())
+    u0.interpolate(lambda x: x[0], cells_local)
+    u1.interpolate(expr, cells_local)
+    assert np.allclose(u0.x.array, u1.x.array, rtol=1.0e-6, atol=1.0e-6)
+
+
 @pytest.mark.parametrize("scalar_element", [
-    ufl.FiniteElement("P", "triangle", 1),
-    ufl.FiniteElement("P", "triangle", 2),
-    ufl.FiniteElement("P", "triangle", 3),
-    ufl.FiniteElement("Q", "quadrilateral", 1),
-    ufl.FiniteElement("Q", "quadrilateral", 2),
-    ufl.FiniteElement("Q", "quadrilateral", 3),
-    ufl.FiniteElement("S", "quadrilateral", 1),
-    ufl.FiniteElement("S", "quadrilateral", 2),
-    ufl.FiniteElement("S", "quadrilateral", 3),
-    ufl.EnrichedElement(ufl.FiniteElement("P", "triangle", 1), ufl.FiniteElement("Bubble", "triangle", 3)),
-    basix.ufl_wrapper._create_enriched_element([
-        basix.ufl_wrapper.create_element("P", "quadrilateral", 1),
-        basix.ufl_wrapper.create_element("Bubble", "quadrilateral", 2)]),
+    element("P", "triangle", 1),
+    element("P", "triangle", 2),
+    element("P", "triangle", 3),
+    element("Q", "quadrilateral", 1),
+    element("Q", "quadrilateral", 2),
+    element("Q", "quadrilateral", 3),
+    element("S", "quadrilateral", 1),
+    element("S", "quadrilateral", 2),
+    element("S", "quadrilateral", 3),
+    enriched_element([element("P", "triangle", 1), element("Bubble", "triangle", 3)]),
+    enriched_element([element("P", "quadrilateral", 1), element("Bubble", "quadrilateral", 2)]),
 ])
 def test_vector_element_interpolation(scalar_element):
     """Test interpolation into a range of vector elements."""
     mesh = create_unit_square(MPI.COMM_WORLD, 10, 10, getattr(CellType, scalar_element.cell().cellname()))
-
-    V = FunctionSpace(mesh, ufl.VectorElement(scalar_element))
-
+    V = FunctionSpace(mesh, blocked_element(scalar_element, shape=(2, )))
     u = Function(V)
     u.interpolate(lambda x: (x[0], x[1]))
-
     u2 = Function(V)
     u2.sub(0).interpolate(lambda x: x[0])
     u2.sub(1).interpolate(lambda x: x[1])
-
     assert np.allclose(u2.x.array, u.x.array)
 
 
 def test_custom_vector_element():
     """Test interpolation into an element with a value size that uses an identity map."""
     mesh = create_unit_square(MPI.COMM_WORLD, 10, 10)
-
     wcoeffs = np.eye(6)
-
     x = [[], [], [], []]
     x[0].append(np.array([[0., 0.]]))
     x[0].append(np.array([[1., 0.]]))
@@ -667,27 +665,23 @@ def test_custom_vector_element():
     for _ in range(3):
         x[1].append(np.zeros((0, 2)))
     x[2].append(np.zeros((0, 2)))
-
     M = [[], [], [], []]
     for _ in range(3):
         M[0].append(np.array([[[[1.]], [[0.]]], [[[0.]], [[1.]]]]))
     for _ in range(3):
         M[1].append(np.zeros((0, 2, 0, 1)))
     M[2].append(np.zeros((0, 2, 0, 1)))
+    e = custom_element(basix.CellType.triangle, [2], wcoeffs, x, M, 0, basix.MapType.identity,
+                       basix.SobolevSpace.H1, False, 1, 1)
 
-    element = basix.create_custom_element(
-        basix.CellType.triangle, [2], wcoeffs, x, M, 0, basix.MapType.identity,
-        basix.SobolevSpace.H1, False, 1, 1)
-
-    V = FunctionSpace(mesh, basix.ufl_wrapper.BasixElement(element))
-    W = VectorFunctionSpace(mesh, ("Lagrange", 1))
-
+    V = FunctionSpace(mesh, e)
+    gdim = mesh.geometry.dim
+    W = FunctionSpace(mesh, ("Lagrange", 1, (gdim,)))
     v = Function(V)
     w = Function(W)
     v.interpolate(lambda x: (x[0], x[1]))
     w.interpolate(lambda x: (x[0], x[1]))
-
-    assert np.isclose(np.abs(assemble_scalar(form(ufl.inner(v - w, v - w) * ufl.dx))), 0)
+    assert np.abs(assemble_scalar(form(ufl.inner(v - w, v - w) * ufl.dx))) == pytest.approx(0)
 
 
 @pytest.mark.skip_in_parallel
@@ -703,20 +697,147 @@ def test_mixed_interpolation_permuting(cell_type, order):
     x = ufl.SpatialCoordinate(mesh)
     dgdy = ufl.cos(x[1])
 
-    curl_el = ufl.FiniteElement("N1curl", mesh.ufl_cell(), 1)
-    vlag_el = ufl.VectorElement("Lagrange", mesh.ufl_cell(), 1)
-    lagr_el = ufl.FiniteElement("Lagrange", mesh.ufl_cell(), order)
+    curl_el = element("N1curl", mesh.basix_cell(), 1)
+    vlag_el = element("Lagrange", mesh.basix_cell(), 1, shape=(mesh.geometry.dim,))
+    lagr_el = element("Lagrange", mesh.basix_cell(), order)
 
-    V = FunctionSpace(mesh, ufl.MixedElement([curl_el, lagr_el]))
+    V = FunctionSpace(mesh, mixed_element([curl_el, lagr_el]))
     Eb_m = Function(V)
     Eb_m.sub(1).interpolate(g)
     diff = Eb_m[2].dx(1) - dgdy
     error = assemble_scalar(form(ufl.dot(diff, diff) * ufl.dx))
 
-    V = FunctionSpace(mesh, ufl.MixedElement([vlag_el, lagr_el]))
+    V = FunctionSpace(mesh, mixed_element([vlag_el, lagr_el]))
     Eb_m = Function(V)
     Eb_m.sub(1).interpolate(g)
     diff = Eb_m[2].dx(1) - dgdy
-    error2 = assemble_scalar(form(ufl.dot(diff, diff) * ufl.dx))
+    assert assemble_scalar(form(ufl.dot(diff, diff) * ufl.dx)) == pytest.approx(error)
 
-    assert np.isclose(error, error2)
+
+@pytest.mark.parametrize("xtype", [np.float64])
+@pytest.mark.parametrize("cell_type0", [CellType.hexahedron, CellType.tetrahedron])
+@pytest.mark.parametrize("cell_type1", [CellType.triangle, CellType.quadrilateral])
+def test_nonmatching_mesh_interpolation(xtype, cell_type0, cell_type1):
+    mesh0 = create_unit_cube(MPI.COMM_WORLD, 5, 6, 7, cell_type=cell_type0, dtype=xtype)
+    mesh1 = create_unit_square(MPI.COMM_WORLD, 5, 4, cell_type=cell_type1, dtype=xtype)
+
+    def f(x):
+        return (7 * x[1], 3 * x[0], x[2] + 0.4)
+
+    el0 = element("Lagrange", mesh0.basix_cell(), 1, shape=(3, ))
+    V0 = FunctionSpace(mesh0, el0)
+    el1 = element("Lagrange", mesh1.basix_cell(), 1, shape=(3, ))
+    V1 = FunctionSpace(mesh1, el1)
+
+    # Interpolate on 3D mesh
+    u0 = Function(V0, dtype=xtype)
+    u0.interpolate(f)
+    u0.x.scatter_forward()
+    padding = 1e-14
+    # Interpolate 3D->2D
+    u1 = Function(V1, dtype=xtype)
+    u1.interpolate(u0, nmm_interpolation_data=create_nonmatching_meshes_interpolation_data(
+        u1.function_space.mesh._cpp_object,
+        u1.function_space.element,
+        u0.function_space.mesh._cpp_object, padding=padding))
+    u1.x.scatter_forward()
+
+    # Exact interpolation on 2D mesh
+    u1_ex = Function(V1, dtype=xtype)
+    u1_ex.interpolate(f)
+    u1_ex.x.scatter_forward()
+
+    assert np.allclose(u1_ex.x.array, u1.x.array, rtol=1.0e-4, atol=1.0e-6)
+
+    # Interpolate 2D->3D
+    u0_2 = Function(V0, dtype=xtype)
+    u0_2.interpolate(u1, nmm_interpolation_data=create_nonmatching_meshes_interpolation_data(
+        u0_2.function_space.mesh._cpp_object,
+        u0_2.function_space.element,
+        u1.function_space.mesh._cpp_object, padding=padding))
+
+    # Check that function values over facets of 3D mesh of the twice interpolated property is preserved
+    def locate_bottom_facets(x):
+        return np.isclose(x[2], 0)
+    facets = locate_entities_boundary(mesh0, mesh0.topology.dim - 1, locate_bottom_facets)
+    facet_tag = meshtags(mesh0, mesh0.topology.dim - 1, facets, np.full(len(facets), 1, dtype=np.int32))
+    residual = ufl.inner(u0 - u0_2, u0 - u0_2) * ufl.ds(domain=mesh0, subdomain_data=facet_tag, subdomain_id=1)
+    assert np.isclose(assemble_scalar(form(residual, dtype=xtype)), 0)
+
+
+@pytest.mark.parametrize("xtype", [np.float64])
+def test_nonmatching_mesh_single_cell_overlap_interpolation(xtype):
+    # mesh2 is contained by a single cell of mesh1. Here we test
+    # interpolation when *not* every process has data to communicate
+
+    # Test interpolation from mesh1 to mesh2
+    n_mesh1 = 2
+    mesh1 = create_rectangle(MPI.COMM_WORLD, [[0.0, 0.0], [1.0, 1.0]], [n_mesh1, n_mesh1],
+                             cell_type=CellType.quadrilateral, dtype=xtype)
+
+    n_mesh2 = 2
+    p0_mesh2 = 1.0 / n_mesh1
+    mesh2 = create_rectangle(MPI.COMM_WORLD, [[0.0, 0.0], [p0_mesh2, p0_mesh2]], [n_mesh2, n_mesh2],
+                             cell_type=CellType.triangle, dtype=xtype)
+
+    u1 = Function(FunctionSpace(mesh1, ("Lagrange", 1)), name="u1", dtype=xtype)
+    u2 = Function(FunctionSpace(mesh2, ("Lagrange", 1)), name="u2", dtype=xtype)
+
+    def f_test1(x):
+        return 1.0 - x[0] * x[1]
+
+    u1.interpolate(f_test1)
+    u1.x.scatter_forward()
+    padding = 1e-14
+    u1_2_u2_nmm_data = create_nonmatching_meshes_interpolation_data(
+        u2.function_space.mesh._cpp_object,
+        u2.function_space.element,
+        u1.function_space.mesh._cpp_object, padding=padding)
+
+    u2.interpolate(u1, nmm_interpolation_data=u1_2_u2_nmm_data)
+    u2.x.scatter_forward()
+
+    # interpolate f which is exactly represented on the element
+    u2_exact = Function(u2.function_space, dtype=xtype)
+    u2_exact.interpolate(f_test1)
+    u2_exact.x.scatter_forward()
+
+    l2_error = assemble_scalar(form((u2 - u2_exact)**2 * ufl.dx, dtype=xtype))
+    assert np.isclose(l2_error, 0.0, rtol=np.finfo(xtype).eps, atol=np.finfo(xtype).eps)
+
+    # Test interpolation from mesh2 to mesh1
+    def f_test2(x):
+        return x[0] * x[1]
+
+    u1.x.array[:] = 0.0
+    u1.x.scatter_forward()
+
+    u2.interpolate(f_test2)
+    u2.x.scatter_forward()
+    padding = 1e-14
+    u2_2_u1_nmm_data = \
+        create_nonmatching_meshes_interpolation_data(
+            u1.function_space.mesh._cpp_object,
+            u1.function_space.element,
+            u2.function_space.mesh._cpp_object, padding)
+
+    u1.interpolate(u2, nmm_interpolation_data=u2_2_u1_nmm_data)
+    u1.x.scatter_forward()
+
+    u1_exact = Function(u1.function_space, dtype=xtype)
+    u1_exact.interpolate(f_test2)
+    u1_exact.x.scatter_forward()
+
+    # Find the single cell in mesh1 which is overlapped by mesh2
+    tree1 = bb_tree(mesh1, mesh1.topology.dim)
+    cells_overlapped1 = compute_collisions_points(tree1, [p0_mesh2 / 2.0, p0_mesh2 / 2.0, 0.0]).array
+    assert cells_overlapped1.shape[0] <= 1
+
+    # Construct the error measure on the overlapped cell
+    cell_label = 1
+    cts = meshtags(mesh1, mesh1.topology.dim, cells_overlapped1,
+                   np.full_like(cells_overlapped1, cell_label))
+    dx_cell = ufl.Measure("dx", subdomain_data=cts)
+
+    l2_error = assemble_scalar(form((u1 - u1_exact)**2 * dx_cell(cell_label), dtype=xtype))
+    assert np.isclose(l2_error, 0.0, rtol=np.finfo(xtype).eps, atol=np.finfo(xtype).eps)
