@@ -6,27 +6,28 @@
 """Unit tests for assembly on vector spaces"""
 
 
-import ufl
-from dolfinx.fem import VectorFunctionSpace, form
-from dolfinx.fem.petsc import assemble_matrix
-from dolfinx.mesh import create_unit_square
-
 from mpi4py import MPI
 
+import ufl
+from dolfinx.fem import FunctionSpace, assemble_matrix, form
+from dolfinx.mesh import create_unit_square
+
 
 def test_vector_assemble_matrix_exterior():
     mesh = create_unit_square(MPI.COMM_WORLD, 3, 3)
-    V = VectorFunctionSpace(mesh, ("Lagrange", 1))
+    gdim = mesh.geometry.dim
+    V = FunctionSpace(mesh, ("Lagrange", 1, (gdim,)))
     u, v = ufl.TrialFunction(V), ufl.TestFunction(V)
     a = form(ufl.inner(u, v) * ufl.ds)
     A = assemble_matrix(a)
-    A.assemble()
+    A.scatter_reverse()
 
 
 def test_vector_assemble_matrix_interior():
     mesh = create_unit_square(MPI.COMM_WORLD, 3, 3)
-    V = VectorFunctionSpace(mesh, ("Lagrange", 1))
+    gdim = mesh.geometry.dim
+    V = FunctionSpace(mesh, ("Lagrange", 1, (gdim,)))
     u, v = ufl.TrialFunction(V), ufl.TestFunction(V)
     a = form(ufl.inner(ufl.jump(u), ufl.jump(v)) * ufl.dS)
     A = assemble_matrix(a)
-    A.assemble()
+    A.scatter_reverse()
