@@ -8,6 +8,10 @@
 
 import pathlib
 
+import petsc4py
+from mpi4py import MPI
+from petsc4py import PETSc
+
 import cppimport
 import numpy as np
 import pytest
@@ -17,10 +21,6 @@ import dolfinx.pkgconfig
 from dolfinx.jit import mpi_jit_decorator
 from dolfinx.wrappers import get_include_path as pybind_inc
 
-import petsc4py
-from mpi4py import MPI
-from petsc4py import PETSc
-
 
 @pytest.mark.skipif(not dolfinx.pkgconfig.exists("dolfinx"),
                     reason="This test needs DOLFINx pkg-config.")
@@ -82,7 +82,7 @@ PYBIND11_MODULE(petsc_casters_cppimport, m)
     x1.create(MPI.COMM_WORLD)
     x1.setSizes((local_range[1] - local_range[0], None))
     x1.setFromOptions()
-    x1.setArray(np.arange(local_range[0], local_range[1]))
+    x1.setArray(np.arange(local_range[0], local_range[1], dtype=PETSc.ScalarType))
     x2 = x1.copy()
 
     # Replace each component by its exponential
