@@ -7,20 +7,21 @@
 
 from contextlib import ExitStack
 
+from mpi4py import MPI
+from petsc4py import PETSc
+
 import numpy as np
 import pytest
 
 import ufl
 from dolfinx import la
-from dolfinx.fem import VectorFunctionSpace, form
+from dolfinx.fem import FunctionSpace, form
 from dolfinx.fem.petsc import assemble_matrix
+from dolfinx.la import create_petsc_vector
 from dolfinx.mesh import (CellType, GhostMode, create_box, create_unit_cube,
                           create_unit_square)
 from ufl import TestFunction, TrialFunction, dx, grad, inner
 
-from mpi4py import MPI
-from petsc4py import PETSc
-
 
 def build_elastic_nullspace(V):
     """Function to build nullspace for 2D/3D elasticity"""
@@ -39,7 +40,7 @@ def build_elastic_nullspace(V):
         vec_local = [stack.enter_context(x.localForm()) for x in ns]
         basis = [np.asarray(x) for x in vec_local]
 
-        dofs = [V.sub(i).dofmap.list.array for i in range(gdim)]
+        dofs = [V.sub(i).dofmap.list.flatten() for i in range(gdim)]
 
         # Build translational null space basis
         for i in range(gdim):
@@ -47,7 +48,7 @@ def build_elastic_nullspace(V):
 
         # Build rotational null space basis
         x = V.tabulate_dof_coordinates()
-        dofs_block = V.dofmap.list.array
+        dofs_block = V.dofmap.list.flatten()
         x0, x1, x2 = x[dofs_block, 0], x[dofs_block, 1], x[dofs_block, 2]
         if gdim == 2:
             basis[2][dofs[0]] = -x1
@@ -67,19 +68,19 @@ def build_broken_elastic_nullspace(V):
     """Function to build incorrect null space for 2D elasticity"""
 
     # Create list of vectors for null space
-    ns = [la.create_petsc_vector(V.dofmap.index_map, V.dofmap.index_map_bs) for i in range(4)]
+    ns = [create_petsc_vector(V.dofmap.index_map, V.dofmap.index_map_bs) for i in range(4)]
 
     with ExitStack() as stack:
         vec_local = [stack.enter_context(x.localForm()) for x in ns]
         basis = [np.asarray(x) for x in vec_local]
 
-        dofs = [V.sub(i).dofmap.list.array for i in range(2)]
+        dofs = [V.sub(i).dofmap.list.flatten() for i in range(2)]
         basis[0][dofs[0]] = 1.0
         basis[1][dofs[1]] = 1.0
 
         # Build rotational null space basis
         x = V.tabulate_dof_coordinates()
-        dofs_block = V.dofmap.list.array
+        dofs_block = V.dofmap.list.flatten()
         x0, x1 = x[dofs_block, 0], x[dofs_block, 1]
         basis[2][dofs[0]] = -x1
         basis[2][dofs[1]] = x0
@@ -97,11 +98,12 @@ def build_broken_elastic_nullspace(V):
 @pytest.mark.parametrize("degree", [1, 2])
 def test_nullspace_orthogonal(mesh, degree):
     """Test that null spaces orthogonalisation"""
-    V = VectorFunctionSpace(mesh, ('Lagrange', degree))
+    gdim = mesh.geometry.dim
+    V = FunctionSpace(mesh, ('Lagrange', degree, (gdim,)))
     nullspace = build_elastic_nullspace(V)
-    assert not la.is_orthonormal(nullspace)
+    assert not la.is_orthonormal(nullspace, eps=1.0e-4)
     la.orthonormalize(nullspace)
-    assert la.is_orthonormal(nullspace)
+    assert la.is_orthonormal(nullspace, eps=1.0e-3)
 
 
 @pytest.mark.parametrize("mesh", [
@@ -114,7 +116,8 @@ def test_nullspace_orthogonal(mesh, degree):
 ])
 @pytest.mark.parametrize("degree", [1, 2])
 def test_nullspace_check(mesh, degree):
-    V = VectorFunctionSpace(mesh, ('Lagrange', degree))
+    gdim = mesh.geometry.dim
+    V = FunctionSpace(mesh, ('Lagrange', degree, (gdim,)))
     u, v = TrialFunction(V), TestFunction(V)
 
     E, nu = 2.0e2, 0.3
