@@ -4,6 +4,8 @@
 #
 # SPDX-License-Identifier:    LGPL-3.0-or-later
 
+from mpi4py import MPI
+
 import numpy as np
 import pytest
 
@@ -12,8 +14,6 @@ from dolfinx.cpp.mesh import cell_normals
 from dolfinx.mesh import (create_unit_cube, create_unit_square,
                           locate_entities_boundary)
 
-from mpi4py import MPI
-
 
 @pytest.fixture
 def cube():
@@ -49,10 +49,10 @@ def test_normals(cube, square):
         return np.isclose(x[0], 0)
     fdim = cube.topology.dim - 1
     facets = locate_entities_boundary(cube, fdim, left_side)
-    normals = cell_normals(cube, fdim, facets)
+    normals = cell_normals(cube._cpp_object, fdim, facets)
     assert np.allclose(normals, [-1, 0, 0])
 
     fdim = square.topology.dim - 1
     facets = locate_entities_boundary(square, fdim, left_side)
-    normals = cell_normals(square, fdim, facets)
+    normals = cell_normals(square._cpp_object, fdim, facets)
     assert np.allclose(normals, [-1, 0, 0])
