@@ -8,12 +8,15 @@
 import random
 from pathlib import Path
 
+from mpi4py import MPI
+
 import numpy as np
 import pytest
 
 import basix
 import ufl
-from basix.ufl_wrapper import create_vector_element
+from basix.ufl import element
+from dolfinx import default_real_type
 from dolfinx.cpp.io import perm_vtk
 from dolfinx.fem import assemble_scalar, form
 from dolfinx.io import XDMFFile
@@ -21,8 +24,6 @@ from dolfinx.io.gmshio import cell_perm_array, ufl_mesh
 from dolfinx.mesh import CellType, create_mesh, create_submesh
 from ufl import dx
 
-from mpi4py import MPI
-
 
 def check_cell_volume(points, cell, domain, volume):
     random.seed(13)
@@ -37,6 +38,7 @@ def check_cell_volume(points, cell, domain, volume):
             ordered_points[j] = points[i]
         ordered_cell = [point_order[i] for i in cell]
 
+        ordered_points = np.array(ordered_points, dtype=default_real_type)
         mesh = create_mesh(MPI.COMM_WORLD, [ordered_cell], ordered_points, domain)
         area = assemble_scalar(form(1 * dx(mesh)))
         assert np.isclose(area, volume)
@@ -94,9 +96,10 @@ def test_submesh(order):
                 for i in range(1, order - j - k):
                     cell.append(coord_to_vertex(i, j, k))
 
-    domain = ufl.Mesh(create_vector_element(
-        "Lagrange", "tetrahedron", order, gdim=3, lagrange_variant=basix.LagrangeVariant.equispaced))
+    domain = ufl.Mesh(element("Lagrange", "tetrahedron", order, gdim=3,
+                      lagrange_variant=basix.LagrangeVariant.equispaced, shape=(3, )))
 
+    points = np.array(points, dtype=default_real_type)
     mesh = create_mesh(MPI.COMM_WORLD, [cell], points, domain)
     for i in range(mesh.topology.dim):
         mesh.topology.create_entities(i)
@@ -142,8 +145,8 @@ def test_triangle_mesh(order):
             for i in range(1, order - j):
                 cell.append(coord_to_vertex(i, j))
 
-    domain = ufl.Mesh(create_vector_element(
-        "Lagrange", "triangle", order, gdim=2, lagrange_variant=basix.LagrangeVariant.equispaced))
+    domain = ufl.Mesh(element(
+        "Lagrange", "triangle", order, gdim=2, lagrange_variant=basix.LagrangeVariant.equispaced, shape=(2, )))
 
     check_cell_volume(points, cell, domain, 0.5)
 
@@ -199,8 +202,8 @@ def test_tetrahedron_mesh(order):
                 for i in range(1, order - j - k):
                     cell.append(coord_to_vertex(i, j, k))
 
-    domain = ufl.Mesh(create_vector_element(
-        "Lagrange", "tetrahedron", order, gdim=3, lagrange_variant=basix.LagrangeVariant.equispaced))
+    domain = ufl.Mesh(element(
+        "Lagrange", "tetrahedron", order, gdim=3, lagrange_variant=basix.LagrangeVariant.equispaced, shape=(3, )))
     check_cell_volume(points, cell, domain, 1 / 6)
 
 
@@ -234,8 +237,8 @@ def test_quadrilateral_mesh(order):
             for i in range(1, order):
                 cell.append(coord_to_vertex(i, j))
 
-    domain = ufl.Mesh(create_vector_element(
-        "Q", "quadrilateral", order, gdim=2, lagrange_variant=basix.LagrangeVariant.equispaced))
+    domain = ufl.Mesh(element("Q", "quadrilateral", order, gdim=2,
+                      lagrange_variant=basix.LagrangeVariant.equispaced, shape=(2, )))
     check_cell_volume(points, cell, domain, 1)
 
 
@@ -310,8 +313,8 @@ def test_hexahedron_mesh(order):
                 for i in range(1, order):
                     cell.append(coord_to_vertex(i, j, k))
 
-    domain = ufl.Mesh(create_vector_element(
-        "Q", "hexahedron", order, gdim=3, lagrange_variant=basix.LagrangeVariant.equispaced))
+    domain = ufl.Mesh(element("Q", "hexahedron", order, gdim=3,
+                      lagrange_variant=basix.LagrangeVariant.equispaced, shape=(3, )))
     check_cell_volume(points, cell, domain, 1)
 
 
@@ -348,8 +351,8 @@ def test_triangle_mesh_vtk(order):
             raise NotImplementedError
 
     cell = np.array(cell)[perm_vtk(CellType.triangle, len(cell))]
-    domain = ufl.Mesh(create_vector_element(
-        "Lagrange", "triangle", order, gdim=2, lagrange_variant=basix.LagrangeVariant.equispaced))
+    domain = ufl.Mesh(element("Lagrange", "triangle", order, gdim=2,
+                      lagrange_variant=basix.LagrangeVariant.equispaced, shape=(2, )))
     check_cell_volume(points, cell, domain, 0.5)
 
 
@@ -440,8 +443,8 @@ def test_tetrahedron_mesh_vtk(order):
                         cell.append(coord_to_vertex(i, j, k))
 
     cell = np.array(cell)[perm_vtk(CellType.tetrahedron, len(cell))]
-    domain = ufl.Mesh(create_vector_element(
-        "Lagrange", "tetrahedron", order, gdim=3, lagrange_variant=basix.LagrangeVariant.equispaced))
+    domain = ufl.Mesh(element(
+        "Lagrange", "tetrahedron", order, gdim=3, lagrange_variant=basix.LagrangeVariant.equispaced, shape=(3, )))
     check_cell_volume(points, cell, domain, 1 / 6)
 
 
@@ -478,8 +481,8 @@ def test_quadrilateral_mesh_vtk(order):
                 cell.append(coord_to_vertex(i, j))
 
     cell = np.array(cell)[perm_vtk(CellType.quadrilateral, len(cell))]
-    domain = ufl.Mesh(create_vector_element(
-        "Q", "quadrilateral", order, gdim=2, lagrange_variant=basix.LagrangeVariant.equispaced))
+    domain = ufl.Mesh(element("Q", "quadrilateral", order, gdim=2,
+                      lagrange_variant=basix.LagrangeVariant.equispaced, shape=(2, )))
     check_cell_volume(points, cell, domain, 1)
 
 
@@ -561,8 +564,8 @@ def test_hexahedron_mesh_vtk(order):
                     cell.append(coord_to_vertex(i, j, k))
 
     cell = np.array(cell)[perm_vtk(CellType.hexahedron, len(cell))]
-    domain = ufl.Mesh(create_vector_element(
-        "Q", "hexahedron", order, gdim=3, lagrange_variant=basix.LagrangeVariant.equispaced))
+    domain = ufl.Mesh(element(
+        "Q", "hexahedron", order, gdim=3, lagrange_variant=basix.LagrangeVariant.equispaced, shape=(3, )))
     check_cell_volume(points, cell, domain, 1)
 
 
@@ -582,6 +585,7 @@ def test_map_vtk_to_dolfin(vtk, dolfin, cell_type):
     assert (cell_p == vtk).all()
 
 
+@pytest.mark.skipif(default_real_type != np.float64, reason="float32 not supported yet")
 @pytest.mark.skip_in_parallel
 def test_xdmf_input_tri(datadir):
     with XDMFFile(MPI.COMM_WORLD, Path(datadir, "mesh.xdmf"), "r", encoding=XDMFFile.Encoding.ASCII) as xdmf:
@@ -604,7 +608,9 @@ def test_gmsh_input_2d(order, cell_type):
     gmsh.option.setNumber("Mesh.CharacteristicLengthMax", res)
 
     if cell_type == CellType.quadrilateral:
-        gmsh.option.setNumber("Mesh.Algorithm", 2 if order == 2 else 5)
+        gmsh.option.setNumber("Mesh.Algorithm", 2)
+        # Force mesh to have no triangles
+        gmsh.option.setNumber("Mesh.RecombinationAlgorithm", 3)
 
     gmsh.model.occ.addSphere(0, 0, 0, 1, tag=1)
     gmsh.model.occ.synchronize()
@@ -632,6 +638,7 @@ def test_gmsh_input_2d(order, cell_type):
     gmsh.finalize()
 
     cells = cells[:, cell_perm_array(cell_type, cells.shape[1])]
+    x = x.astype(default_real_type)
     mesh = create_mesh(MPI.COMM_WORLD, cells, x, ufl_mesh(gmsh_cell_id, x.shape[1]))
     surface = assemble_scalar(form(1 * dx(mesh)))
 
@@ -691,6 +698,7 @@ def test_gmsh_input_3d(order, cell_type):
     domain = ufl_mesh(gmsh_cell_id, 3)
     cells = cells[:, cell_perm_array(cell_type, cells.shape[1])]
 
+    x = x.astype(default_real_type)
     mesh = create_mesh(MPI.COMM_WORLD, cells, x, domain)
     volume = assemble_scalar(form(1 * dx(mesh)))
     assert mesh.comm.allreduce(volume, op=MPI.SUM) == pytest.approx(np.pi, rel=10 ** (-1 - order))
@@ -709,6 +717,6 @@ def test_quadrilateral_cell_order_3():
     ]
 
     cell = list(range(16))
-    domain = ufl.Mesh(create_vector_element(
-        "Q", "quadrilateral", 3, gdim=2, lagrange_variant=basix.LagrangeVariant.equispaced))
+    domain = ufl.Mesh(element(
+        "Q", "quadrilateral", 3, gdim=2, lagrange_variant=basix.LagrangeVariant.equispaced, shape=(2, )))
     check_cell_volume(points, cell, domain, 5 / 6)
