@@ -5,21 +5,22 @@
 # SPDX-License-Identifier:    LGPL-3.0-or-later
 """Unit tests for Newton solver assembly"""
 
+from mpi4py import MPI
+from petsc4py import PETSc
+
 import numpy as np
 
 import ufl
 from dolfinx import cpp as _cpp
-from dolfinx import la
+from dolfinx import default_real_type
 from dolfinx.fem import (Function, FunctionSpace, dirichletbc, form,
                          locate_dofs_geometrical)
 from dolfinx.fem.petsc import (apply_lifting, assemble_matrix, assemble_vector,
                                create_matrix, create_vector, set_bc)
+from dolfinx.la import create_petsc_vector
 from dolfinx.mesh import create_unit_square
 from ufl import TestFunction, TrialFunction, derivative, dx, grad, inner
 
-from mpi4py import MPI
-from petsc4py import PETSc
-
 
 class NonlinearPDEProblem:
     """Nonlinear problem class for a PDE problem."""
@@ -107,6 +108,8 @@ def test_linear_pde():
     solver.setF(problem.F, problem.vector())
     solver.setJ(problem.J, problem.matrix())
     solver.set_form(problem.form)
+    solver.atol = 1.0e-8
+    solver.rtol = 1.0e2 * np.finfo(default_real_type).eps
     n, converged = solver.solve(u.vector)
     assert converged
     assert n == 1
@@ -120,7 +123,6 @@ def test_linear_pde():
 
 def test_nonlinear_pde():
     """Test Newton solver for a simple nonlinear PDE"""
-    # Create mesh and function space
     mesh = create_unit_square(MPI.COMM_WORLD, 12, 5)
     V = FunctionSpace(mesh, ("Lagrange", 1))
     u = Function(V)
@@ -141,6 +143,8 @@ def test_nonlinear_pde():
     solver.setF(problem.F, problem.vector())
     solver.setJ(problem.J, problem.matrix())
     solver.set_form(problem.form)
+    solver.atol = 1.0e-8
+    solver.rtol = 1.0e2 * np.finfo(default_real_type).eps
     n, converged = solver.solve(u.vector)
     assert converged
     assert n < 6
@@ -154,13 +158,11 @@ def test_nonlinear_pde():
 
 def test_nonlinear_pde_snes():
     """Test Newton solver for a simple nonlinear PDE"""
-    # Create mesh and function space
     mesh = create_unit_square(MPI.COMM_WORLD, 12, 15)
     V = FunctionSpace(mesh, ("Lagrange", 1))
     u = Function(V)
     v = TestFunction(V)
-    F = inner(5.0, v) * dx - ufl.sqrt(u * u) * inner(
-        grad(u), grad(v)) * dx - inner(u, v) * dx
+    F = inner(5.0, v) * dx - ufl.sqrt(u * u) * inner(grad(u), grad(v)) * dx - inner(u, v) * dx
 
     u_bc = Function(V)
     u_bc.x.array[:] = 1.0
@@ -171,7 +173,7 @@ def test_nonlinear_pde_snes():
     problem = NonlinearPDE_SNESProblem(F, u, bc)
 
     u.x.array[:] = 0.9
-    b = la.create_petsc_vector(V.dofmap.index_map, V.dofmap.index_map_bs)
+    b = create_petsc_vector(V.dofmap.index_map, V.dofmap.index_map_bs)
     J = create_matrix(problem.a)
 
     # Create Newton solver and solve
@@ -195,3 +197,7 @@ def test_nonlinear_pde_snes():
     assert snes.getIterationNumber() < 6
     # print(snes.getIterationNumber())
     # print(snes.getFunctionNorm())
+
+    snes.destroy()
+    b.destroy()
+    J.destroy()
